package com.bria.common.controller.calllog

import com.bria.common.controller.Controllers
import com.bria.common.controller.calllog.db.CallLogEntity
import com.bria.common.controller.contacts.local.Contact
import com.bria.common.controller.settings.ESetting
import com.bria.common.modules.BriaGraph
import com.bria.common.util.SipAddressUtils
import com.bria.common.util.Utils
import com.bria.common.util.Validator
import io.reactivex.schedulers.Schedulers

object CallLogUtils {
    private val isLuckyMobileBrand: Boolean by lazy { Utils.Brands.isLuckyMobileBrand(BriaGraph.application) }
    internal val isMetaswitchBrand: Boolean by lazy { Utils.Brands.isMetaswitchBrand(BriaGraph.application) }

    fun compareNumbers(number1: String, number2: String) = SipAddressUtils.compare(number1, number2, userOnly = isLuckyMobileBrand)
}

@JvmOverloads
fun CallLogEntity.getContactDisplayName(defaultDisplayName: String = getDefaultDisplayName()) =
        getAssociatedContact().subscribeOn(Schedulers.io()).map(Contact::getDisplayName).blockingGet(defaultDisplayName).orEmpty()

fun CallLogEntity.getDefaultDisplayName() = if (remoteName.isNotEmpty()) remoteName else getFormattedNumber()

fun CallLogEntity.getAssociatedContact() = Controllers.get().contacts.contactsApi.contact.byNumber(getNumberWithDomain()).asMaybe()

fun CallLogEntity.getNumberForContacts(): String {
    val num = SipAddressUtils.removeDomain(number) // domain can still be exist in DB (old call log entries)
    return if (Controllers.get().settings.getBool(ESetting.RemoveFullyQualifiedNameFromContacts) || Validator.isNumeric(num)) {
        num
    } else {
        getNumberWithDomain()
    }
}

fun CallLogEntity.getFormattedNumber(): String {
    return if (Controllers.get().settings.getBool(ESetting.ShowUriDomain)) {
        getNumberForContacts()
    } else {
        // domain can still be exist in DB (old call log entries)
        Controllers.get().phone.getFormattedNumber(SipAddressUtils.removeDomain(number), false)
    }.let {
        if (CallLogUtils.isMetaswitchBrand) Validator.getMetaswitchFormattedNumber(it) else it
    }
}

fun CallLogEntity.getNumberWithDomain() = SipAddressUtils.getAddress(number, getDomain(), emptyIfNoDomain = false)

fun CallLogEntity.getDomain() = SipAddressUtils.removePort(remoteDomain.takeIf { it.isNotEmpty() } ?: accountDomain)

fun CallLogEntity.compareNumber(other: CallLogEntity) = compareNumber(other.getNumberWithDomain())

fun CallLogEntity.compareNumber(otherNumber: String) = CallLogUtils.compareNumbers(getNumberWithDomain(), otherNumber)

