package com.bria.common.controller.calllog.db

import android.arch.persistence.room.*
import io.reactivex.Flowable
import io.reactivex.Maybe
import io.reactivex.Single

/**
 * Note, all Rx Flowable operations of this DAO work on IO thread because of Room design
 */
@Dao
interface CallLogDao {
    @Insert(onConflict = OnConflictStrategy.REPLACE)
    fun put(vararg entities: CallLogEntity): List<Long>

    @Query("SELECT * FROM calllog WHERE id IS :id AND (:user IS NULL OR provisioningUser IS :user)")
    fun get(id: Long, user: String?): Maybe<CallLogEntity?>

    @Query("SELECT * FROM calllog WHERE (callType IS :callType AND provisioningUser IS :user) ORDER BY date DESC LIMIT 1")
    fun getLast(callType: CallType, user: String?): Maybe<CallLogEntity?>

    @Query("SELECT * FROM calllog WHERE (:user IS NULL OR provisioningUser IS :user) ORDER BY date DESC")
    fun getAll(user: String?): Flowable<List<CallLogEntity>>

    @Delete
    fun delete(vararg entities: CallLogEntity)

    @Query("DELETE FROM calllog WHERE callType IS :callType AND (:user IS NULL OR provisioningUser IS :user)")
    fun delete(callType: CallType, user: String?)

    @Query("DELETE FROM calllog WHERE :user IS NULL OR provisioningUser IS :user")
    fun deleteAll(user: String?)

    @Query("UPDATE calllog SET unread = 0 WHERE unread = 1 AND (:user IS NULL OR provisioningUser IS :user)")
    fun markAllRead(user: String?)

    @Query("SELECT COUNT(id) FROM calllog WHERE unread = 1 AND callType = 'MISSED' AND (:user IS NULL OR provisioningUser IS :user)")
    fun getCountOfUnreadMissedCalls(user: String?): Flowable<Int>

    @Query("SELECT COUNT(id) FROM calllog WHERE :user IS NULL OR provisioningUser IS :user")
    fun getCount(user: String?): Single<Int>
}
