package com.bria.common.controller.contacts.buddy.v2

import com.bria.common.controller.buddy.BuddyAvailability
import com.bria.common.controller.contacts.buddy.Buddy
import com.bria.common.controller.contacts.buddy.IBuddyCtrlEvents
import com.bria.common.controller.contacts.buddy.IBuddyCtrlObserver
import com.bria.common.controller.contacts.buddy.IExtendedBuddyCtrlObserver
import com.bria.common.modules.BriaGraph
import com.bria.common.uireusable.dataprovider.AbstractFilterableListDataProvider
import com.bria.common.uireusable.dataprovider.FilterQueryMatcher
import com.bria.common.uireusable.dataprovider.IContactActionDataProvider
import com.bria.common.uireusable.dataprovider.IExtendedFilterableDataProvider
import com.bria.common.uireusable.datatypes.IPersonListItem
import com.bria.common.util.IObservable

class V2BuddyDataProvider(val iBuddyCtrlEvents: IBuddyCtrlEvents, val iBuddyObservable: IObservable<IBuddyCtrlObserver>) : AbstractFilterableListDataProvider<IPersonListItem>(), IExtendedBuddyCtrlObserver, IContactActionDataProvider<IPersonListItem> {

    var buddyFilterType: BuddyAvailability = BuddyAvailability.All

    override fun initializeLoadingData() {
        initialize()
    }


    private lateinit var iBuddyDataProviderListener: IExtendedFilterableDataProvider

    init {
        iBuddyObservable.attachWeakObserver(this)
    }

    override fun provideFullDataSet(): List<IPersonListItem> {

        return iBuddyCtrlEvents.getListOfBuddiesSorted(Buddy.EBuddyType.ALL, buddyFilterType)
                .filter { x -> !BriaGraph.teams.isBuddyFromTeam(x) }
    }

    override fun provideFilteredDataSet(fullDataSet: List<IPersonListItem>, query: String): List<IPersonListItem> {
        val filterQueryMatcher = FilterQueryMatcher(query)

        return fullDataSet
                .filter { filterQueryMatcher.isMatch(it.firstName, it.lastName) }
                .toMutableList()
    }


    override fun removeActiveItem(position: Int) {
        if (itemsCount >= position) {
            val buddyListItem = getItemAt(position) as Buddy
            if (buddyListItem.buddy != null) {
                val accountId = buddyListItem.accountNickname
                val address = buddyListItem.buddy.imAddress
                iBuddyCtrlEvents.removeBuddy(accountId, address, true)
            }
        }
        //iBuddyDataProviderListener.removedItem(0)
    }

    override fun onBuddyRemoved(buddy: Buddy) {
        TODO("not implemented") //To change body of created functions use File | Settings | File Templates.
    }

    override fun onBuddyListUpdated() {
        refilter()

    }

    override fun onBuddyPresenceChanged(buddy: Buddy?) {

        mDisplayedData.firstOrNull { t ->
            t.uniqueIdentifier == buddy?.uniqueIdentifier ?: false
        }?.let { iBuddyDataProviderListener.updatedItem(mDisplayedData.indexOf(it)) }

    }

    override fun onBuddyFilterChanged(aBuddyFilterType: IBuddyCtrlEvents.EBuddyFilterType?) {
    }


    override fun attachWeakListenerV2(iExtendedFilterableDataProvider: IExtendedFilterableDataProvider) {
        attachWeakListener(iExtendedFilterableDataProvider)
        this.iBuddyDataProviderListener = iExtendedFilterableDataProvider
    }

    override fun filterDataProvider(query: CharSequence) {
        filterData(query)
    }

    override fun filterDataProvider(buddyAvailability: BuddyAvailability) {
        buddyFilterType = buddyAvailability
        refilter()

    }


}
