package com.bria.common.controller.contacts.ldap

import com.bria.common.controller.buddy.BuddyAvailability
import com.bria.common.uireusable.dataprovider.AbstractFilterableListDataProvider
import com.bria.common.uireusable.dataprovider.FilterQueryMatcher
import com.bria.common.uireusable.dataprovider.IContactActionDataProvider
import com.bria.common.uireusable.dataprovider.IExtendedFilterableDataProvider
import com.bria.common.uireusable.datatypes.IPersonListItem

class LdapDataProviderV2(val ldapModule: LdapModule?) : AbstractFilterableListDataProvider<IPersonListItem>(), IContactActionDataProvider<IPersonListItem>, ILdapModuleObserver {

    init {
        ldapModule?.attachWeakObserver(this)
    }

    override fun onDirectoryListUpdated() {
        refilter()
    }

    override fun attachWeakListenerV2(iExtendedFilterableDataProvider: IExtendedFilterableDataProvider) {
        attachWeakListener(iExtendedFilterableDataProvider)
    }

    override fun initializeLoadingData() {
        initialize()
    }

    override fun filterDataProvider(query: CharSequence) {
        ldapModule?.setDirectorySearchString(query.toString())
    }

    override fun filterDataProvider(subFilter: BuddyAvailability?) {
    }

    override fun removeActiveItem(position: Int?) {
    }

    override fun provideFullDataSet(): List<IPersonListItem> {
        return ldapModule!!.getLdapList()
    }

    override fun provideFilteredDataSet(fullDataSet: List<IPersonListItem>, query: String): List<IPersonListItem> {
        val filterQueryMatcher = FilterQueryMatcher(query)

        return fullDataSet
                .filter { filterQueryMatcher.isMatch(it.firstName, it.lastName) }
                .toMutableList()
    }

    override fun isDataLoading(): Boolean = ldapModule!!.isDataLoading()


}
