package com.bria.common.controller.contacts.local.favorites

import android.text.TextUtils
import com.bria.common.controller.buddy.BuddyAvailability
import com.bria.common.controller.contacts.buddy.Buddy
import com.bria.common.controller.contacts.buddy.IBuddyCtrlEvents
import com.bria.common.controller.contacts.buddy.IBuddyCtrlObserver
import com.bria.common.uireusable.dataprovider.AbstractFilterableListDataProvider
import com.bria.common.uireusable.dataprovider.IContactActionDataProvider
import com.bria.common.uireusable.dataprovider.IExtendedFilterableDataProvider
import com.bria.common.uireusable.datatypes.IPersonListItem
import com.bria.common.util.IObservable
import java.util.*

class V2FavoriteDataProvider(private val mFavoritesApi: FavoritesApi, iBuddyObservable: IObservable<IBuddyCtrlObserver>) : AbstractFilterableListDataProvider<IPersonListItem>(), FavoritesApiListener, IContactActionDataProvider<IPersonListItem>, IBuddyCtrlObserver {
    companion object {
        private val TAG = "FavoriteDataProviderV2"
    }

    private lateinit var iFavorireDataProviderListener: IExtendedFilterableDataProvider


    init {
        mFavoritesApi.addListener(this)
        iBuddyObservable.attachWeakObserver(this)
    }


    override fun initializeLoadingData() {
        initialize()
    }

    override fun provideFullDataSet(): List<IPersonListItem> {
        return ArrayList<IPersonListItem>(mFavoritesApi.listOfAllFavorite)

    }

    override fun provideFilteredDataSet(fullDataSet: List<IPersonListItem>, query: String): List<IPersonListItem> {
        return ArrayList<IPersonListItem>(mFavoritesApi.getListOfAllFavorite(query))
    }

    override fun filterDataProvider(query: CharSequence) {
        filterData(query)
    }

    override fun onFavoriteUpdated(favorite: FavoriteDataItem) {
        refilter()

    }

    override fun onFavoriteAdded(favorite: FavoriteDataItem) {
        refilter()
    }

    override fun onFavoriteRemoved(favorite: FavoriteDataItem) {
        refilter()
    }

    override fun onFavoritesCleared() {
        refilter()

    }

    override fun removeActiveItem(position: Int?) {
        //handle from favorite tab
        var favoriteDataItem: FavoriteDataItem?
        favoriteDataItem = getItemAt(position!!) as FavoriteDataItem

        if (favoriteDataItem == null) {
            //handle from contact tab
            val contactId = getItemAt(position).uniqueIdentifier
            val intId = if (TextUtils.isEmpty(contactId)) -1 else Integer.valueOf(contactId)
            favoriteDataItem = mFavoritesApi.getFavorite(intId!!)
        }
        mFavoritesApi.removeFavorite(favoriteDataItem!!)
    }

    override fun attachWeakListenerV2(iExtendedFilterableDataProvider: IExtendedFilterableDataProvider) {
        attachWeakListener(iExtendedFilterableDataProvider)
        this.iFavorireDataProviderListener = iExtendedFilterableDataProvider
    }

    override fun onBuddyListUpdated() {
        refilter()
    }

    override fun onBuddyPresenceChanged(buddy: Buddy?) {
        mDisplayedData.firstOrNull { t ->
            t.uniqueIdentifier == buddy?.uniqueIdentifier ?: false
        }?.let { iFavorireDataProviderListener.updatedItem(mDisplayedData.indexOf(it)) }
    }

    override fun onBuddyFilterChanged(aBuddyFilterType: IBuddyCtrlEvents.EBuddyFilterType?) {
    }

    override fun filterDataProvider(subFilter: BuddyAvailability) {

    }


}
