package com.bria.common.controller.im.refactoring.adapter

import android.content.Context
import android.graphics.Bitmap
import android.graphics.drawable.BitmapDrawable
import android.support.v7.util.DiffUtil
import android.support.v7.util.ListUpdateCallback
import android.support.v7.widget.RecyclerView
import android.text.TextUtils
import android.text.method.LinkMovementMethod
import android.view.LayoutInflater
import android.view.View
import android.view.ViewGroup
import android.widget.ImageView
import android.widget.TextView
import com.bria.common.R
import com.bria.common.controller.ClientConfig
import com.bria.common.controller.Controllers
import com.bria.common.controller.collaboration.utils.matcher.MatcherKey
import com.bria.common.controller.collaboration.utils.matcher.OnParticipantMatchedListener
import com.bria.common.controller.collaboration.utils.matcher.ParticipantMatch
import com.bria.common.controller.collaboration.utils.matcher.ParticipantMatcher
import com.bria.common.controller.contacts.buddy.Buddy
import com.bria.common.controller.contacts.buddy.XmppBuddy
import com.bria.common.controller.im.ImpsUtils
import com.bria.common.controller.im.refactoring.dataprovider.FilterableInstantMessageDataProvider
import com.bria.common.controller.im.refactoring.dataprovider.InstantMessageDiffUtils
import com.bria.common.controller.settings.ESetting
import com.bria.common.ui.util.SearchQueryColorizer
import com.bria.common.uiframework.coloring.Coloring
import com.bria.common.uiframework.lists.recycler.IOnRecyclerViewItemEventsListener
import com.bria.common.uireusable.datatypes.MessageListItemData
import com.bria.common.util.AndroidUtils
import com.bria.common.util.LinkifyUtils
import com.bria.common.util.Log
import io.reactivex.Observer
import io.reactivex.android.schedulers.AndroidSchedulers
import io.reactivex.disposables.CompositeDisposable
import io.reactivex.disposables.Disposable
import io.reactivex.schedulers.Schedulers
import java.util.concurrent.TimeUnit

class FilterableInstantMessageListAdapter(private val context: Context,
                                          private var matcher: ParticipantMatcher,
                                          private val eventsListener: IOnRecyclerViewItemEventsListener?,
                                          internal var provider: FilterableInstantMessageDataProvider,
                                          private val imListAdapterMentionsHelper: ImListAdapterMentionsHelper,
                                          private val callbacks : ICallbacks)
    : RecyclerView.Adapter<FilterableInstantMessageListAdapter.InstantMessageViewHolder>(), OnParticipantMatchedListener {

    private var mDefaultAvatar: Bitmap? = null

    private val disposable = CompositeDisposable()

    private var list = ArrayList<MessageListItemData>()

    private val mContactsSearchQueryColorizer = SearchQueryColorizer(Coloring.get().decodeColor(Controllers.get().settings.getStr(ESetting.ColorBrandTint)))

    private val lock = Any()

    var selfInfo : Buddy? = null

    var previousFilter = ""

    init {
        matcher.setListener(this)
        subscribeOnProvide()
    }

    private fun subscribeOnProvide() {
        provider.source.debounce(50, TimeUnit.MILLISECONDS).toObservable()
                .subscribeOn(Schedulers.computation())
                .map { Pair<ArrayList<MessageListItemData>, DiffUtil.DiffResult>(it, DiffUtil.calculateDiff(InstantMessageDiffUtils(list, it))) }
                .observeOn(AndroidSchedulers.mainThread()).subscribe(object : Observer<Pair<List<MessageListItemData>, DiffUtil.DiffResult>> {
                    override fun onSubscribe(d: Disposable) {
                        disposable.add(d)
                    }

                    override fun onNext(pair: Pair<List<MessageListItemData>, DiffUtil.DiffResult>) {
                        synchronized(lock) {
                            //Log.d(TAG, list.joinToString(separator = "\n") { it.message })
                            list.clear()
                            list.addAll(pair.first)
                            if (previousFilter != getFilter().toString()) { //not updating properly if filter is changed
                                previousFilter = getFilter().toString()
                                notifyDataSetChanged()
                            } else {
                                pair.second.dispatchUpdatesTo(updateCallBack)
                            }
                            callbacks.listUpdated()
                        }
                    }

                    override fun onError(e: Throwable) {
                        Log.e(TAG, Log.getCallerStackStr(30))
                        Log.e(TAG, """message ${e.message} ; onError: ${e.printStackTrace()}""")
                    }

                    override fun onComplete() {
                        Log.d(TAG, "onComplete:")
                    }
                })
    }


    val updateCallBack: ListUpdateCallback = object : ListUpdateCallback {
        override fun onChanged(position: Int, count: Int, payload: Any?) {
            Log.d(TAG, "notifyItemRangeChanged(pos:$position, count $count, payload $payload)")
            this@FilterableInstantMessageListAdapter.notifyItemRangeChanged(position, count)
        }

        override fun onMoved(fromPosition: Int, toPosition: Int) {
            Log.d(TAG, "notifyItemMoved(fromPosition:$fromPosition, toPosition $toPosition)")
            this@FilterableInstantMessageListAdapter.notifyItemMoved(fromPosition, toPosition)
        }

        override fun onInserted(position: Int, count: Int) {
            Log.d(TAG, "notifyItemRangeInserted(pos:$position, count $count)")
            this@FilterableInstantMessageListAdapter.notifyItemRangeInserted(position, count)
        }

        override fun onRemoved(position: Int, count: Int) {
            Log.d(TAG, "notifyItemRangeRemoved(pos:$position, count $count)")
            this@FilterableInstantMessageListAdapter.notifyItemRangeRemoved(position, count)
        }

    }

    override fun onCreateViewHolder(parent: ViewGroup, viewType: Int): FilterableInstantMessageListAdapter.InstantMessageViewHolder {

        val v = LayoutInflater.from(parent.context).inflate(R.layout.im_message_item_search, parent, false)
        val vh = InstantMessageViewHolder(v)
        return vh
    }

    override fun onBindViewHolder(holder: InstantMessageViewHolder, position: Int) {
        val item = getItemAt(position)
        updateView(holder, item)
        linkify(holder.message)
    }

    fun filterData(filter : String) {
        provider.filterData(filter)
    }

    fun getFilter() : CharSequence {
        return provider.filter
    }

    private fun updateView(viewHolder: InstantMessageViewHolder, item: MessageListItemData) {
        var messageText = item.message
        viewHolder.message.text = imListAdapterMentionsHelper.processMentions(
                messageText,
                context.resources.getColor(R.color.link_color),
                provider.isChatRoom,
                callbacks::displayBuddy)
        val timeString = context.getString(R.string.tDateAtTime, item.dateString, item.timeString)
        viewHolder.time.text = timeString

        viewHolder.participantName.visibility = View.VISIBLE
        viewHolder.avatar.visibility = View.VISIBLE

        viewHolder.avatar.layoutParams?.height = viewHolder.avatar.layoutParams?.width

        val key = MatcherKey()
        key.xmppJid = item.jid

        val isSip = provider.extendedConversationData?.isSipType() == true
        if (isSip) key.sipNumber = ImpsUtils.removeDomainFromAddress(item.jid)

        val details = matcher.getMatchedInfo(key)
        if (item.isOutgoing()) {
            if (selfInfo != null) {
                viewHolder.setAvatar(selfInfo!!.avatarIcon)
                viewHolder.participantName.text = selfInfo!!.displayName
            } else {
                viewHolder.participantName.text = "Me"
            }
        } else {
            var name: String?
            if (details != null) {
                name = details.displayName
                if (name == item.jid && !TextUtils.isEmpty(item.displayName)) {
                    name = item.displayName
                }
                viewHolder.setAvatar(details.photo)
            } else {
                name = item.participantName
                viewHolder.setAvatar(item.avatar)
                matcher.loadParticipantInfo(key, item.jid)
            }
            name = ImpsUtils.removeDomainFromAddress(name)
            viewHolder.participantName.text = name
        }
        viewHolder.itemView.setOnClickListener {
            callbacks.goToMessage(item.id)
        }
        mContactsSearchQueryColorizer.colorize(viewHolder.message, getFilter().toString())
    }

    private fun linkify(target: TextView) {
        LinkifyUtils.addLinks(target, ImListAdapterMentionsHelper.spanTypesUsed)
    }

    override fun getItemCount(): Int {
        synchronized(lock) {
            return list.size
        }
    }

    fun getItemAt(position: Int): MessageListItemData {
        synchronized(lock) {
            return list[position]
        }
    }

    override fun getItemViewType(position: Int): Int = getItemAt(position).msgType

    override fun getItemId(position: Int): Long = getItemAt(position).id

    private val circleDefaultAvatar: Bitmap?
        get() {
            if (mDefaultAvatar == null) {
                val photo = AndroidUtils.getDrawable(context, R.drawable.default_avatar)
                mDefaultAvatar = Controllers.get().image.cropToCircle((photo as BitmapDrawable).bitmap)
            }
            return mDefaultAvatar
        }


    /* Participant matcher listener implementation goes below this line */
    override fun onParticipantMatched(matched: ParticipantMatch) {
        if (ClientConfig.get().isDebugMode) {
            Log.d(TAG, "Matched: " + matched.toString())
        }
        notifyDataSetChanged() //nvtd try better
    }

    fun cleanup() {
        disposable.clear()
        //super.cleanup()
    }

    companion object {
        private const val TAG = "FilterableInstantMessageListAdapter"

        const val BUBBLE_COLOR_BLUE = 0 // used by default
        const val BUBBLE_COLOR_GRAY = 1
    }

    fun loadMoreMessages() {
        provider.loadMoreMessages()
    }

    inner class InstantMessageViewHolder(itemView: View) : RecyclerView.ViewHolder(itemView) {

        var message: TextView = itemView.findViewById(R.id.im_message_item_message_text)

        init {
            message.movementMethod = LinkMovementMethod.getInstance()
        }

        var avatar: ImageView = itemView.findViewById(R.id.im_message_item_avatar)
        var participantName: TextView = itemView.findViewById(R.id.im_message_item_participant_name)
        var time: TextView = itemView.findViewById(R.id.im_message_item_message_time)
        fun setAvatar(newAvatar: Bitmap?) {
            avatar.setImageBitmap(newAvatar ?: circleDefaultAvatar)
        }
    }

    interface ICallbacks {
        fun displayBuddy(buddy: XmppBuddy)
        fun goToMessage(messageID : Long)
        fun listUpdated()
    }
}
