package com.bria.common.controller.im.refactoring.chatroom

import com.bria.common.controller.contacts.buddy.XmppBuddy
import com.bria.common.controller.im.refactoring.roomdb.LocalRoomInfo
import com.bria.common.controller.im.refactoring.roomdb.entities.ChatRoom
import com.bria.common.util.im.Participant
import com.counterpath.sdk.XmppAccount
import com.counterpath.sdk.handler.XmppMultiUserChatHandler
import io.reactivex.Observable
import io.reactivex.Single
import io.reactivex.subjects.Subject

interface ChatRoomApi {
    fun createRoom(chatRoom: ChatRoom)
    fun getChatHandler() : XmppMultiUserChatHandler
    fun isGroupChatCapable(buddyKey : String) : Boolean
    fun isServiceAvailable(userAtDomain: String) : Boolean
    fun isChatRoomActive(chatRoomKey: String) : Boolean
    fun sendMessage(chatRoom: ChatRoom, message: String)
    fun joinRooms(roomsJid: List<LocalRoomInfo>, account : XmppAccount)
    fun deleteRoom(chatRoom: ChatRoom)
    fun leaveRoom(chatRoom: ChatRoom)
    fun updateRoomDetails(chatRoom: ChatRoom)
    fun removeMember(chatRoom: ChatRoom, participant: Participant)
    fun clearCacheForAccount(account : XmppAccount)
    fun accountConnected(xmppAccount: XmppAccount)
    fun accountDisconnected(xmppAccount: XmppAccount)
    fun inviteMembers(chatRoom: ChatRoom)
    fun initCollabChat(roomId: String, username: String, password: String, account: XmppAccount): Single<Long>
    fun requestMembersList(chatRoom: ChatRoom) : Boolean
    val onErrorEventObservable: Observable<ChatRoomErrorEvent>
    fun getOnChatRoomStateObservable(): Subject<ChatRoomStateEvent>
    fun getRoomInfo(accId: String, chatKey :String)
    fun updateRoomList(downloadDetails: Boolean)

    fun updateJoinedRooms()
    fun reInviteToPrivateRoom(chatRoomId: Long, buddy: XmppBuddy)
    fun isServiceAvailableForAnyAccount(): Boolean
    fun getOnServiceAvailabilityObservable(): Subject<Set<String>>
    fun deleteCollabChatRoom(jid :String)
    fun forceOffline(chatKey: String)
}
