package com.bria.common.controller.im.refactoring.chatroom

object ChatRoomColorBuilder {
    const val TAG = "ChatRoomColorBuilder"
    private fun getHashCode(s: String): Int {
        var hash = 0L
        for (i in 0 until s.length) {
            val c = Character.codePointAt(s, i)
            hash = ((hash shl 5) - hash) + c
            hash = hash and hash
        }
        return hash.toInt()
    }

    private fun getIntInRangeFromHash(string: String, min: Int, max: Int): Double {
        val hash: Double = Math.abs(getHashCode(string)).toDouble() / 2147483647
        return (Math.floor((hash * (max - min + 1))) + min)
    }

    fun getColorFromString(s: String): Int {
        val luminance = getIntInRangeFromHash(s + "lumina", 60, 80)
        val saturation = getIntInRangeFromHash(s + "saturm", 30, 80)
        val hue = getIntInRangeFromHash(s + "hubris", 0, 360)
        return lch2Rgb(luminance, saturation, hue)
    }


    private fun lch2Rgb(l: Double, c: Double, h: Double): Int {
        val lab = lch2Lab(Triple(l, c, h))
        val rgb = lab2Rgb(lab)
        val redval = Math.round(rgb.first).toInt()
        val greenval = Math.round(rgb.second).toInt()
        val blueval = Math.round(rgb.third).toInt()
        return (0xff shl 24) or (redval shl 16) or (greenval shl 8) or blueval
    }

    private const val Xn = 0.950470
    private const val Yn = 1
    private const val Zn = 1.088830

    private const val t0 = 0.137931034
    private const val t1 = 0.206896552
    private const val t2 = 0.12841855

    private fun lch2Lab(lch: Triple<Double, Double, Double>): Triple<Double, Double, Double> {
        val hh = Math.toRadians(lch.third)
        return Triple(lch.first, Math.cos(hh) * lch.second, Math.sin(hh) * lch.second)
    }

    private fun lab2Rgb(lab: Triple<Double, Double, Double>): Triple<Double, Double, Double> {
        var y = (lab.first + 16f) / 116
        var x = if (lab.second == 0.0) y else y + lab.second / 500
        var z = if (lab.third == 0.0) y else y - lab.third / 200
        y = Yn * labXyz(y)
        x = Xn * labXyz(x)
        z = Zn * labXyz(z)
        var r = xyzRgb(3.2404542 * x - 1.5371385 * y - 0.4985314 * z)
        r = minOf(r, 255.0)
        r = maxOf(r, 0.0)
        var g = xyzRgb(-0.9692660 * x + 1.8760108 * y + 0.0415560 * z)
        g = minOf(g, 255.0)
        g = maxOf(g, 0.0)
        var b = xyzRgb(0.0556434 * x - 0.2040259 * y + 1.0572252 * z)
        b = minOf(b, 255.0)
        b = maxOf(b, 0.0)
        return Triple(r, g, b)
    }

    private fun xyzRgb(r: Double): Double = 255 * (if (r <= 0.00304) 12.92 * r else 1.055 * Math.pow(r, 1 / 2.4) - 0.055)
    private fun labXyz(t: Double): Double = if (t > t1) t * t * t else t2 * (t - t0)
}


