package com.bria.common.controller.im.refactoring.chatroom

import android.content.Context
import com.bria.common.R
import com.bria.common.controller.im.refactoring.roomdb.entities.ChatRoom
import com.bria.common.validation.ValidationFailure
import com.bria.common.validation.ValidationResult

/*
* Does not / SHOULD not look beyond the chat room object.
*
* TODO: maybe use https://github.com/konform-kt/konform
*
* */
object ChatRoomFieldValidator {
    private const val MAX_ROOM_NAME_SIZE = 40
    private const val MAX_ROOM_DESCRIPTION_SIZE = 140
    private const val MAX_ROOM_TOPIC_SIZE = 140
    private const val INVALID_CHARS = "\"&\'/:<>@"

    fun validate(chatRoom: ChatRoom, context: Context): ValidationResult<ChatRoom> {

        val failures = mutableListOf<ValidationFailure<ChatRoom>>()

        if (chatRoom.name.isBlank()) {
            failures.add(ValidationFailure(ChatRoom::name, context.getString(R.string.tChatRoomEmptyError)))
        }

        if (chatRoom.name.length > MAX_ROOM_NAME_SIZE) {
            failures.add(ValidationFailure(ChatRoom::name, context.getString(R.string.tChatRoomNameTooLong, MAX_ROOM_NAME_SIZE)))
        }

        //removed for now as it can be properly escaped
//        if (INVALID_CHARS.any { chatRoom.name.contains(it) }) {
        //  failures.add(ValidationFailure(ChatRoom::name, getString(R.string.tChatRoomNameInvalidCharacters))))
//        }

        if (chatRoom.description.length > MAX_ROOM_DESCRIPTION_SIZE) {
            failures.add(ValidationFailure(ChatRoom::description, context.getString(R.string.tChatRoomDescriptionTooLong, MAX_ROOM_DESCRIPTION_SIZE)))
        }

        if (chatRoom.topic.length > MAX_ROOM_TOPIC_SIZE) {
            failures.add(ValidationFailure(ChatRoom::topic, context.getString(R.string.tChatRoomTopicTooLong, MAX_ROOM_TOPIC_SIZE)))
        }


        return ValidationResult(failures)
    }
}
