package com.bria.common.controller.im.refactoring.chatroom

import com.bria.common.util.Optional
import io.reactivex.Observable
import io.reactivex.subjects.PublishSubject

data class ChatRoomMembers(var owners: Set<String> = setOf(), var members: Set<String> = setOf())

data class ChatRoomMemberCacheChange(val chatRoomId: Long, val chatRoomMembers: ChatRoomMembers)

class ChatRoomMemberCache {
    private val store = mutableMapOf<Long, ChatRoomMembers>() // key: chat room id

    private val publishSubject = PublishSubject.create<ChatRoomMemberCacheChange>()


    fun getAllMembers(chatRoomId: Long): Optional<ChatRoomMembers> {
        val result = store.get(chatRoomId)

        return Optional.fromNullable(result)
    }

    fun putMembers(chatRoomId: Long, memberBuddyKeys: Set<String>) {
        var members = store[chatRoomId]
        if (members == null) {
            members = ChatRoomMembers()
            store[chatRoomId] = members
        }

        members.members = memberBuddyKeys

        publishSubject.onNext(ChatRoomMemberCacheChange(chatRoomId, members))
    }

    fun putOwners(chatRoomId: Long, ownerBuddyKeys:Set<String>) {
        var members = store[chatRoomId]
        if (members == null) {
            members = ChatRoomMembers()
            store[chatRoomId] = members
        }

        members.owners = ownerBuddyKeys

        publishSubject.onNext(ChatRoomMemberCacheChange(chatRoomId, members))
    }

    val observable: Observable<ChatRoomMemberCacheChange>
        get () = publishSubject
}
