package com.bria.common.controller.im.refactoring.chatroom

object MentionsFinder {

    fun isMentionDelimiter(char: Char) : Boolean {
        return char.isWhitespace()
    }

    private fun Char.isDelimiter(): Boolean {
        return isMentionDelimiter(this)
    }

    fun findMentions(text: String?): List<Mention> {
        if (text.isNullOrBlank()) {
            return listOf()
        }

        val result = mutableListOf<Mention>()

        var startIndex: Int? = null

        for (i in text!!.indices) {
            val c = text[i]
            if (c == '@') {
                // nice it's an at sign

                if (i < text.length - 1) {
                    // next char exists. otherwise no match

                    val nextChar = text[i + 1]
                    if (!nextChar.isDelimiter()) {
                        // furthermore, it is a letter or something nice

                        // now, check the previous char
                        if (i == 0) {
                            // start of the string: ok
                            startIndex = i
                        } else {
                            val previousChar = text[i - 1]
                            if (previousChar.isDelimiter()) {
                                // nice, previous is not a letter
                                startIndex = i
                            }
                        }
                    }
                }
            } else {
                if (c.isDelimiter()) {
                    if (startIndex != null) {
                        val length = i - startIndex

                        result.add(Mention(startIndex, length))

                        startIndex = null
                    }
                }
            }
        }

        if (startIndex != null) {
            val length = text.length - startIndex
            result.add(Mention(startIndex, length))
        }

        return result
    }
}

data class Mention(val start: Int, val length: Int)
