package com.bria.common.controller.im.refactoring.chatroom.joinedroom

import com.bria.common.util.Log
import org.xml.sax.Attributes
import org.xml.sax.SAXException
import org.xml.sax.helpers.DefaultHandler
import java.util.*

class XMPPDataStorageXmlHandler : DefaultHandler() {

    companion object {
        const val ROOM_KEY = "room"
        const val TAG = "XMPPDataStorageXmlHandler"
    }

    val joinedRooms = mutableListOf<JoinedRoomData>()
    private val elementStack = Stack<String>() //As we read any XML element we will push that in this stack

    private val objectStack = Stack<Any>()

    @Throws(SAXException::class)
    override fun startElement(uri: String, localName: String, qName: String, attributes: Attributes) {
        //Push it in element stack
        this.elementStack.push(qName)
        if (ROOM_KEY == qName) {
            val joinedRoomData = JoinedRoomData()
            this.objectStack.push(joinedRoomData)
        }
    }

    @Throws(SAXException::class)
    override fun endElement(uri: String?, localName: String?, qName: String?) {
        this.elementStack.pop()
        if (ROOM_KEY == qName) {
            val roomData = this.objectStack.pop()
            this.joinedRooms.add(roomData as JoinedRoomData)

        }
    }

    override fun characters(ch: CharArray, start: Int, length: Int) {
        if (objectStack.empty()) {
            return
        }

        val roomData = objectStack.peek() as JoinedRoomData

        val value = String(ch, start, length)

        when (currentElement()) {
            JoinRoomStorageField.ROOM_ID.xmlTag -> {
                roomData.roomId += value
            }
            JoinRoomStorageField.ROOM_CREATOR.xmlTag -> {
                roomData.roomCreator += value
            }
            JoinRoomStorageField.JOIN_TIME.xmlTag -> {
                try {
                    roomData.joinTime = value.toLong()
                } catch (e: java.lang.Exception) {
                    Log.e(TAG, "Trouble Parsing jointime value: $value")
                    roomData.joinTime = 0

                }
            }
            JoinRoomStorageField.LAST_VIEWED.xmlTag -> {
                try {
                    roomData.lastViewed = value.toLong()
                } catch (e: Exception) {
                    Log.e(TAG, "Trouble Parsing lastViewed value: $value")
                    roomData.lastViewed = 0
                }
            }
            JoinRoomStorageField.ROOM_NAME.xmlTag -> {
                roomData.roomName += value
            }
            JoinRoomStorageField.LAST_VIEWED_ID.xmlTag -> {
                roomData.lastViewedMsgId = value
            }
            else -> {
                roomData.undefinedElements[currentElement()] = value
            }
        }
    }

    private fun currentElement(): String {
        return this.elementStack.peek()
    }
}
