package com.bria.common.controller.im.refactoring.dataprovider

import android.content.Context
import com.bria.common.controller.im.refactoring.adapter.IDataProviderEvents
import com.bria.common.uireusable.datatypes.MessageListItemData
import com.bria.common.util.Log
import io.reactivex.Flowable
import io.reactivex.schedulers.Schedulers
import java.util.concurrent.ScheduledExecutorService

class FilterableInstantMessageDataProvider(mContext: Context, callback: IDataProviderEvents, imExecutorService: ScheduledExecutorService) : InstantMessageDataProvider( mContext, callback, imExecutorService) {

    override val source: Flowable<ArrayList<MessageListItemData>> = sourceOfNewConversation
            .mergeWith(sourceOfChatRoomAddedMessages)
            .mergeWith(sourceOfChatRoomRemovedMessages)
            .observeOn(Schedulers.from(imExecutorService))
            .map {
                Log.d(TAG, "Event: ${it.first}")
                try {
                    when (it.first) {
                        NEW_LIST_EVENT, MESSAGES_ADDED_EVENT, MESSAGES_REMOVED_EVENT -> {
                            val listOfMessages = if (isChatRoom) {
                                getChatRoomMessagesForActiveChatRoom(0, numberOfLoadedMessages())
                            } else {
                                dbListToUIList(getImData().getNMessageWithOffsetForConversation(conversationId, 0, numberOfLoadedMessages()))
                            }
                            loading.set(false)
                            callback.onListLoaded(listOfMessages.size)
                            removedMessagesId.clear()
                            list.clear()
                            addMessagesToList(listOfMessages, list)
                        }
                        else -> {
                            Log.d(TAG, "Else event: ${it.first}")
                        }
                    }
                } catch (e : Exception) {

                }
                list

            }.mergeWith(sourceOfManuallyChangedList).replay(1).autoConnect()
    var filter : String = ""
    fun filterData(filter : String) {
        this.filter = filter
        sourceOfNewConversation.onNext(Pair(NEW_LIST_EVENT, emptyList()))
    }

    fun refilter() {
        sourceOfNewConversation.onNext(Pair(NEW_LIST_EVENT, emptyList()))
    }

    override fun getChatRoomMessagesForActiveChatRoom(offset: Int, numberOfLoadedMessages: Int): List<MessageListItemData> {
        val messages = getImData().chatRoomRepo.getFilteredMessagesWithOffsetForChat(conversationId, 0, 200, filter).filter { it.isIncoming() || it.isOutgoing() }
        return roomDbListToUIList(messages) //nvtd change this numbers
        // later!
    }

    override fun addMessagesToList(inputList: List<MessageListItemData>, outputList: MutableList<MessageListItemData>): ArrayList<MessageListItemData> {
        val addedMessages = ArrayList<MessageListItemData>()
        inputList.filter { !it.deleted }.forEach { uiMessage ->
            if (outputList.firstOrNull { it.id == uiMessage.id } == null) {
                addMessageToTheList(outputList, uiMessage)
                addedMessages.add(uiMessage)
            }
        }
        return addedMessages
    }
}
