package com.bria.common.controller.im.refactoring.roomdb

import android.arch.persistence.room.*
import com.bria.common.controller.im.refactoring.db.entities.ImStatusData
import com.bria.common.controller.im.refactoring.roomdb.entities.ChatRoom
import com.bria.common.controller.im.refactoring.roomdb.entities.Message
import com.bria.common.uireusable.datatypes.ExtendedConversationData
import io.reactivex.Maybe
import io.reactivex.Single


//nvtd move all to reactive?
@Dao
//@formatter:off
interface ChatRoomDao {
    @Insert(onConflict = OnConflictStrategy.REPLACE)
    fun insertRoom(chatRoom: ChatRoom) : Long?

    @Delete
    fun deleteRoom(room: ChatRoom)

    @Query("SELECT * FROM rooms WHERE type = ${ExtendedConversationData.CHAT_ROOM_TYPE} ORDER BY modTime DESC")  //where accountID in list
    fun getAllRooms(): Single<List<ChatRoom>>

    @Query("SELECT * FROM rooms WHERE id IS :id")
    fun getRoom(id: Long): ChatRoom?

    @Query("SELECT * FROM rooms WHERE chatKey IS :key")
    fun getRoomByRoomKey(key: String): ChatRoom?

    @Query("SELECT * FROM messages WHERE chatId = :id ORDER BY modificationTime DESC LIMIT 1")
    fun getLastMessageForChatRoom(id: Long): Maybe<Message>

    @Query("SELECT * FROM messages WHERE chatId = :id AND status = 10 ORDER BY modificationTime LIMIT 1")
    fun getLastIncomingMessageForChatRoom(id: Long): Maybe<Message>

    @Query("SELECT COUNT(status) FROM messages WHERE status IN (${ImStatusData.INCOMING_UNREAD}, ${ImStatusData.INCOMING_COLLAB_UNREAD}, ${ImStatusData.LOCAL}) AND read != 1 AND chatId = :id")
    fun getNumberOfUnreadMessagesForChatRoom(id: Long): Single<Int>

    @Query("SELECT * FROM messages WHERE status IN (${ImStatusData.INCOMING_UNREAD}, ${ImStatusData.INCOMING_COLLAB_UNREAD}, ${ImStatusData.LOCAL}) AND read != 1")
    fun getUnreadMessagesAllChatRooms(): Single<List<Message>>

    @Query("SELECT * FROM messages WHERE status IN (${ImStatusData.INCOMING_UNREAD}, ${ImStatusData.LOCAL}) AND read = 0")
    fun getUnreadNotSwipedMessages(): Single<List<Message>>

    @Query("SELECT * FROM messages WHERE status IN (${ImStatusData.INCOMING_UNREAD}, ${ImStatusData.INCOMING_COLLAB_UNREAD}, ${ImStatusData.LOCAL}) AND read != 1 AND chatId = :id")
    fun getUnreadMessagesForChatRoom(id: Long): Single<List<Message>>

    @Insert(onConflict = OnConflictStrategy.IGNORE)
    fun insertMessage(message: Message) : Long?

    @Insert(onConflict = OnConflictStrategy.IGNORE)
    fun insertMessages(messages: List<Message>) : Array<Long?>

    @Delete
    fun deleteMessages(messages: List<Message>) : Int

    @Update
    fun updateMessages(message: List<Message>) : Int

    @Query("SELECT * FROM messages WHERE id IS :id")
    fun getMessageById(id: Long) : Maybe<Message>

    @Query("UPDATE messages SET read = 1 WHERE id in (:messageIds)")
    fun markRead(messageIds : List<Long>)

    @Query("UPDATE messages SET read = 1 WHERE chatId = :chatRoomId")
    fun markReadChatRoom(chatRoomId: Long ) : Int

    @Query("SELECT * FROM messages WHERE chatId = :id ORDER BY modificationTime DESC LIMIT :limit OFFSET :offset")
    fun getNMessageWithOffsetForChat(id: Long, offset : Int, limit : Int) : List<Message>

    @Query("UPDATE rooms SET modTime = :time WHERE id = :chatId")
    fun updateChatTime(time: Long, chatId: Long)

    @Update
    fun updateChatRoom(chatRoom: ChatRoom)

    @Query("DELETE FROM messages WHERE chatId = :id")
    fun deleteMessagesForChatRoom(id: Long)

    @Query("SELECT * FROM rooms")
    fun getAllPrivateChatRooms(): Single<List<ChatRoom>>

    @Query("SELECT COUNT(status) FROM messages WHERE status = ${ImStatusData.INCOMING_UNREAD} AND read = 0")
    fun getNumberOfUnreadMessages(): Single<Int>

    @Query("SELECT * FROM messages WHERE chatId = :id AND text LIKE '%' || :filter || '%' ORDER BY modificationTime DESC LIMIT :limit OFFSET :offset")
    fun getFilteredMessagesWithOffsetForChat(id: Long, offset: Int, limit: Int, filter: String): List<Message>

    @Query("SELECT * FROM messages WHERE externalId IS :externalId")
    fun getMessageByExternalId(externalId: String): Message?

    @Query("SELECT * FROM messages WHERE id IN (:messageIds)")
    fun getMessagesWithIds(messageIds: List<Long>) : Single<List<Message>>

    @Query("SELECT * FROM messages WHERE status IN (${ImStatusData.INCOMING_UNREAD}, ${ImStatusData.INCOMING_COLLAB_UNREAD}, ${ImStatusData.LOCAL}) AND read!= 1 AND chatId = :id AND creationTime <= :time")
    fun getUnreadMessagesForChatRoomUpToTime(id: Long, time: Long): Single<List<Message>>

}
