package com.bria.common.controller.im.refactoring.roomdb

import com.bria.common.controller.im.refactoring.roomdb.entities.ChatRoom
import com.bria.common.controller.im.refactoring.roomdb.entities.Message
import io.reactivex.Maybe
import io.reactivex.Observable
import io.reactivex.Single
import io.reactivex.subjects.Subject

interface ChatRoomRepo {
    fun addChatRoom(chatRoom : ChatRoom) : Long

    fun deleteChatRoom(chatRoom: ChatRoom)

    fun changeOwner(owner : String)

    fun getOnRoomAddedObservable() : Observable<List<ChatRoom>>
    fun getOnRoomUpdatedObservable() : Observable<List<ChatRoom>>
    fun getOnRoomDeletedObservable() : Observable<List<ChatRoom>>

    fun getAllChatRooms() : Single<List<ChatRoom>>

    fun getLastMessageForChatRoom(id : Long) : Maybe<Message>

    fun getLastIncomingMessageForChatRoom(id : Long) : Maybe<Message>

    fun getNumberOfUnreadMessagesForChatRoom(id: Long): Int
    fun getNumberOfUnreadMessages(): Int

    fun getNumberOfChatsWithUnreadMessages() : Int

    fun addMessage(message: Message)

    fun getNMessageWithOffsetForChat(id: Long, offset: Int, limit: Int) : List<Message>

    fun getFilteredMessagesWithOffsetForChat(id: Long, offset: Int, limit: Int, filter : String) : List<Message>

    fun getOnMessagesAddedObservable(): Subject<List<Message>>

    fun getOnMessagesUpdatedObservable() : Subject<List<Message>>

    fun getDataChangedObservable() : Observable<Int>

    fun getOnServerRoomListChangedObservable() : Subject<List<LocalRoomInfo>>

    fun getOnMessagesRemovedObservable() : Subject<List<Message>>
    fun getOnMessagesReadObservable(): Subject<List<Message>>
    fun addMessages(messages: List<Message>)
    fun updateMessages(messages: List<Message>): Int
    fun deleteMessages(messages: List<Message>)
    fun markRead(messages: List<Message>)
    fun markReadChatRoom(id: Long)
    fun getRoomByRoomKey(key: String): ChatRoom?
    fun getChatRoom(id: Long): ChatRoom?
    fun getChatRoomByChatKey(roomKey: String): ChatRoom?
    fun getMessageById(id: Long): Maybe<Message>
    fun updateChatRoom(chatRoom: ChatRoom)


    fun removeRoomInfosForAcc(accId: String)
    fun roomNameInTheList(name: String): Boolean

    fun destroy()
    fun getAllPrivateChatRooms() : Single<List<ChatRoom>>
    fun getUnreadMessagesAllChatRooms(): Single<List<Message>>
    fun getUnreadMessagesForChatRoom(id: Long): Single<List<Message>>
    fun getUnreadMessagesForChatRoomUpToTime(id: Long, time : Long): Single<List<Message>>
    fun getOnRoomReadObservable(): Subject<List<ChatRoom>>

    fun removeRoomInfoWithJid(jid: String)
    fun getNumberOfUnreadCollabMessages(): Int
    fun getUnreadMessagesNotSwipedAllChatRooms(): Single<List<Message>>
    fun roomInTheServerList(jid: String): Boolean

    //LocalRoomInfo
    fun addRoomInfo(info: LocalRoomInfo)
    fun replaceRoomInfos(infos: List<LocalRoomInfo>)
    fun getRoomInfoByJid(jid: String): LocalRoomInfo?
    fun updateRoomInfo(info : LocalRoomInfo)

    fun getServerRoomList(query: String, withInfo: Boolean, hideLocalRooms : Boolean = false): MutableList<LocalRoomInfo>
    fun getMessagesWithIds(messageIds: List<Long>): Single<List<Message>>
    fun getMessageByExternalId(externalID: String): Message?
}
