package com.bria.common.controller.im.util

import android.content.Context
import android.text.Html
import android.text.TextUtils
import com.bria.common.R
import com.bria.common.controller.Controllers
import com.bria.common.controller.im.ImpsUtils
import com.bria.common.controller.im.refactoring.BuddyKeyUtils
import com.bria.common.controller.im.refactoring.ChatApi
import com.bria.common.controller.im.refactoring.db.entities.InstantMessageData
import com.bria.common.util.MimeTypes
import kotlin.math.abs

/**
 * For the lack of a better name.
 * */
object ImHelper {
    /**
     * @param contentType one of the MIME types
     */
    fun processIncomingMessage(incomingMessage: String?, contentType: String?): String {
        if (incomingMessage.isNullOrEmpty()) {
            return ""
        }

        if (contentType.equals(MimeTypes.TEXT_HTML, true)) {
            val body = Html.fromHtml(incomingMessage)
            return body.toString()
        }

        return incomingMessage ?: ""
    }

    /**
     * Tries to detect duplicate message, for cases when messages comes both thru SIP channel and via push.
     * We are forced to do simple comparison like this because there is no unique message id.
     * */
    fun isDuplicate(message1: InstantMessageData?, message2: InstantMessageData?): Boolean {
        if (message1 == null && message2 == null) {
            return true
        }

        if (message1 == null && message2 != null) {
            return false
        }

        if (message1 != null && message2 == null) {
            return false
        }

        var result = message1!!.message == message2!!.message

        val timeDifferenceThreshold = 200 //ms
        result = result && abs(message1.time - message2.time) < timeDifferenceThreshold

        return result
    }

    fun formatLocalMessage(context: Context, rawMessage: String): String {
        val separatorPosition = rawMessage.indexOf(ChatApi.LOCAL_MESSAGE_SEPARATOR)
        if (separatorPosition == -1) {
            return rawMessage
        }
        val eventType = rawMessage.substring(0, separatorPosition)
        var displayName = ""
        val displayNameNeeded = eventType.isNotBlank() && eventType in listOf<String>(ChatApi.PARTICIPANT_JOINED_EVENT_ID, ChatApi.PARTICIPANT_LEFT_EVENT_ID, ChatApi.PARTICIPANT_ADDED)
        if (displayNameNeeded) {
            val buddyKey = rawMessage.substring(separatorPosition + 1)
            val buddy = Controllers.get().buddy.getBuddyByNewKey(buddyKey)
            displayName = buddyKey
            if (buddy != null) {
                displayName = buddy.displayName
            } else {
                val address = BuddyKeyUtils.getImAdressFromNewBuddyKey(buddyKey)
                if (address != null) {
                    displayName = ImpsUtils.getUsernameFromAddress(address)
                }
            }
        }
        return if (TextUtils.isEmpty(eventType)) {
            rawMessage
        } else {
            when (eventType) {
                ChatApi.PARTICIPANT_JOINED_EVENT_ID ->
                    context.getString(R.string.tParticipantJoined, displayName) ?: displayName+"has joined the Group Chat"
                ChatApi.PARTICIPANT_LOCAL_INVITED_EVENT_ID ->
                    context.getString(R.string.tYouWereInvitedToChatRoom)
                ChatApi.PARTICIPANT_LOCAL_JOINED_EVENT_ID ->
                    context.getString(R.string.tYouHaveEnteredTheRoom)
                ChatApi.PARTICIPANT_LEFT_EVENT_ID ->
                    context.getString(R.string.tParticipantLeft, displayName) ?: displayName+"has left the Group Chat"
                ChatApi.PARTICIPANT_ADDED ->
                    context.getString(R.string.tAddedParticipant, displayName) ?: "Added $displayName"
                else -> rawMessage
            }

        }
    }
}
