package com.bria.common.controller.presence

import android.content.Context
import android.graphics.drawable.Drawable
import android.text.TextUtils
import android.util.Pair
import com.bria.common.controller.contacts.buddy.IBuddyCtrlEvents
import com.bria.common.controller.contacts.buddy.SipBuddy
import com.bria.common.util.kotlin.ListUtils


open class FavoritePresenceMatcher(val context: Context, val iBuddyCtrlEvents: IBuddyCtrlEvents) : IPresenceMatcher {


    override fun getPresence(id: String): BuddyPresence? {
        val allBuddies = iBuddyCtrlEvents.allBuddies
        val sipBuddies = ListUtils.filter(allBuddies) { x -> x is SipBuddy }
        val buddy = ListUtils.firstOrNull(sipBuddies) { x -> TextUtils.equals((x as SipBuddy).contactId, id) }
        return if (buddy?.presence == null) return null else buddy.presence
    }

    override fun getPresenceResources(id: String): Pair<Drawable, String> {

        val allBuddies = iBuddyCtrlEvents.allBuddies
        val sipBuddies = ListUtils.filter(allBuddies) { x -> x is SipBuddy }
        val buddy = ListUtils.firstOrNull(sipBuddies) { x -> TextUtils.equals((x as SipBuddy).contactId, id) } ?: return Pair<Drawable, String>(null, null)
        val presence = buddy.presence ?: return Pair<Drawable, String>(null, null)
        val status = presence.status ?: return Pair<Drawable, String>(null, null)
        val icon = status.getIcon(context)
        val presenceNote = presence.getPresenceNote(context)
        return Pair<Drawable, String>(icon, presenceNote)

    }

}
