package com.bria.common.crashlytics

import android.app.Application
import android.util.Log
import com.crashlytics.android.Crashlytics
import io.fabric.sdk.android.Fabric

class CrashlyticsImpl(application: Application) : ICrashlytics {
    init {
        Fabric.with(application, Crashlytics())
    }

    override fun sendPendingReports() {
        // just a handle to ensure init block is executed
    }

    override fun logNonFatalIssue(tag: String, message: String) {
        try {
            val exception = RuntimeException(tag + " " + message)

            var stackTrace = exception.stackTrace

            val skipMethods = listOf("logNonFatalIssue", "CrashInDebug", "fail")

            for (i in 1..3){
                val stackTraceElement = stackTrace[0]
                if(skipMethods.map { stackTraceElement.methodName.contains(it, true) }.any() ){
                    stackTrace = stackTrace.drop(1).toTypedArray()
                }
            }

            exception.stackTrace = stackTrace

            logNonFatalException(exception)
        } catch (_: Throwable) {
            // ignore errors during logging
        }
    }

    override fun logNonFatalException(throwable: Throwable) {
        try {
            Crashlytics.logException(throwable)
        } catch (_: Throwable) {
            // ignore errors during logging
        }
    }

    override fun logContextInfo(tag: String, message: String) {
        try {
            Crashlytics.log(Log.DEBUG, tag, message)
        } catch (_: Throwable) {
            // ignore errors during logging
        }
    }
}
