package com.bria.common.modules.android

import android.content.Context
import android.os.PowerManager
import com.bria.common.util.Log

object IncomingCallWakeLock {
    private val TAG = "IncomingCallWakeLock"

    private var mIncomingCallWakeLock : PowerManager.WakeLock? = null

    private val syncLock = Any()

    // ensure processor is working hard, so the app is not suspended mid startup during push call
    fun turnProcessorUpToEleven(context: Context, indefinite: Boolean = false) {
        synchronized(syncLock) {
            if (mIncomingCallWakeLock == null) {
                val powerManager = context.getSystemService(Context.POWER_SERVICE) as PowerManager? ?: return

                mIncomingCallWakeLock = powerManager.newWakeLock(PowerManager.SCREEN_BRIGHT_WAKE_LOCK or PowerManager.ACQUIRE_CAUSES_WAKEUP, javaClass.name)
            }

            if (mIncomingCallWakeLock!!.isHeld) {
                mIncomingCallWakeLock!!.release()
            }

            val oneMinute = 1 * 60 * 1000L
            if (indefinite) {
                mIncomingCallWakeLock!!.acquire(100 * oneMinute)
            } else {
                mIncomingCallWakeLock!!.acquire(oneMinute)
            }
        }
    }

    fun releaseWakeLock() {
        synchronized(syncLock){
            if (mIncomingCallWakeLock == null) {
                Log.d(TAG, "BriaApplication wakeLock is null")
                return
            }

            if (mIncomingCallWakeLock!!.isHeld) {
                Log.d(TAG, "Releasing BriaApplication wakeLock")
                mIncomingCallWakeLock!!.release()
            } else {
                Log.d(TAG, "BriaApplication wakeLock is not held")
            }

        }
    }
}

