package com.bria.common.modules.android

import android.app.Activity
import android.app.Application
import android.os.Bundle
import com.bria.common.uiframework.activities.AbstractActivity
import com.bria.common.uiframework.activities.ActivityLifecycleCallbacksAdapter
import com.bria.common.util.IObservable
import com.bria.common.util.Log
import com.bria.common.util.SyncObservableDelegate
import io.reactivex.functions.Consumer

class LivingActivitiesTracker(private val application: Application) {
    private val TAG = "LivingActivitiesTracker"

    private val mLivingActivities = mutableListOf<Activity>()
    val livingActivities: List<Activity>
        get() = mLivingActivities

    fun track() {
        application.registerActivityLifecycleCallbacks(object : ActivityLifecycleCallbacksAdapter() {

            override fun onActivityCreated(activity: Activity, savedInstanceState: Bundle?) {
                Log.d(TAG, "On activity created: " + activity.toString())

                mLivingActivities.add(activity)
                mObservable.notifyObservers { it.accept(mLivingActivities) }
            }

            override fun onActivityResumed(activity: Activity) {
                Log.d(TAG, "On activity resumed: " + activity.toString())

                if (activity is AbstractActivity) {
                    AbstractActivity.setFocusedActivity(activity)
                }

                BackgroundOrForegroundObservable.notify(BackgroundOrForegroundState.Foreground)
            }

            override fun onActivityDestroyed(activity: Activity) {
                Log.d(TAG, "onActivityDestroyed: " + activity.toString())

                mLivingActivities.remove(activity)
                mObservable.notifyObservers { it.accept(mLivingActivities) }
            }
        })
    }

    private val mObservable = SyncObservableDelegate<Consumer<List<Activity>>>()
    val observable: IObservable<Consumer<List<Activity>>>
        get() = mObservable
}
