package com.bria.common.modules.android

import android.content.BroadcastReceiver
import android.content.Context
import android.content.Intent
import android.content.IntentFilter
import com.bria.common.util.IObservable
import com.bria.common.util.Log
import com.bria.common.util.SyncObservableDelegate

class ScreenOnOffReceiver : BroadcastReceiver() {

    companion object {
        private val TAG = "ScreenOnOffReceiver"

        val intentFilter: IntentFilter = IntentFilter()

        init {
            intentFilter.addAction(Intent.ACTION_SCREEN_ON)
            intentFilter.addAction(Intent.ACTION_SCREEN_OFF)
        }
    }

    override fun onReceive(context: Context, intent: Intent?) {
        // check current state from intent
        var isInteractive: Boolean? = null
        if (intent?.action == Intent.ACTION_SCREEN_OFF) {
            isInteractive = false
        } else if (intent?.action == Intent.ACTION_SCREEN_ON) {
            isInteractive = true
        }

        Log.d(TAG, "Screen state changed, is device interactive: $isInteractive")

        if(isInteractive != null){
            mObservable.notifyObservers { it.newValue(isInteractive) }
        }
    }

    private val mObservable = SyncObservableDelegate<IDeviceInteractivityChanged>()
    val deviceInteractivityObservable: IObservable<IDeviceInteractivityChanged>
        get() = mObservable
}
