package com.bria.common.notification;

import android.app.Notification
import android.app.NotificationChannel
import android.app.NotificationManager
import android.content.Context
import android.content.Intent
import android.support.v4.app.NotificationCompat
import android.support.v4.app.NotificationManagerCompat
import com.bria.common.R
import com.bria.common.controller.Controller
import com.bria.common.controller.notifications.NotificationParams
import com.bria.common.controller.remotedebug.EHdaConnectionStatus
import com.bria.common.controller.remotedebug.IRemoteDebugCtrlObserver
import com.bria.common.controller.remotedebug.RemoteDebugController
import com.bria.common.controller.remotedebug.RemoteDebugStatusMessage
import com.bria.common.controller.settings.ESetting
import com.bria.common.controller.settings.core.ISettingsObserver
import com.bria.common.controller.settings.core.Settings
import com.bria.common.modules.BriaGraph.settings
import com.bria.common.notification.NotificationHelper.SERVICE_MESSAGES_CHANNEL_ID
import com.bria.common.util.GlobalConstants

class NotificationRemoteDebug(val mAppContext: Context, val mController: Controller, val mNotificationManager: NotificationManagerCompat) :
        AbstractNotification(), IRemoteDebugCtrlObserver {

    private var mNotificationsRemoteDebugEnabled: Boolean
    private val TAG = "NotificationRemoteDebug"

    private var mSettingsObserver: ISettingsObserver

    private var mSettings: Settings?

    init {
        RemoteDebugController.getInstance().observable.attachWeakObserver(this)

        mSettings = Settings.get(mAppContext)

        mNotificationsRemoteDebugEnabled = settings.getBool(ESetting.NotificationsRemoteDebugEnabled)

        mSettingsObserver = object : ISettingsObserver {
            override fun onBeforeOwnerChange(newOwner: String?) {}

            override fun onOwnerChanged() {}

            override fun onSettingsChanged(changedSettings: MutableSet<ESetting>) {
                if (changedSettings.contains(ESetting.NotificationsRemoteDebugEnabled)) {
                    mNotificationsRemoteDebugEnabled = settings.getBool(ESetting.NotificationsRemoteDebugEnabled)
                    if (!mNotificationsRemoteDebugEnabled) {
                        mNotificationManager.cancel(NotificationParams.ENotificationType.RemoteDebug.ordinal)
                    }
                }
            }
        }

        settings.attachWeakObserver(mSettingsObserver, setOf(ESetting.NotificationsRemoteDebugEnabled))

        createNotificationChannel()
    }

    override fun addActionButtons(builder: NotificationCompat.Builder?, notificationParams: NotificationParams) {
    }

    override fun createNotification(builder: NotificationCompat.Builder, notificationParams: NotificationParams) {
        val notification = builder.build()
        notification.flags = notification.flags or notificationParams.flags
        NotificationHelper.postNotification(notificationParams.notificationID, notification, mNotificationManager)
        NotificationHelper.addToActiveMap(notificationParams)
    }

    override fun createNotificationChannel() {
        val notificationManager = mAppContext.getSystemService(Context.NOTIFICATION_SERVICE) as NotificationManager
        if (android.os.Build.VERSION.SDK_INT >= android.os.Build.VERSION_CODES.O) {
            val serviceChannelName = mAppContext.getString(R.string.tNotificationChannelServiceMessagesTitle)
            val serviceChannelDescription = mAppContext.getString(R.string.tNotificationChannelServiceMessagesDescription)
            val serviceNotificationChannel = NotificationChannel(SERVICE_MESSAGES_CHANNEL_ID, serviceChannelName,
                    NotificationManager.IMPORTANCE_MIN)
            serviceNotificationChannel.description = serviceChannelDescription
            serviceNotificationChannel.setShowBadge(false)
            notificationManager.createNotificationChannel(serviceNotificationChannel)
        }
    }

    override fun setupNotification() {
    }

    override fun destroy() {
        RemoteDebugController.getInstance().observable.detachObserver(this)
        mSettings?.detachObserver(mSettingsObserver)
    }

    override fun createNotificationBuilder(notificationParams: NotificationParams): NotificationCompat.Builder = NotificationHelper.createNotificationBuilder(mAppContext, mController, notificationParams)

    override fun handleAction(intent: Intent?) {
    }

    override fun onStatusChanged(remoteDebugStatusMessage: RemoteDebugStatusMessage, status: EHdaConnectionStatus) {
        if (remoteDebugStatusMessage.category == RemoteDebugStatusMessage.ECategory.REMOTE_DEBUG) {
            updateRemoteDebugNotification(remoteDebugStatusMessage.notifBar)
        }
    }

    fun updateRemoteDebugNotification(show: Boolean) {
        if (show) {
            val notificationId = NotificationParams.ENotificationType.RemoteDebug.ordinal
            val notificationParams = NotificationHelper.updateNotificationParams(notificationId, R.drawable
                    .ic_stat_raw_icons_remote_debugging_remote_debugging,
                    null, mAppContext.getString(R.string.tRemoteDebug), mAppContext.getString(R.string.tRemoteDebugIsConnected), null, 0,
                    GlobalConstants.INTENT_NOTIFICATION_HELP_DESK_ASSISTANT, NotificationParams.ENotificationType.RemoteDebug, null, false,
                    Notification.PRIORITY_DEFAULT, System.currentTimeMillis())

            val builder = createNotificationBuilder(notificationParams)
            createNotification(builder, notificationParams)
        } else {
            mNotificationManager.cancel(NotificationParams.ENotificationType.RemoteDebug.ordinal)
        }
    }
}
