package com.bria.common.uiframework.helpers

import android.content.Context
import android.net.Uri
import com.bria.common.R
import com.bria.common.controller.contacts.buddy.IBuddyCtrlEvents
import com.bria.common.controller.contacts.buddy.IBuddyCtrlObserver
import com.bria.common.controller.contacts.buddy.v2.BuddyRequestDataProviderV2
import com.bria.common.controller.contacts.buddy.v2.V2BuddyDataProvider
import com.bria.common.controller.contacts.ldap.LdapDataProviderV2
import com.bria.common.controller.contacts.local.ContactsApi
import com.bria.common.controller.contacts.local.favorites.FavoritesApi
import com.bria.common.controller.contacts.local.favorites.V2FavoriteDataProvider
import com.bria.common.controller.presence.*
import com.bria.common.controller.settings.ESetting
import com.bria.common.controller.settings.core.Settings
import com.bria.common.modules.BriaGraph
import com.bria.common.uireusable.dataprovider.ContactDataProviderV2
import com.bria.common.uireusable.dataprovider.IContactActionDataProvider
import com.bria.common.uireusable.datatypes.ContactTabItem
import com.bria.common.uireusable.datatypes.IPersonListItem
import com.bria.common.util.GlobalConstants
import com.bria.common.util.IObservable
import io.reactivex.Observable

class ContactTabManager(
        private val context: Context,
        iBuddyCtrlEvents: IBuddyCtrlEvents,
        iBuddyCtrlObserver: IObservable<IBuddyCtrlObserver>,
        favoritesApi: FavoritesApi,
        contactChangedObservable: Observable<Uri>,
        contactsApi: ContactsApi,
        iPresenceCtrlObserver: IObservable<IPresenceCtrlObserver>,
        iPresenceCtrlEvents: IPresenceCtrlEvents) {

    //Sub-filter constants
    val SUBTYPE_ALL = 0
    val SUBTYPE_ONLINE = 1
    val SUBTYPE_REQUESTS = 2


    private var tabList: ArrayList<ContactTabItem>? = ArrayList()
    private var activeTab: ContactTabItem
    private lateinit var buddyRequestDataProviderV2: BuddyRequestDataProviderV2
    private var subFilterType = 0

    init {

        val contactsTab = ContactTabItem(GlobalConstants.EContactsFilterType.CONTACTS, context.getString(R.string.tSwitchToContacts), true, false, context.getString(R.string.tContactsLoading), context.getString(R.string.tContactsEmpty), presenceMatcher = ContactPresenceMatcher(context, iBuddyCtrlEvents), dataProvider = ContactDataProviderV2(context, contactChangedObservable, contactsApi, iBuddyCtrlObserver))
        tabList?.add(contactsTab)
        val buddiesTab = ContactTabItem(GlobalConstants.EContactsFilterType.BUDDIES, context.getString(R.string.tSwitchToBuddies), false, true, context.getString(R.string.tBuddiesLoading), context.getString(R.string.tBuddiesEmpty), presenceMatcher = BuddyPresenceMatcher(context, iBuddyCtrlEvents), dataProvider = V2BuddyDataProvider(iBuddyCtrlEvents, iBuddyCtrlObserver))
        tabList?.add(buddiesTab)

        if (isLdapOn()) {
            val ldapTab = ContactTabItem(GlobalConstants.EContactsFilterType.LDAP, context.getString(R.string.tDirectory), false, false, context.getString(R.string.tBuddiesLoading), context.getString(R.string.tNoResults), dataProvider = LdapDataProviderV2(BriaGraph.ldapModule), pullContentFromServer = true, cleanSearchOnClose = false)
            tabList?.add(ldapTab)
        }


        val favoritesTab = ContactTabItem(GlobalConstants.EContactsFilterType.FAVORITES, context.getString(R.string.tFavorites), true, false, context.getString(R.string.tContactsLoading), context.getString(R.string.tFavoritesEmpty), hasSearchFromMenu = false, presenceMatcher = FavoritePresenceMatcher(context, iBuddyCtrlEvents), dataProvider = V2FavoriteDataProvider(favoritesApi, iBuddyCtrlObserver))
        tabList?.add(favoritesTab)

        activeTab = contactsTab

        buddyRequestDataProviderV2 = BuddyRequestDataProviderV2(iPresenceCtrlEvents, iPresenceCtrlObserver)
        buddyRequestDataProviderV2.initializeLoadingData()

    }

    fun getBuddyReqestDataProvider(): BuddyRequestDataProviderV2 = buddyRequestDataProviderV2


    fun getActiveDataProvider(): IContactActionDataProvider<IPersonListItem> {
        if (getActiveTab().tagName == GlobalConstants.EContactsFilterType.BUDDIES && subFilterType == SUBTYPE_REQUESTS) {
            return buddyRequestDataProviderV2
        }
        return getActiveTab().dataProvider
    }

    fun getTabList(): ArrayList<ContactTabItem>? {
        return tabList
    }

    fun setActiveTab(activeTab: GlobalConstants.EContactsFilterType) {
        tabList?.forEach { t ->
            if (t.tagName == activeTab)
                this.activeTab = t
        }
    }

    fun getActiveTab(): ContactTabItem {
        return activeTab
    }

    fun setSubFilterType(subFilterType: Int) {
        this.subFilterType = subFilterType
    }

    fun getSubFilterType(): Int = subFilterType
    private fun isLdapOn(): Boolean = Settings.get(context).getBool(ESetting.FeatureLdap) && Settings.get(context).getBool(ESetting.LdapEnabled)


}
