package com.bria.common.uiframework.presenters

import com.bria.common.permission.PermissionChecker

class AbstractPresenterPermissionHelper(
        private val permissionChecker: PermissionChecker,
        private val requestPermission: (neededPermission: String, requestCode: Int, explanation: String) -> Unit
) {
    private val TAG = "AbstractPresenterPermissionHelper"

    private var mContinuation: Holder? = null

    fun executeDependingOnPermissions(
            neededPermission: String,
            whenGranted: () -> Unit,
            whenDenied: (() -> Unit)?,
            requestCode: Int,
            explanation: String
    ) {
        if (permissionChecker.isPermissionGranted(neededPermission)) {
            whenGranted()
        } else {
            mContinuation = Holder(requestCode, whenGranted, whenDenied)
            requestPermission(neededPermission, requestCode, explanation)
        }
    }

    fun handlePermissionResult(requestCode: Int, granted: Boolean) {
        if (mContinuation == null) {
            return
        }

        if (mContinuation?.requestCode != requestCode) {
            return
        }

        if (granted) {
            mContinuation?.whenGranted?.invoke()
        } else {
            mContinuation?.whenDenied?.invoke()
        }
    }

    private data class Holder(
            val requestCode: Int,
            val whenGranted: () -> Unit,
            val whenDenied: (() -> Unit)?)
}
