package com.bria.common.util

import com.bria.common.BuildConfig
import com.bria.common.controller.ClientConfig
import com.bria.common.crashlytics.CrashlyticsStub
import com.bria.common.crashlytics.ICrashlytics

object CrashInDebug {

    private var crashlytics: ICrashlytics = CrashlyticsStub()

    /**
     * init after crashlytics is initialized
     */
    fun init(crashlytics: ICrashlytics) {
        this.crashlytics = crashlytics
    }

    /**
     * Crashes the application with the given message and tag when [ClientConfig.isDebugMode] is true. If it's false, then it just logs with an [ ][Log.e] command.
     *
     * @param tag The tag to be used for the log.
     * @param withMessage The message that will be displayed in the log or the Exception.
     */
    @JvmStatic
    fun with(tag: String, withMessage: String) {
        Log.e(tag, withMessage)

        if (BuildConfig.DEBUG) {
            Log.e(tag, withMessage, RuntimeException(withMessage))
            System.exit(-1)
        } else {
            crashlytics.logNonFatalIssue(tag, withMessage)
        }
    }

    /**
     * Crashes the application with the given message and tag when [ClientConfig.isDebugMode] is true. If it's false, then it just logs with an [ ][Log.e] command.
     *
     * @param tag The tag to be used for the log.
     * @param throwable Throwable that you would like to throw.
     */
    @JvmStatic
    fun with(tag: String, throwable: Throwable?) {
        Log.e(tag, throwable?.message, throwable)

        if (BuildConfig.DEBUG) {
            System.exit(-1)
        } else {
            if (throwable != null) {
                crashlytics.logNonFatalException(throwable)
            }
        }
    }
}
