package com.bria.common.util

import com.bria.common.controller.accounts.Account
import com.bria.common.controller.settings.EAccountSetting

object SipAddressUtils {
    @JvmStatic
    @JvmOverloads
    fun getAddress(address: String, domain: String, emptyIfNoDomain: Boolean = true) =
            if (address.isNotEmpty() && domain.isNotEmpty()) "${removeDomain(address)}@${domain.removePrefix("@")}" else
                if (domain.isEmpty() && emptyIfNoDomain) "" else address

    @JvmStatic
    fun getAddress(account: Account) = SipAddressUtils.getAddress(account.getStr(EAccountSetting.UserName), account.getStr(EAccountSetting.Domain))

    @JvmStatic
    fun getUser(address: String) = SipAddressUtils.startIndexOfDomain(address).let { if (it > 0) address.substring(0, it - 1) else "" }

    @JvmStatic
    @JvmOverloads
    fun getDomain(address: String, isRemovePort: Boolean = false) = SipAddressUtils.startIndexOfDomain(address)
            .let { if (it > 0) address.substring(it, address.length) else "" }
            .let { if (isRemovePort) SipAddressUtils.removePort(it) else it }

    @JvmStatic
    fun isHasDomain(address: String) = SipAddressUtils.startIndexOfDomain(address) > 0

    @JvmStatic
    fun startIndexOfDomain(address: String) = address.indexOf('@').let { if (it > 0 && (it + 1) < address.length) it + 1 else -1 }

    @JvmStatic
    fun removeDomain(address: String) = address.indexOf('@').let { if (it != -1) address.substring(0, it) else address }

    @JvmStatic
    fun removePort(address: String) = address.indexOfLast { it == ':' }.let { if (it != -1) address.substring(0, it) else address }

    @JvmStatic
    fun compare(address1: String, address2: String, userOnly: Boolean = false) : Boolean {
        // If both addresses in the same format (both has or both has no domain)
        if (!userOnly && SipAddressUtils.isHasDomain(address1) == SipAddressUtils.isHasDomain(address2)) {
            // Compare addresses as is (except port)
            return SipAddressUtils.removePort(address1).equals(SipAddressUtils.removePort(address2), ignoreCase = true)
        }

        // If only one address has domain or userOnly is true
        // Compare addresses by user only
        return SipAddressUtils.removeDomain(address1).equals(SipAddressUtils.removeDomain(address2), ignoreCase = true)
    }
}
