package com.bria.voip.ui.main.calllog.refactoring

import android.os.Bundle
import com.bria.common.controller.calllog.compareNumber
import com.bria.common.controller.calllog.db.CallLogEntity
import com.bria.common.controller.calllog.getNumberForContacts
import com.bria.common.uiframework.presenters.IPresenterEventTypeEnum
import com.bria.common.util.Log
import com.bria.common.util.SipAddressUtils
import com.bria.voip.R
import io.reactivex.disposables.Disposable
import io.reactivex.rxkotlin.subscribeBy
import io.reactivex.schedulers.Schedulers
import io.reactivex.subjects.BehaviorSubject

class CallLogDetailsPresenter : CallLogAbstractPresenter() {
    companion object {
        private const val TAG = "CallLogDetailsPresenter"

        const val KEY_CALL_LOG_DETAILS_GROUP_ID = "KEY_CALL_LOG_DETAILS_GROUP_ID"
    }

    enum class Events : IPresenterEventTypeEnum {
        GO_UP
        // define new events here
        ;
        override fun getType(): IPresenterEventTypeEnum = this
    }

    override val isTabsVisible = false

    // Group Entity ID its ID of first Call Log Entity in group
    private var mGroupEntityId: Long? = null

    private val mBehaviorSubject = BehaviorSubject.create<List<CallLogEntity>>()

    private var mGetAllDisposable: Disposable? = null
    private var mCallLogDataChangedDisposable: Disposable? = null


    override fun onCreate() {
        super.onCreate()
        mGetAllDisposable = mCallLogApi.getAll().subscribe(
                mBehaviorSubject::onNext,
                mBehaviorSubject::onError,
                mBehaviorSubject::onComplete)
    }

    override fun onDestroy() {
        super.onDestroy()
        mGetAllDisposable?.dispose()
    }

    override fun onUnsubscribe() {
        super.onUnsubscribe()
        mCallLogDataChangedDisposable?.dispose()
    }

    // = PUBLIC section

    fun onScreenStart(bundle: Bundle?) {
        mGroupEntityId = bundle?.getLong(KEY_CALL_LOG_DETAILS_GROUP_ID, mGroupEntityId ?: -1) ?: mGroupEntityId
        mCallLogDataChangedDisposable?.dispose()
        mCallLogDataChangedDisposable = mBehaviorSubject
                .subscribeOn(Schedulers.io())
                .observeOn(Schedulers.io())
                .doOnNext { list -> list.find { it.id == mGroupEntityId } ?: firePresenterEvent(Events.GO_UP) }
                .subscribeBy(
                        onNext = this::onCallLogDataChanged,
                        onError = { Log.e(TAG, "CallLogApi error", it) },
                        onComplete = { Log.w(TAG, "CallLogApi complete") }
                )
    }

    override fun getTitle() = getString(R.string.tCallDetails)

    override fun onScreenNewConfig(bundle: Bundle?) = onScreenStart(bundle)

    // = PRIVATE section

    override fun buildGroupedList(newEntityList: List<CallLogEntity>): List<CallLogItemModel> {
        val newGroupDetailsList = mutableListOf<CallLogItemModel>()

        val indexOfGroup = newEntityList.indexOfFirst { it.id == mGroupEntityId }
        if (indexOfGroup != -1) {
            val groupEntity = newEntityList[indexOfGroup]
            val subEntityList = mutableListOf<CallLogEntity>()

            for (i in indexOfGroup downTo 0) {
                val entity = newEntityList[i]
                if (!groupEntity.compareNumber(entity)) {
                    break
                }
                subEntityList += entity
            }

            var group: CallLogGroupItem? = null
            for (entity in subEntityList.asReversed()) {
                if (group == null) {
                    group = buildGroup(entity).apply { isExpanded = true; isCallButtonVisible = false }
                } else {
                    group.items += buildContent(entity)
                }
            }

            group?.let {
                newGroupDetailsList += it
                newGroupDetailsList += it.items
            }
        }

        return newGroupDetailsList
    }
}
