package com.bria.voip.ui.main.calllog.refactoring

import android.graphics.Bitmap
import com.bria.common.controller.calllog.db.CallType
import com.bria.common.uireusable.datatypes.IdentifiableDataItem

abstract class CallLogItemModel : IdentifiableDataItem {
    open val isGroup = false
    open val id: String = ""

    override fun getUniqueIdentifier() = id
}

data class CallLogGroupItem(
        override val id: String,
        val number: String,
        var numberType: String = "",
        var displayName: String = "",
        val date: Long,
        var contactId: String = "",
        var buddyKey: String? = null,
        var contactPhoto: Bitmap? = null,
        var isContactInfoDeprecated: Boolean = true,
        var isCallButtonVisible: Boolean = true,
        val callTypes: MutableList<CallType> = mutableListOf(),
        val items : MutableList<CallLogContentItem> = mutableListOf(),
        // Below not used in adapter directly
        val accountId: Int,
        val accountDomain: String,
        val remoteDomain: String,
        val hostedCollab: Boolean,
        var externalId: String = "" // Optional. Can be used to store Genband Key or Broadworks remote log ID
) : CallLogItemModel() {
    override val isGroup = true
    var isExpanded = false
    var isSelected = false

    companion object {
        const val TAG = "CallLogGroupItem"
    }
}

data class CallLogContentItem(
        override val id: String,
        val callType: CallType,
        val date: Long,
        val durationSeconds: Int,
        val recordFileName: String,
        val forwardedTo: String,
        val isCallForwardedInfoVisible: Boolean,
        val isCallDateColored: Boolean,
        val isCallDurationVisible: Boolean,
        val isRecordInfoVisible: Boolean,
        val isSaveButtonVisible: Boolean,
        val remoteName: String,
        val hostedCollab: Boolean,
        var externalId: String // Optional. Can be used to store Genband Key or Broadworks remote log ID
) : CallLogItemModel() {
    override val isGroup = false
}
