package com.bria.voip.ui.main.calllog.refactoring

import android.graphics.Color
import android.support.v7.widget.CardView
import android.support.v7.widget.RecyclerView
import android.view.ContextMenu
import android.view.View
import android.widget.ImageView
import android.widget.TextView
import com.bria.common.controller.Controllers
import com.bria.common.controller.settings.ESetting
import com.bria.common.uiframework.coloring.Coloring
import com.bria.common.uiframework.lists.recycler.AbstractRecyclerAdapter
import com.bria.common.uireusable.dataprovider.ISimpleDataProvider
import com.bria.common.util.CrashInDebug
import com.bria.common.util.kotlin.setGone
import com.bria.common.util.kotlin.setInvisible
import com.bria.common.util.kotlin.setVisible
import com.bria.voip.R
import com.bria.voip.ui.shared.helpers.setVisible
import de.hdodenhof.circleimageview.CircleImageView

class CallLogListAdapter(recyclerView: RecyclerView, dataProvider: ISimpleDataProvider<CallLogItemModel>) :
        AbstractRecyclerAdapter<CallLogItemModel, AbstractRecyclerAdapter.AbstractViewHolder>(recyclerView, dataProvider,
                R.layout.calllog_list_item_group,
                R.layout.calllog_list_item_content,
                R.layout.calllog_list_item_content_record) {

    companion object {
        private const val TAG = "CallLogListAdapter"

        private const val TYPE_ITEM_GROUP = 0
        private const val TYPE_ITEM_CONTENT = 1
        private const val TYPE_ITEM_CONTENT_RECORD = 2
    }
    private val mBrandColor = Coloring.get().decodeColor(Controllers.get().settings.getStr(ESetting.ColorBrandDefault))

    var onCreateContextMenuListener: ((ContextMenu, View, ContextMenu.ContextMenuInfo?, Int) -> Any)? = null
    var onItemClickListener: ((View, Int) -> Any)? = null
    var onItemContactButtonClickListener: ((View, Int) -> Any)? = null
    var onItemCallButtonClickListener: ((View, Int) -> Any)? = null
    var onItemDeleteRecordButtonClickListener: ((View, Int) -> Any)? = null
    var onItemSaveRecordButtonClickListener: ((View, Int) -> Any)? = null
    var onItemShareRecordButtonClickListener: ((View, Int) -> Any)? = null
    var onItemPlayRecordButtonClickListener: ((View, Int) -> Any)? = null

    class GroupViewHolder(itemView: View) : AbstractRecyclerAdapter.AbstractViewHolder(itemView) {
        val cardView: CardView = itemView.findViewById(R.id.calllog_list_item_group_card)
        val contactPhotoCircle: CircleImageView = itemView.findViewById(R.id.calllog_list_item_group_contact_photo_circle)
        val contactPhotoSquare: ImageView = itemView.findViewById(R.id.calllog_list_item_group_contact_photo_square)
        val contactPresence: ImageView = itemView.findViewById(R.id.calllog_list_item_group_contact_presence)
        val recordIndicator: ImageView = itemView.findViewById(R.id.calllog_list_item_group_record_indicator)
        val displayName: TextView = itemView.findViewById(R.id.calllog_list_item_group_display_name)
        val callTypeIcons: CallTypeView = itemView.findViewById(R.id.calllog_list_item_group_call_type_icons)
        val callInfo: TextView = itemView.findViewById(R.id.calllog_list_item_group_call_info)
        val callButton: ImageView = itemView.findViewById(R.id.calllog_list_item_group_call_button)

        override fun hasDivider() = false
        override fun isClickable() = false
        override fun isLongClickable() = false
    }

    class ContentViewHolder(itemView: View) : AbstractRecyclerAdapter.AbstractViewHolder(itemView) {
        val callTypeIcon: ImageView = itemView.findViewById(R.id.calllog_list_item_content_call_type_icon)
        val callInfo: TextView = itemView.findViewById(R.id.calllog_list_item_content_call_info)
        val callDuration: TextView = itemView.findViewById(R.id.calllog_list_item_content_call_duration)
        val callDate: TextView = itemView.findViewById(R.id.calllog_list_item_content_call_date)
        val callForwardedInfo: TextView = itemView.findViewById(R.id.calllog_list_item_content_call_forwarded_info)

        override fun hasDivider() = false
        override fun isClickable() = false
        override fun isLongClickable() = false
    }

    class ContentRecordViewHolder(itemView: View) : AbstractRecyclerAdapter.AbstractViewHolder(itemView) {
        val callTypeIcon: ImageView = itemView.findViewById(R.id.calllog_list_item_content_record_call_type_icon)
        val callInfo: TextView = itemView.findViewById(R.id.calllog_list_item_content_record_call_info)
        val callDuration: TextView = itemView.findViewById(R.id.calllog_list_item_content_record_call_duration)
        val callDate: TextView = itemView.findViewById(R.id.calllog_list_item_content_record_call_date)
        val recordInfo: TextView = itemView.findViewById(R.id.calllog_list_item_content_record_file_info)
        val deleteButton: ImageView = itemView.findViewById(R.id.calllog_list_item_content_record_delete_button)
        val saveButton: ImageView = itemView.findViewById(R.id.calllog_list_item_content_record_save_button)
        val shareButton: ImageView = itemView.findViewById(R.id.calllog_list_item_content_record_share_button)
        val playButton: ImageView = itemView.findViewById(R.id.calllog_list_item_content_record_play_button)

        override fun hasDivider() = false
        override fun isClickable() = false
        override fun isLongClickable() = false
    }

    override fun getItemViewType(position: Int): Int {
        return getItemAt(position)!!.let {
            when {
                it.isGroup -> TYPE_ITEM_GROUP
                it is CallLogContentItem && it.isRecordInfoVisible -> TYPE_ITEM_CONTENT_RECORD
                else -> TYPE_ITEM_CONTENT
            }
        }
    }

    override fun provideNewViewHolder(inflatedView: View, viewType: Int): AbstractRecyclerAdapter.AbstractViewHolder {
        return when (viewType) {
            TYPE_ITEM_GROUP -> CallLogListAdapter.GroupViewHolder(inflatedView)
            TYPE_ITEM_CONTENT -> CallLogListAdapter.ContentViewHolder(inflatedView)
            TYPE_ITEM_CONTENT_RECORD -> CallLogListAdapter.ContentRecordViewHolder(inflatedView)
            else -> throw RuntimeException("Unexpected viewType [$viewType]")
        }
    }

    override fun onNewViewHolderReady(viewHolder: AbstractRecyclerAdapter.AbstractViewHolder, viewType: Int) { }

    override fun onUpdateNeeded(viewHolder: AbstractRecyclerAdapter.AbstractViewHolder, position: Int) {
        val item = getItemAt(position)
        if (item == null) {
            CrashInDebug.with(TAG, "Item at position $position is null, something went wrong")
            return
        }

        when (viewHolder.itemViewType) {
            TYPE_ITEM_GROUP -> updateGroupViewHolder(
                    viewHolder as CallLogListAdapter.GroupViewHolder, item as CallLogGroupItem)
            TYPE_ITEM_CONTENT -> updateContentViewHolder(
                    viewHolder as CallLogListAdapter.ContentViewHolder, item as CallLogContentItem)
            TYPE_ITEM_CONTENT_RECORD -> updateContentRecordViewHolder(
                    viewHolder as CallLogListAdapter.ContentRecordViewHolder, item as CallLogContentItem)
        }
    }

    private fun updateGroupViewHolder(viewHolder: CallLogListAdapter.GroupViewHolder, dataItem: CallLogGroupItem) {
        viewHolder.cardView.setCardBackgroundColor(if (dataItem.isSelected) mBrandColor else Color.WHITE)
        // todo val contrastColor = Coloring.get().getContrastColor(dataItem.backgroundColor)

        viewHolder.displayName.text = dataItem.displayName
        viewHolder.callTypeIcons.apply {
            if (dataItem.isCallTypeIconsVisible()) {
                setVisible()
                clear()
                dataItem.callTypes.forEach { add(it) }
                requestLayout()
            } else {
                setGone()
            }
        }

        viewHolder.callInfo.apply {
            text = dataItem.getCallInfo()
            setVisible(dataItem.isCallInfoVisible())
        }

        // Circle Image View is used for contacts
        viewHolder.contactPhotoCircle.apply {
            setVisible(!dataItem.isConference())
            when {
                dataItem.isConference() -> setImageDrawable(null) // clear
                dataItem.contactPhoto != null -> setImageBitmap(dataItem.contactPhoto)
                dataItem.isVoiceMail() -> setImageResource(R.drawable.ic_voice_mail)
                else -> setImageResource(R.drawable.default_avatar)
            }
        }

        // Usual (square) Image View is used for conferences
        viewHolder.contactPhotoSquare.apply {
            setVisible(dataItem.isConference())
            when {
                dataItem.isConference() && !dataItem.hostedCollab -> setImageResource(R.drawable.ic_calllog_conference)
                dataItem.isConference() && dataItem.hostedCollab -> setImageResource(R.drawable.ic_calllog_hosted_conference)
                else -> setImageDrawable(null) // clear
            }
        }

        viewHolder.contactPresence.apply {
            val resId = dataItem.getPresenceResId()
            setImageResource(resId)
            if (resId != 0) setVisible() else setInvisible()
        }

        viewHolder.recordIndicator.apply { if (dataItem.isRecordIndicatorVisible()) setVisible() else setInvisible() }
        viewHolder.callButton.setVisible(dataItem.isCallButtonVisible)

        onCreateContextMenuListener?.let {
            viewHolder.cardView.setOnCreateContextMenuListener { menu, view, menuInfo ->
                it.invoke(menu, view, menuInfo, viewHolder.adapterPosition)
            }
        }
        onItemClickListener?.let { viewHolder.cardView.setOnClickListener { view -> it.invoke(view, viewHolder.adapterPosition) } }
        onItemContactButtonClickListener?.let { viewHolder.contactPhotoCircle.setOnClickListener { view -> it.invoke(view, viewHolder.adapterPosition) } }
        onItemCallButtonClickListener?.let { viewHolder.callButton.setOnClickListener { view -> it.invoke(view, viewHolder.adapterPosition) } }
    }

    private fun updateContentViewHolder(viewHolder: CallLogListAdapter.ContentViewHolder, dataItem: CallLogContentItem) {
        viewHolder.callTypeIcon.setImageResource(dataItem.getCallTypeIconResId())
        viewHolder.callInfo.setText(dataItem.getCallInfo())
        viewHolder.callDate.apply {
            text = dataItem.getCallDate(context)
            setTextColor(if (dataItem.isCallDateColored) Color.RED else resources.getColor(R.color.gray600))
        }
        viewHolder.callDuration.apply {
            text = dataItem.getCallDuration(context)
            setVisible(dataItem.isCallDurationVisible)
        }
        viewHolder.callForwardedInfo.apply {
            text = dataItem.getCallForwardedInfo(context)
            setVisible(dataItem.isCallForwardedInfoVisible)
        }
    }

    private fun updateContentRecordViewHolder(viewHolder: CallLogListAdapter.ContentRecordViewHolder, dataItem: CallLogContentItem) {
        viewHolder.callTypeIcon.setImageResource(dataItem.getCallTypeIconResId())
        viewHolder.callInfo.setText(dataItem.getCallInfo())
        viewHolder.callDate.apply { text = dataItem.getCallDate(context) }
        viewHolder.callDuration.apply {
            text = dataItem.getCallDuration(context)
            setVisible(dataItem.isCallDurationVisible)
        }
        viewHolder.recordInfo.apply { text = dataItem.getRecordInfo(context) }
        viewHolder.saveButton.setVisible(dataItem.isSaveButtonVisible)

        onItemDeleteRecordButtonClickListener?.let { viewHolder.deleteButton.setOnClickListener { view -> it.invoke(view, viewHolder.adapterPosition) } }
        onItemSaveRecordButtonClickListener?.let { viewHolder.saveButton.setOnClickListener { view -> it.invoke(view, viewHolder.adapterPosition) } }
        onItemShareRecordButtonClickListener?.let { viewHolder.shareButton.setOnClickListener { view -> it.invoke(view, viewHolder.adapterPosition) } }
        onItemPlayRecordButtonClickListener?.let { viewHolder.playButton.setOnClickListener { view -> it.invoke(view, viewHolder.adapterPosition) } }
    }
}
