package com.bria.voip.ui.main.calllog.refactoring.genband

import android.os.Bundle
import com.bria.common.controller.contacts.genband.GenbandContactDataConversion
import com.bria.common.controller.settings.EAccountSetting
import com.bria.common.util.GlobalConstants
import com.bria.voip.ui.main.calllog.refactoring.CallLogListPresenter
import com.bria.voip.ui.main.calllog.refactoring.getNumberForContacts
import com.bria.voip.ui.main.calllog.refactoring.isLocalContactExists

class GenbandCallLogListPresenter: CallLogListPresenter() {
    override fun isMenuAddContactEnabled(position: Int): Boolean {
        val group = getGroupItemAt(position) ?: return false
        // no need to check isAnonymous for Genband here, don't know why, it just port of old code
        return group.getNumberForContacts().isNotEmpty() && !group.isLocalContactExists()
    }

    override fun isMenuSendImEnabled(position: Int) = !isMeetMeConference(position) && super.isMenuSendImEnabled(position)

    override fun isMenuVideoCallEnabled(position: Int) = !isMeetMeConference(position) && super.isMenuVideoCallEnabled(position)

    override fun isMenuTransferCallEnabled(position: Int) =
            mControllers.accounts.primaryAccount?.getBool(EAccountSetting.GenbandAccDisableCallTransfer) != true && super.isMenuTransferCallEnabled(position)

    override fun onMenuViewContact(position: Int) {
        val group = getGroupItemAt(position) ?: return

        if (!group.isLocalContactExists()) {
            val genbandKey = group.externalId
            if (genbandKey.isNotEmpty()) {
                val accountNickname = mControllers.accounts.getAccount(group.accountId)?.getStr(EAccountSetting.Nickname) ?: ""
                val friend = mControllers.genbandContact.getContactByNickname(genbandKey, accountNickname)
                if (friend != null) {
                    val contactId = GenbandContactDataConversion.getFriendsContact(friend).id
                    val bundle = Bundle(2)
                    bundle.putSerializable(GlobalConstants.KEY_CONTACT_TYPE, GlobalConstants.EContactsFilterType.CONTACTS)
                    bundle.putString(GlobalConstants.KEY_CONTACT_ID, contactId)

                    firePresenterEvent(Events.SHOW_CONTACT_DETAILS, bundle)
                    return
                }
            }
        }

        super.onMenuViewContact(position)
    }

    private fun isMeetMeConference(position: Int) : Boolean {
        val group = getGroupItemAt(position) ?: return false
        val account = mControllers.accounts.getAccount(group.accountId) ?: return false
        return group.number.equals(account.getStr(EAccountSetting.GenbandAccMeetMeConferenceSipUri), ignoreCase = true)
    }
}
