package com.bria.voip.ui.main.contacts

import com.bria.common.controller.Controllers
import com.bria.common.controller.buddy.BuddyAvailability
import com.bria.common.controller.contacts.buddy.Buddy
import com.bria.common.controller.contacts.buddy.IBuddyCtrlEvents
import com.bria.common.controller.contacts.buddy.IBuddyCtrlObserver
import com.bria.common.teams.TeamsModule
import com.bria.common.uireusable.dataprovider.AbstractFilterableListDataProvider
import com.bria.common.uireusable.dataprovider.FilterQueryMatcher
import com.bria.common.uireusable.dataprovider.IFilterableDataProvider
import com.bria.common.util.IObservable

class BriaXContactsListTeamBuddiesDataProvider(
        private val buddyCtrl: IBuddyCtrlEvents,
        private val buddyCtrlObservable: IObservable<IBuddyCtrlObserver>,
        private val teamsModule: TeamsModule
) : AbstractFilterableListDataProvider<Buddy>() {

    private var mBuddyCtrlObserver = object : IBuddyCtrlObserver {
        override fun onBuddyListUpdated() {
            refilter()
        }

        override fun onBuddyPresenceChanged(buddy: Buddy?) {
            refilter()
        }

        override fun onBuddyFilterChanged(aBuddyFilterType: IBuddyCtrlEvents.EBuddyFilterType?) {
            refilter()
        }
    }

    override fun attachWeakListener(dataProviderListener: IFilterableDataProvider.DataProviderListener) {
        super.attachWeakListener(dataProviderListener)

        refilter()
        buddyCtrlObservable.attachWeakObserver(mBuddyCtrlObserver)
    }

    override fun detachWeakListener(dataProviderListener: IFilterableDataProvider.DataProviderListener) {
        super.detachWeakListener(dataProviderListener)

        buddyCtrlObservable.detachObserver(mBuddyCtrlObserver)
    }

    override fun provideFullDataSet(): List<Buddy> {
        val online = Controllers.get().buddy.buddyFilterType == IBuddyCtrlEvents.EBuddyFilterType.ONLINE
        val buddyAvailability = if (online) BuddyAvailability.Online else BuddyAvailability.All

        return buddyCtrl
            .getListOfBuddiesSorted(Buddy.EBuddyType.ALL, buddyAvailability)
            .filter { teamsModule.isBuddyFromTeam(it) }
    }

    override fun provideFilteredDataSet(fullDataSet: List<Buddy>, query: String): List<Buddy> {
        val filterQueryMatcher = FilterQueryMatcher(query)

        return fullDataSet
            .filter { filterQueryMatcher.isMatch(it.firstName, it.lastName) }
            .toMutableList()
    }
}
