package com.bria.voip.ui.main.contacts

import android.os.Bundle
import com.bria.common.controller.contacts.buddy.BuddyRequest
import com.bria.common.controller.contacts.buddy.IBuddyRequest
import com.bria.common.uiframework.lists.iconized.IconizedListItem
import com.bria.common.uiframework.presenters.AbstractPresenter
import com.bria.common.uiframework.presenters.IPresenterEventTypeEnum
import com.bria.common.uireusable.dataprovider.ISimpleDataProvider
import com.bria.common.util.GlobalConstants
import com.bria.voip.R
import java.util.*


class BuddyRequestActionPresenter() : AbstractPresenter() {


    enum class Events : IPresenterEventTypeEnum {
        UPDATE_TITLE;


        override fun getType(): IPresenterEventTypeEnum {
            return this
        }
    }

    private val mRequestActionList = LinkedList<IconizedListItem>()
    private lateinit var buddyRequest: BuddyRequest

    private val mRequestActionDataProvider = object : ISimpleDataProvider<IconizedListItem> {
        override fun getItemsCount(): Int {
            return mRequestActionList.size
        }

        override fun getItemAt(position: Int): IconizedListItem {
            return mRequestActionList.get(position)
        }

        override fun clean() {
            mRequestActionList.clear()
        }
    }

    internal fun getRequestActionDataProvider(): ISimpleDataProvider<IconizedListItem> {
        return mRequestActionDataProvider
    }


    fun initData() {
        mRequestActionList.clear()
        mRequestActionList.add(IconizedListItem(getString(R.string.tAccept), R.drawable.check_selected))
        mRequestActionList.add(IconizedListItem(getString(R.string.tIgnore), R.drawable.check_box_unselected))
        mRequestActionList.add(IconizedListItem(getString(R.string.tDecline), R.drawable.ic_offline))
    }


    fun setConfiguration(bundle: Bundle?) {
        buddyRequest = bundle?.getSerializable(GlobalConstants.REQUEST_BUDDY_ID) as BuddyRequest
        val mDisplayName = buddyRequest.displayName
        firePresenterEvent(Events.UPDATE_TITLE, mDisplayName)


    }

    fun handleRequestResponse(request: BuddyRequest, response: IBuddyRequest.EXmppResponseType) {
        mControllers.presence.handleXmppResponse(request, response)

    }

    fun getBuddyRequest(): BuddyRequest = buddyRequest


}
