package com.bria.voip.ui.main.contacts

import android.os.Bundle
import android.support.v7.widget.LinearLayoutManager
import android.support.v7.widget.RecyclerView
import android.view.View
import android.widget.TextView
import com.bria.common.controller.contacts.buddy.IBuddyRequest.EXmppResponseType
import com.bria.common.uiframework.annotations.Inject
import com.bria.common.uiframework.annotations.Layout
import com.bria.common.uiframework.lists.iconized.IconizedListAdapter
import com.bria.common.uiframework.lists.recycler.OnRecyclerItemClickListener
import com.bria.common.uiframework.presenters.IPresenterEvent
import com.bria.common.uiframework.screens.AbstractScreen
import com.bria.voip.R

@Layout(R.layout.buddy_request_action_layout)
class BuddyRequestActionScreen() : AbstractScreen<BuddyRequestActionPresenter>(), OnRecyclerItemClickListener {

    private lateinit var ACCEPT: String
    private lateinit var DECLINE: String
    private lateinit var IGNORE: String

    @Inject(id = R.id.buddy_request_action_recycler, col = false)
    protected var mRecyclerView: RecyclerView? = null

    @Inject(R.id.dialog_screen_title)
    private val mTitleView: TextView? = null


    private var mAdapter: IconizedListAdapter? = null
    override fun getTitleView(): TextView? {
        return mTitleView
    }

    override fun getPresenterClass(): Class<out BuddyRequestActionPresenter> {
        return BuddyRequestActionPresenter::class.java
    }

    private var mDisplayName: String = ""

    override fun onPresenterEvent(event: IPresenterEvent<*, *>) {
        if (event.type is BuddyRequestActionPresenter.Events) {
            val type = event.type as BuddyRequestActionPresenter.Events
            when (type) {
                BuddyRequestActionPresenter.Events.UPDATE_TITLE -> {
                    mDisplayName = event.data as String
                    mTitleView?.setText(mDisplayName)
                }

            }


        }
    }

    override fun getTitle(): String? {
        return mDisplayName
    }


    override fun onDestroy(finishing: Boolean) {
        super.onDestroy(finishing)
        presenter.unsubscribe()
    }

    private fun setupAdapter() {
        mAdapter = IconizedListAdapter(mRecyclerView)
        mAdapter!!.setDataProvider(presenter.getRequestActionDataProvider())
        mAdapter!!.setShowIcons(true)
        mAdapter!!.setOnItemClickListener(this)
        mRecyclerView?.setLayoutManager(LinearLayoutManager(activity))
        mRecyclerView?.setAdapter(mAdapter)
    }

    override fun onCreate(bundle: Bundle?) {
        super.onCreate(bundle)
        presenter.subscribe(this)
        presenter.setConfiguration(bundle)
        presenter.initData()
        setupAdapter()
        initStringsValue()

    }


    override fun onItemClick(item: View?, index: Int) {
        val actionId = presenter.getRequestActionDataProvider().getItemAt(index).name
        val buddyRequest = presenter.getBuddyRequest()
        when (actionId) {
            ACCEPT -> getPresenter().handleRequestResponse(buddyRequest, EXmppResponseType.Accept);
            DECLINE -> getPresenter().handleRequestResponse(buddyRequest, EXmppResponseType.Block);
            IGNORE -> getPresenter().handleRequestResponse(buddyRequest, EXmppResponseType.Ignore);

        }
        dismissScreenHolderDialog()

    }

    fun initStringsValue() {
        ACCEPT = getString(R.string.tAccept)
        DECLINE = getString(R.string.tDecline)
        IGNORE = getString(R.string.tIgnore)
    }

}
