package com.bria.voip.ui.main.contacts

import android.os.Bundle
import com.bria.common.R
import com.bria.common.controller.contacts.buddy.XmppBuddy
import com.bria.common.controller.coordinatedevents.CoordinatedEventType
import com.bria.common.modules.BriaGraph
import com.bria.common.uiframework.presenters.AbstractPresenter
import com.bria.common.util.GlobalConstants

class ChooseNumberOfXmppBuddyPresenter : AbstractPresenter() {

    companion object {
        const val TAG = "ChooseNumberOfXmppBuddyPresenter"

        fun getNumbers(buddy: XmppBuddy): List<String> {
            val result = mutableListOf<String>()

            buddy.vCard?.phoneList?.map { it.number }?.let {
                result.addAll(it)
            }

            val softphone = buddy.vCard?.softphone
            if (!softphone.isNullOrBlank()) {
                result.add(softphone!!)
            }

            return result
        }
    }

    // dependencies
    private val buddyController
        get() = BriaGraph.buddyCtrl

    private val crashlytics
        get() = BriaGraph.crashlytics

    private val coordinatedEventAggregatorPublishSide
        get() = BriaGraph.coordinatedEventAggregatorPublishSide


    var buddyKey = ""
    var numbers = listOf<String>()

    fun init(xmppChatParticipantKey: String?): Boolean {
        if (xmppChatParticipantKey == null) {
            crashlytics.logContextInfo(TAG, "xmppChatParticipantKey is null")
            return false
        }

        val xmppBuddy = buddyController.getBuddyByNewKey(xmppChatParticipantKey) as? XmppBuddy
        if (xmppBuddy == null) {
            crashlytics.logContextInfo(TAG, "buddy not found, key: $xmppChatParticipantKey")
            return false
        }

        this.buddyKey = xmppChatParticipantKey
        numbers = getNumbers(xmppBuddy)


        if (numbers.isEmpty()) {
            crashlytics.logContextInfo(TAG, "no numbers for buddy")

            val bundle = Bundle()
            bundle.putString(GlobalConstants.SHOW_TOAST_KEY, getString(R.string.tBuddyHasNoNumbers))
            // using global toast as we will navigate away immediately
            coordinatedEventAggregatorPublishSide.sendCoordinatedEvent(CoordinatedEventType.SHOW_TOAST_EVENT, bundle)

            return false
        }

        return true
    }
}
