package com.bria.voip.ui.main.contacts

import android.os.Bundle
import android.view.View
import android.view.ViewGroup
import android.widget.LinearLayout
import android.widget.TextView
import com.bria.common.uiframework.annotations.Inject
import com.bria.common.uiframework.annotations.Layout
import com.bria.common.uiframework.presenters.IPresenterEvent
import com.bria.common.uiframework.screens.AbstractScreen
import com.bria.common.util.CrashInDebug
import com.bria.voip.R

@Layout(R.layout.im_xmpp_number_chooser)
class ChooseNumberOfXmppBuddyScreen : AbstractScreen<ChooseNumberOfXmppBuddyPresenter>() {

    companion object {
        const val TAG = "ChooseNumberOfXmppBuddyScreen"

        // incoming params
        const val IN_BUDDY_KEY = "IN_BUDDY_KEY"

        // result
        const val RESULT_SELECTED_NUMBER = "RESULT_SELECTED_NUMBER"
        const val RESULT_BUDDY_KEY = "RESULT_BUDDY_KEY"
    }

    @Inject(R.id.xmpp_buddy_number_chooser_linear_layout)
    private lateinit var mLinearLayout: LinearLayout

    override fun getPresenterClass(): Class<out ChooseNumberOfXmppBuddyPresenter> {
        return ChooseNumberOfXmppBuddyPresenter::class.java
    }

    override fun getTitle(): String? {
        return null
    }

    override fun onPresenterEvent(event: IPresenterEvent<*, *>) {

    }

    override fun onCreate(bundle: Bundle?) {
        super.onCreate(bundle)

        if (bundle != null) {
            // after rotate we will not get the key and need to rely on data in presenter
            if (bundle.containsKey(IN_BUDDY_KEY)) {
                val xmppChatParticipantKey = bundle.getString(IN_BUDDY_KEY)
                if (!presenter.init(xmppChatParticipantKey)) {
                    abortScreenCreation()
                    CrashInDebug.with(TAG, "init presenter failed")
                }
            }
        }

        for (number in presenter.numbers) {
            // create text item view
            val itemView = View.inflate(activity, com.bria.common.R.layout.dialog_list_item, null) as TextView
            itemView.text = number

            // set click listener
            itemView.setOnClickListener {
                val bundle = Bundle().apply {
                    putString(RESULT_BUDDY_KEY, presenter.buddyKey)
                    putString(RESULT_SELECTED_NUMBER, number)
                }

                publishResult(bundle)
                dismissScreenHolderDialog()
            }

            // add them to the layout (TextView has intrinsic layout params)
            mLinearLayout.addView(itemView)


            // create a divider
            val divider = View(activity)
            val dividerH = activity.resources.getDimensionPixelSize(com.bria.common.R.dimen.divider_height)
            val params = LinearLayout.LayoutParams(ViewGroup.LayoutParams.MATCH_PARENT, dividerH)
            divider.setBackgroundColor(activity.resources.getColor(R.color.primary_light))

            mLinearLayout.addView(divider, params)
        }
    }
}
