package com.bria.voip.ui.main.contacts

import android.annotation.SuppressLint
import android.os.Bundle
import com.bria.common.R
import com.bria.common.analytics.BIAnalytics
import com.bria.common.controller.accounts.Account
import com.bria.common.controller.accounts.AccountsFilter
import com.bria.common.controller.buddy.BuddyAvailability
import com.bria.common.controller.contacts.buddy.BuddyRequest
import com.bria.common.controller.contacts.buddy.SipBuddy
import com.bria.common.controller.contacts.buddy.XmppBuddy
import com.bria.common.controller.contacts.ldap.ILdapContactCtrlObserver
import com.bria.common.controller.contacts.local.favorites.FavoriteDataItem
import com.bria.common.controller.im.ImpsUtils
import com.bria.common.controller.presence.IPresenceCtrlObserver
import com.bria.common.controller.presence.IPresenceMatcher
import com.bria.common.controller.presence.OwnPresence
import com.bria.common.controller.presence.refactoring.EPresenceStatus
import com.bria.common.modules.BriaGraph
import com.bria.common.sdkwrapper.CallData
import com.bria.common.uiframework.helpers.ContactTabManager
import com.bria.common.uiframework.presenters.AbstractPresenter
import com.bria.common.uiframework.presenters.IPresenterEventTypeEnum
import com.bria.common.uireusable.dataprovider.IContactActionDataProvider
import com.bria.common.uireusable.dataprovider.IExtendedFilterableDataProvider
import com.bria.common.uireusable.dataprovider.IFilterableDataProvider
import com.bria.common.uireusable.datatypes.ContactTabItem
import com.bria.common.uireusable.datatypes.IPersonListItem
import com.bria.common.util.GlobalConstants
import com.bria.common.util.GlobalConstants.*
import com.bria.common.util.GlobalConstants.EContactsFilterType.*
import java.util.*


open class V2ContactListPresenter : AbstractPresenter(), ILdapContactCtrlObserver, IPresenceCtrlObserver {

    private val TAG = "ContactListPresenter"
    private var permissionRequestSource: EPermissionRequestSource = EPermissionRequestSource.UNKNOWN


    private lateinit var mContactTabManager: ContactTabManager


    enum class EPermissionRequestSource {
        DELETE_DIALOG,
        ADD_PERSON,
        UNKNOWN;
    }

    //FIXME LDAP
    override fun onDirectoryContactListUpdated() {
    }

    protected var mActiveItemPosition = ITEM_NOT_SELECTED_ID

    enum class Events : IPresenterEventTypeEnum {
        FILTER_TYPE_CHANGED,
        LIST_UPDATED,
        ITEM_UPDATED,
        REMOVED_ACTIVE_ITEM,
        SHOW_TOAST,
        FAVORITE_CALL_OPTION_DIALOG,
        SHOW_CONTACT_EDIT_SCREEN,
        SHOW_BUDDY_EDIT_SCREEN,
        SHOW_UNDO_ACTION_FOR_REMOVE,
        SETUP_VISIBILITY,
        SETUP_SPINER,
        SHOW_DETAILS_SCREEN;


        override fun getType(): IPresenterEventTypeEnum {
            return this
        }
    }


    fun getSearchQuery(): String {
        return mContactTabManager.getActiveTab().lastSearchString
    }


    override fun onCreate() {

        super.onCreate()
        mContactTabManager = ContactTabManager(
                context,
                mControllers.buddy,
                mObservables.buddy,
                mControllers.contacts.favoritesApi,
                mControllers.contacts.contactsApi.observable,
                mControllers.contacts.contactsApi,
                mObservables.presence,
                mControllers.presence)
        mObservables.presence.attachWeakObserver(this)
    }

    override fun onDestroy() {
        super.onDestroy()
        mObservables.presence.detachObserver(this)
    }

    fun getAllTabs(): ArrayList<ContactTabItem> {
        return mContactTabManager.getTabList()!!
    }


    private val iExtendedFilterableDataProvider = object : IExtendedFilterableDataProvider {

        override fun onDataLoading() {}

        override fun onDataLoaded(isEmpty: Boolean) {
            firePresenterEvent(Events.LIST_UPDATED)
            firePresenterEvent(Events.SETUP_VISIBILITY)
            if (isRequestSubFilter()) {
                firePresenterEvent(Events.SETUP_SPINER)
            }
        }

        override fun updatedItem(position: Int?) {
            firePresenterEvent(Events.ITEM_UPDATED, position, 0)
        }

        override fun addedItem() {}

        override fun removedItem(position: Int?) {
            firePresenterEvent(Events.REMOVED_ACTIVE_ITEM, position, 0)
        }
    }

    fun initDataProviders() {
        mContactTabManager.getTabList()?.forEach { t ->
            run {
                val dataProvider = t.dataProvider
                dataProvider.attachWeakListenerV2(iExtendedFilterableDataProvider)
                dataProvider.initializeLoadingData()
            }
        }

    }


    fun getDataProvider(): IContactActionDataProvider<out Any>? {
        return mContactTabManager.getActiveDataProvider()
    }

    fun isDataProviderLoading(): Boolean {
        val dataProvider = mContactTabManager.getActiveDataProvider() as IFilterableDataProvider<*>
        return dataProvider.isDataLoading
    }

    fun isDataProviderEmpty(): Boolean {
        return mContactTabManager.getActiveDataProvider().itemsCount == 0
    }


    fun viewDetailsScreen() {
        if (mContactTabManager.getActiveTab().tagName == FAVORITES) handleFavoriteView() else showDetailsScreen()
    }

    fun isRequestSubFilter(): Boolean = mContactTabManager.getSubFilterType() == mContactTabManager.SUBTYPE_REQUESTS && mContactTabManager.getActiveTab().tagName == BUDDIES


    protected fun showDetailsScreen() {
        firePresenterEvent(Events.SHOW_DETAILS_SCREEN, getActiveItemData())
    }


    protected fun getActiveContactId(): String {
        val iPersonLisItem = mContactTabManager.getActiveDataProvider().getItemAt(mActiveItemPosition)
        return iPersonLisItem.uniqueIdentifier
    }

    fun getActiveItemData(): Bundle {
        val activeItemData = Bundle()
        activeItemData.putSerializable(KEY_CONTACT_TYPE, mContactTabManager.getActiveTab().tagName)

        when (mContactTabManager.getActiveTab().tagName) {
            CONTACTS -> activeItemData.putString(KEY_CONTACT_ID, getActiveContactId())
            BUDDIES -> {
                if (isRequestSubFilter()) {
                    activeItemData.putSerializable(GlobalConstants.REQUEST_BUDDY_ID, mContactTabManager.getBuddyReqestDataProvider().getItemAt(mActiveItemPosition) as BuddyRequest)

                }
                val buddy = IPersonListItem::class.java.cast(mContactTabManager.getActiveDataProvider().getItemAt(mActiveItemPosition))
                if (buddy is XmppBuddy) {
                    activeItemData.putString(GlobalConstants.KEY_CONTACT_SUB_TYPE, GlobalConstants.BUDDY_SUBTYPE_XMPP)
                    activeItemData.putString(GlobalConstants.KEY_BUDDY_ID, ImpsUtils.getBuddyKey(buddy))
                } else if (buddy is SipBuddy) {
                    activeItemData.putString(GlobalConstants.KEY_CONTACT_SUB_TYPE, GlobalConstants.BUDDY_SUBTYPE_SIP)
                    activeItemData.putString(GlobalConstants.KEY_CONTACT_ID, buddy.contactId)
                }
            }
            FAVORITES -> activeItemData.putSerializable(FAVORITE_DATA_ID, mContactTabManager.getActiveDataProvider().getItemAt(mActiveItemPosition) as FavoriteDataItem)
            LDAP -> activeItemData.putString(GlobalConstants.KEY_CONTACT_ID, mActiveItemPosition.toString())
        }
        return activeItemData
    }


    @SuppressLint("SwitchIntDef")
    fun handleFavoriteView() {
        val favorite = getActiveItemData().getSerializable(GlobalConstants.FAVORITE_DATA_ID) as FavoriteDataItem
        when (favorite.preferredAction) {
            FavoriteDataItem.ACTION_TYPE_CALL -> {

                directlyCallFavorite(favorite, favorite.preferredAddress!!)
            }
            FavoriteDataItem.ACTION_TYPE_NONE -> {
                // no preferred action
                offloadLightWork { scheduleFavoriteCallOptionsPopup(favorite) }
            }
            else -> {
                offloadLightWork { scheduleFavoriteCallOptionsPopup(favorite) }
            }
        }
    }

    private fun scheduleFavoriteCallOptionsPopup(favorite: FavoriteDataItem) {
        val numbers = mControllers.contacts.favoritesApi.loadPhoneNumbersBlocking(favorite)

        // check if displaying multiple numbers is possible
        if (numbers.isEmpty()) {
            // no numbers? ... good API
            firePresenterEvent(Events.SHOW_TOAST, getString(R.string.tFavoriteNoNumbers))
            return
        } else if (numbers.size == 1) {
            // only one number, just call it

            directlyCallFavorite(favorite, numbers.get(0).secondaryText!!)
            return
        }

        val bundle = Bundle(2)
        bundle.putParcelable(ContactsListScreen.KEY_FAVORITE_ITEM, favorite)
        bundle.putParcelableArrayList(ContactsListScreen.KEY_FAVORITE_NUMBERS, numbers)
        firePresenterEvent(Events.FAVORITE_CALL_OPTION_DIALOG, bundle)
    }

    fun updateFavoriteCallChoice(favorite: FavoriteDataItem, number: String?, rememberChoice: Boolean) {
        mControllers.contacts.favoritesApi.updateFavoriteCallChoice(favorite, number, rememberChoice)
    }


    fun directlyCallFavorite(favorite: FavoriteDataItem, number: String) {
        BIAnalytics.get().reportUIClick("dial-from-Favorites-listing")
        if (!mControllers.phone.call(number, null, favorite.name, CallData.ECallType.Generic)) {
            firePresenterEvent(Events.SHOW_TOAST, mControllers.phone.lastError.description)
        }
    }


    fun changedFilterTab(activeTab: GlobalConstants.EContactsFilterType) {
        setActiveTab(activeTab)
        firePresenterEvent(Events.FILTER_TYPE_CHANGED)
    }

    fun getLoadingString(): String {
        return mContactTabManager.getActiveTab().loadingString
    }

    fun getErrorDataString(): String {
        return mContactTabManager.getActiveTab().errorDataString
    }

    fun setActiveTab(activeTab: GlobalConstants.EContactsFilterType) {
        mContactTabManager.setActiveTab(activeTab)
    }

    fun getActiveTab(): ContactTabItem {
        return mContactTabManager.getActiveTab()
    }

    fun setActiveItemPosition(position: Int) {
        mActiveItemPosition = position
    }

    fun viewEditScreen() {
        val editBundle = getActiveItemData()
        if (IPersonListItem::class.java.cast(mContactTabManager.getActiveDataProvider().getItemAt(mActiveItemPosition)) is XmppBuddy) {
            firePresenterEvent(Events.SHOW_BUDDY_EDIT_SCREEN, editBundle)
        } else {
            editBundle.putString(ContactEditPresenter.KEY_SCREEN_ACTION, ContactEditPresenter.ACTION_EDIT)
            firePresenterEvent(Events.SHOW_CONTACT_EDIT_SCREEN, editBundle)
        }
    }

    @SuppressLint("RxLeakedSubscription", "RxSubscribeOnError")
    fun deleteActiveItem() {
        mControllers.contacts.favoritesApi.calculateIsContactFavorite(getActiveContactId())
                .doOnError { throwable: Any -> mContactTabManager.getActiveDataProvider().removeActiveItem(mActiveItemPosition) }
                .subscribe { favoriteDataItem ->
                    firePresenterEvent(Events.SHOW_UNDO_ACTION_FOR_REMOVE, favoriteDataItem)
                    mControllers.contacts.favoritesApi.removeFavorite(favoriteDataItem)
                }


        /*if (calculateIsContactFavorite) {
            firePresenterEvent(Events.SHOW_UNDO_ACTION_FOR_REMOVE)
            mControllers.contacts.favoritesApi.removeFavorite(getFavoriteItem()!!)
            return
        }
        getDataProvider()?.removeActiveItem(mActiveItemPosition)*/
    }

/*    fun getFavoriteItem(): FavoriteDataItem? {
        val favoriteDataItem: FavoriteDataItem?
        if (mContactTabManager.getActiveTab().tagName == FAVORITES) {
            favoriteDataItem = getDataProvider()?.getItemAt(mActiveItemPosition) as FavoriteDataItem
        } else {
            val contactId = getActiveContactId()
            val intId = if (TextUtils.isEmpty(contactId)) -1 else Integer.valueOf(contactId)
            favoriteDataItem = mControllers.contacts.favoritesApi.getFavorite(intId)

        }
        return favoriteDataItem
    }*/

    fun undoFavoriteRemove(favorite: FavoriteDataItem) {
        mControllers.contacts.favoritesApi.addFavorite(favorite)
    }

    fun isContactFavorite(): Boolean {
        return mControllers.contacts.favoritesApi.isContactFavorite(getActiveContactId())

    }

    fun getPresenceMatcher(): IPresenceMatcher? {
        return mContactTabManager.getActiveTab().presenceMatcher
    }

    fun setSearchQuery(searchString: String) {
        mContactTabManager.getActiveTab().lastSearchString = searchString


    }

    fun initSearch(queryForSearch: String) {
        mContactTabManager.getActiveDataProvider().filterDataProvider(queryForSearch)

    }

    fun setSubFilterType(position: Int) {
        val lastSubFilterType = mContactTabManager.getSubFilterType()
        mContactTabManager.setSubFilterType(position)

        if (position == mContactTabManager.SUBTYPE_ALL || position == mContactTabManager.SUBTYPE_ONLINE) {
            mContactTabManager.getActiveDataProvider().filterDataProvider(switcherAvailabiity(position))
            if (lastSubFilterType == mContactTabManager.SUBTYPE_REQUESTS) {
                firePresenterEvent(Events.FILTER_TYPE_CHANGED)
            }
        } else {
            firePresenterEvent(Events.FILTER_TYPE_CHANGED)
        }
    }

    fun switcherAvailabiity(position: Int): BuddyAvailability {
        when (position) {
            mContactTabManager.SUBTYPE_ALL -> return BuddyAvailability.All
            mContactTabManager.SUBTYPE_ONLINE -> return BuddyAvailability.Online
        }
        return BuddyAvailability.All

    }


    fun getSubFilterType(): Int = mContactTabManager.getSubFilterType()

    fun isRequestAvailable(): Boolean = mContactTabManager.getBuddyReqestDataProvider().itemsCount > 0

    override fun onRemoteEndPresenceChanged(accId: String?, remoteAddr: String?, presenceStatus: EPresenceStatus?, presenceMsg: String?) {
    }

    override fun onPresenceUpdate(presence: OwnPresence?, previousPresence: OwnPresence?) {
    }

    override fun onSubscriptionRequest() {
    }

    override fun onBuddyRequestNumberChanged(newRequestCount: Int, allRequestsCount: Int) {
        if (newRequestCount > 0) {
            mContactTabManager.setActiveTab(GlobalConstants.EContactsFilterType.BUDDIES)
            mContactTabManager.setSubFilterType(mContactTabManager.SUBTYPE_REQUESTS)
        }
    }


    fun areRequestsButtonsVisible(): Boolean = isRequestSubFilter() && mContactTabManager.getBuddyReqestDataProvider().itemsCount > 1
    fun acceptAllBuddyRequests() = mControllers.presence.acceptAllBuddyRequests()
    fun declineAllBuddyRequests() = mControllers.presence.declineAllBuddyRequests()
    fun getAllBuddyAddableAccounts(): List<Account> = mControllers.accounts.accountsProvider.getAccounts(AccountsFilter.ACTIVE_IM_NO_HARDWIRED)
    fun isPresenceStatusOffline() = mControllers.presence.presence != null && mControllers.presence.presence!!.isStatusOffline

    fun getPermisssionRequestSoruce() = permissionRequestSource

    fun setPermissionRequestSource(state: EPermissionRequestSource) {
        permissionRequestSource = state
    }

    fun addContactToFavorite() {
        val contactToAdd = mControllers.contacts
                .contactsApi
                .contact
                .byId(getActiveContactId())
                .asMaybe()
                .blockingGet() ?: return // blocking is temporary for now

        // add favorite to the list
        val favorite = FavoriteDataItem.builder(contactToAdd.intId, contactToAdd.displayName ?: "")
                .setImageUri(contactToAdd.photoUri)
                .build()

        if (mControllers.contacts.favoritesApi.addFavorite(favorite)) {
            firePresenterEvent(Events.SHOW_TOAST, getString(R.string.tFavoriteAddSuccess))
        }

    }

    fun stopDirectoryProcess() = BriaGraph.ldapModule?.disconnect()

    fun startDirectoryProcess() = BriaGraph.ldapModule?.connect()


}
