package com.bria.voip.ui.main.contacts

import android.Manifest
import android.app.Dialog
import android.content.pm.PackageManager
import android.os.Bundle
import android.support.annotation.MainThread
import android.support.design.widget.TabLayout
import android.support.v7.app.AlertDialog
import android.support.v7.app.AppCompatDialog
import android.support.v7.widget.AppCompatSpinner
import android.support.v7.widget.LinearLayoutManager
import android.support.v7.widget.RecyclerView
import android.text.TextUtils
import android.view.Gravity
import android.view.Menu
import android.view.MenuItem
import android.view.View
import android.widget.AdapterView
import android.widget.CheckBox
import android.widget.LinearLayout
import android.widget.TextView
import com.bria.common.R.string
import com.bria.common.controller.accounts.Account
import com.bria.common.controller.contacts.local.favorites.FavoriteDataItem
import com.bria.common.controller.settings.EAccountSetting
import com.bria.common.controller.settings.ESetting
import com.bria.common.controller.settings.branding.EAccountType
import com.bria.common.customelements.internal.AnimationUtils
import com.bria.common.customelements.internal.SpinnerUtils
import com.bria.common.customelements.internal.views.indexer.IndexLettersView
import com.bria.common.customelements.internal.views.undo.UndoActionView
import com.bria.common.permission.PermissionRequestCode
import com.bria.common.uiframework.annotations.Clickable
import com.bria.common.uiframework.annotations.Inject
import com.bria.common.uiframework.annotations.Layout
import com.bria.common.uiframework.branding.AbstractCustomizer
import com.bria.common.uiframework.coloring.Coloring
import com.bria.common.uiframework.dialogs.ScreenHolderDialog
import com.bria.common.uiframework.lists.array.ArrayListAdapter
import com.bria.common.uiframework.lists.array.ArrayListItem
import com.bria.common.uiframework.lists.recycler.OnRecyclerItemClickListener
import com.bria.common.uiframework.lists.recycler.OnRecyclerItemLongClickListener
import com.bria.common.uiframework.presenters.IPresenterEvent
import com.bria.common.uiframework.screens.AbstractScreen
import com.bria.common.uiframework.screens.IScreenEnum
import com.bria.common.uireusable.CustomSearchView
import com.bria.common.uireusable.adapters.GenericPersonAdapter
import com.bria.common.uireusable.dataprovider.ISimpleDataProvider
import com.bria.common.uireusable.datatypes.IPersonListItem
import com.bria.common.util.GlobalConstants
import com.bria.common.util.GlobalConstants.EContactsFilterType.*
import com.bria.common.util.LoaderFactory
import com.bria.common.util.Log
import com.bria.voip.R
import com.bria.voip.ui.main.contacts.ContactsListScreen.KEY_FAVORITE_ITEM
import com.bria.voip.ui.main.contacts.ContactsListScreen.KEY_FAVORITE_NUMBERS
import com.bria.voip.ui.main.contacts.V2ContactListPresenter.EPermissionRequestSource.ADD_PERSON
import com.bria.voip.ui.main.contacts.V2ContactListPresenter.EPermissionRequestSource.DELETE_DIALOG
import com.bria.voip.ui.main.dialer.MultiStateViewHelper
import com.bria.voip.ui.main.misc.EScreenList
import com.bria.voip.ui.main.settings.accountselect.AccountSelectScreen
import com.kennyc.view.MultiStateView

@Layout(R.layout.v2_contact_list_screen)
@com.bria.common.uiframework.annotations.Menu(R.menu.contact_list_menu)
open class V2ContactListScreen<Presenter : V2ContactListPresenter> : AbstractScreen<Presenter>() {
    private val TAG = "ContactsListScreen"
    private val SURE_DELETE_DIALOG_ID = 1
    private val FAVORITE_CALL_DIALOG_ID = 3
    private val CONTACT_EDIT_OVERLAY = 0xCE0002
    private val UNDO_DELAY = 3000
    private val SEARCH_STATE = "CONTACTS_SEARCH_STATE"
    @Inject(id = R.id.v2_contacts_list_screen_search_view, col = false)
    protected var mSearchView: CustomSearchView? = null

    @Inject(id = R.id.screen_toolbar_left_spinner, col = false)
    private lateinit var mSpinner: AppCompatSpinner

    @Inject(id = R.id.v2_contacts_list_screen_tabs)
    private lateinit var mTabWidget: TabLayout

    protected lateinit var mPersonsList: RecyclerView

    @Inject(id = R.id.v2_contacts_list_screen_requests_buttons, col = false)
    private val mBuddyRequestsButtons: LinearLayout? = null

    @Inject(id = R.id.v2_contacts_list_screen_accept_all, back = ESetting.ColorBrandTint, inv = true, tag = AbstractCustomizer.Tag.FCS)
    @Clickable
    private val mButtonAcceptAll: TextView? = null

    @Clickable
    @Inject(id = R.id.v2_contacts_list_screen_decline_all, back = ESetting.ColorBrandTint, inv = true, tag = AbstractCustomizer.Tag.FCS)
    private val mButtonDeclineAll: TextView? = null

    private var mIndexLetterScroller: IndexLettersView? = null

    @Inject(id = R.id.v2_contacts_list_screen_touched_letter, back = ESetting.ColorBrandTint, tag = AbstractCustomizer.Tag.FCS, inv = true)
    private val mTouchedIndexLetter: TextView? = null

    protected lateinit var mAdapter: GenericPersonAdapter

    private val mPersonItemClickListener = PersonOnClickListener()
    private var mActiveItemView: View? = null


    @Suppress("UNCHECKED_CAST")
    override fun getPresenterClass(): Class<Presenter> = V2ContactListPresenter::class.java as Class<Presenter>

    override fun onPresenterEvent(event: IPresenterEvent<*, *>) {

        if (event.type is V2ContactListPresenter.Events) {
            val type = event.type as V2ContactListPresenter.Events
            Log.i(TAG, "Presenter Event: " + type.name)
            when (type) {
                V2ContactListPresenter.Events.SETUP_VISIBILITY -> {
                    setVisibilities()
                }
                V2ContactListPresenter.Events.SETUP_SPINER -> {
                    setupSpinner()
                }
                V2ContactListPresenter.Events.FILTER_TYPE_CHANGED -> {
                    reloadDataScreen()
                    setVisibilities()
                    invalidateMenu()
                    if (presenter.getActiveTab().pullContentFromServer) {
                        presenter.startDirectoryProcess()
                        mSearchView?.open(presenter.getActiveTab().cleanSearchOnClose)
                    }
                }
                V2ContactListPresenter.Events.LIST_UPDATED -> {
                    mAdapter.notifyDataChanged()
                }
                V2ContactListPresenter.Events.ITEM_UPDATED -> {
                    val positions = event.data as Int
                    mAdapter.notifyItemChanged(positions)
                }
                V2ContactListPresenter.Events.REMOVED_ACTIVE_ITEM -> {
                    val positions = event.data as Int
                    mAdapter.notifyItemRemoved(positions)
                }
                V2ContactListPresenter.Events.SHOW_DETAILS_SCREEN -> {
                    if (presenter.getActiveTab().tagName == BUDDIES) {
                        mCoordinator.flow(event.data as Bundle).goTo(EScreenList.BUDDY_DISPLAY)
                    } else {
                        mCoordinator.flow(event.data as Bundle).goTo(EScreenList.CONTACT_DISPLAY)
                    }
                }

                V2ContactListPresenter.Events.SHOW_BUDDY_EDIT_SCREEN -> {
                    mCoordinator.flow(event.data as Bundle).goTo(EScreenList.BUDDY_DISPLAY)
                }
                V2ContactListPresenter.Events.SHOW_CONTACT_EDIT_SCREEN -> {
                    goToContactEditScreen(event.data as Bundle)
                }

                V2ContactListPresenter.Events.FAVORITE_CALL_OPTION_DIALOG -> {
                    showDialog(FAVORITE_CALL_DIALOG_ID, event.data as Bundle, true)

                }
                V2ContactListPresenter.Events.SHOW_UNDO_ACTION_FOR_REMOVE -> {
                    val favorite = event.data as FavoriteDataItem

                    val text = getString(R.string.tFavoriteRemoveSuccess, favorite.name)
                    // noinspection ConstantConditions
                    undoAction!!.setButtonText(R.string.undo_action).setDuration(UNDO_DELAY).setListener(object : UndoActionView.UndoActionAdapter() {
                        override fun onUndoClicked() {
                            presenter.undoFavoriteRemove(favorite)
                        }
                    }).setMessageText(text).show(UndoActionView.POSITION_HIGH)

                }

            }


        }
    }

    fun goToContactEditScreen(bundle: Bundle) {
        if (isInTabletMode) {
            showDialog(CONTACT_EDIT_OVERLAY, bundle)
        } else {
            mCoordinator.flow(bundle).goTo(EScreenList.CONTACT_EDIT)
        }
    }

    private inner class PersonOnClickListener : OnRecyclerItemClickListener, OnRecyclerItemLongClickListener {

        override fun onItemClick(item: View, index: Int) {
            presenter.setActiveItemPosition(index)
            mActiveItemView = item
            if (presenter.isRequestSubFilter()) showOptionsDialog() else presenter.viewDetailsScreen()
        }

        override fun onItemLongClick(item: View, index: Int) {
            if (!isInTabletMode || presenter.getActiveTab().tagName == FAVORITES) {
                presenter.setActiveItemPosition(index)
                mActiveItemView = item
                showOptionsDialog()
            }
        }
    }


    @MainThread
    override fun onClick(v: View) {
        super.onClick(v)
        when (v.id) {
            R.id.v2_contacts_list_screen_accept_all -> presenter.acceptAllBuddyRequests()
            R.id.v2_contacts_list_screen_decline_all -> presenter.declineAllBuddyRequests()
        }
    }

    protected fun showOptionsDialog() {
        if (presenter.getActiveTab().tagName == CONTACTS) {
            mActiveItemView?.let { showPopupMenu(R.menu.contact_list_operations, it, Gravity.CENTER_HORIZONTAL or Gravity.BOTTOM) }
        }

        if (presenter.getActiveTab().tagName == BUDDIES) {
            if (presenter.isRequestSubFilter())
                showScreenForResult(EScreenList.BUDDY_REQUEST_ACTIONS, ScreenHolderDialog.DIALOG_BOTTOM_SHEET, presenter.getActiveItemData())
            else
                mActiveItemView?.let { showPopupMenu(R.menu.buddy_list_operations, it, Gravity.CENTER_HORIZONTAL or Gravity.BOTTOM) }
        }
        if (presenter.getActiveTab().tagName == LDAP) {
            mActiveItemView?.let { showPopupMenu(R.menu.ldap_contact_list_operations, it, Gravity.CENTER_HORIZONTAL or Gravity.BOTTOM) }

        }
        if (presenter.getActiveTab().tagName == FAVORITES) {
            mActiveItemView?.let { showPopupMenu(R.menu.favorite_list_operations, it, Gravity.CENTER_HORIZONTAL or Gravity.BOTTOM) }
        }
    }


    fun reloadDataScreen() {
        setDataProvider()
        mPersonsList.adapter = mAdapter
        mAdapter.setOnItemClickListener(mPersonItemClickListener)
        mAdapter.setOnItemLongClickListener(mPersonItemClickListener)
        mAdapter.notifyDataChanged()
        mAdapter.setPresenceMatcher(presenter.getPresenceMatcher())
    }

    private fun setDataProvider() {
        val dataProvider = presenter.getDataProvider()
        mAdapter.dataProvider = dataProvider as ISimpleDataProvider<IPersonListItem>
    }

    override fun onStart(bundle: Bundle?) {
        super.onStart(bundle)
        presenter.subscribe(this)
    }

    fun saveLayoutState(isScreenFinishing: Boolean) {
        save(SEARCH_STATE, mSearchView?.saveState())
    }

    fun restoreLayoutState() {
        if (restore(SEARCH_STATE) != null) {
            mSearchView?.restoreState(restore(SEARCH_STATE) as Bundle)
        }
    }

    override fun onStop(finishing: Boolean) {
        super.onStop(finishing)
        saveLayoutState(finishing)
        presenter.unsubscribe()
    }

    override fun getTitle(): String? = "NEW CONTACT LIST SCREEN"

    private lateinit var mMultiStateHelper: MultiStateViewHelper

    override fun onCreate(bundle: Bundle?) {
        super.onCreate(bundle)
        mMultiStateHelper = MultiStateViewHelper(layout, activity)
        mPersonsList = mMultiStateHelper.recyclerView
        val mLinearLayoutManager = LinearLayoutManager(activity)
        mPersonsList.layoutManager = mLinearLayoutManager

        updateTabs()
        mAdapter = GenericPersonAdapter(mPersonsList, LoaderFactory.createContactImageLoader(activity), LoaderFactory
                .createPresenceLoader(activity))
        presenter.initDataProviders()
        reloadDataScreen()
        setupSpinner()
        initSearchView()
        restoreLayoutState()

    }


    private fun initSearchView() {
        mSearchView?.setEventHandler(object : CustomSearchView.EventHandler {
            override fun searchFor(searchString: String) {
                presenter.setSearchQuery(searchString)
                presenter.initSearch(searchString)
            }

            override fun onClosed() {
                presenter.setSearchQuery(mSearchView!!.getSearchString())
                showTabWidget()
                if (presenter.getActiveTab().pullContentFromServer) presenter.stopDirectoryProcess()
            }

            override fun onOpened() {
                val searchedString = presenter.getSearchQuery()
                mSearchView!!.setSearchString(searchedString)
                when {
                    presenter.getActiveTab().pullContentFromServer -> {
                        presenter.initSearch(searchedString)
                        setVisibilities()
                    }
                    else -> if (!searchedString.isEmpty()) presenter.initSearch(searchedString)
                }
                hideTabWidget()

            }

            override fun onClear() {
                //presenter.clearDirectoryDataProvider()
            }

        })
        //  setupSearchSettings()
    }

    protected fun hideTabWidget() {
        val bundle = Bundle()
        bundle.putBoolean(AnimationUtils.FLAG_MAKE_INVISIBLE_AFTER_ANIMATION, true)
        AnimationUtils.animateViewUp(mTabWidget, -mTabWidget.getHeight(), bundle)
    }

    protected fun showTabWidget() {
        mTabWidget.setVisibility(View.VISIBLE)
        AnimationUtils.animateViewDown(mTabWidget, -mTabWidget.getHeight())

    }


    private fun updateTabs() {
        mTabWidget.removeAllTabs()
        mTabWidget.removeOnTabSelectedListener(mTabListener)

        presenter.getAllTabs().forEach {
            mTabWidget.addTab(
                    mTabWidget.newTab()
                            .setText(it.textName)
                            .setTag(it.tagName), false) // 'setSelected = true' didn't work properly here because recoloring applied later
        }

        recolorTabs()
        updateSelectedTab() // set initial selection

        mTabWidget.addOnTabSelectedListener(mTabListener)
    }

    private fun recolorTabs() {
        val customizer = Coloring.get().getCustomizer(TabLayout::class.java)
        customizer.setTarget(mTabWidget)
        customizer.applyChanges()
    }

    private fun updateSelectedTab() {
        val activeTab = presenter.getActiveTab()
        for (i in 0 until mTabWidget.tabCount) {
            val tab = mTabWidget.getTabAt(i)
            if (tab?.tag === activeTab.tagName) {
                tab.select()
            }
        }
    }


    private val mTabListener = object : TabLayout.OnTabSelectedListener {
        override fun onTabSelected(tab: TabLayout.Tab) {
            if (tab.tag != null) {
                //TODO Implemented
                // saveLayoutState()
                presenter.changedFilterTab(tab.tag as GlobalConstants.EContactsFilterType)

            }
        }

        override fun onTabUnselected(tab: TabLayout.Tab) {}

        override fun onTabReselected(tab: TabLayout.Tab) {}
    }

    override fun onResume() {
        super.onResume()
        setVisibilities()
    }

    private fun setVisibilities() {
        val activeTab = presenter.getActiveTab()
        if (activeTab.hasSubFilter) mSpinner.visibility = View.VISIBLE else mSpinner.visibility = View.INVISIBLE
        val noPermission = activeTab.hasPermissionForCheck && !checkPermission(Manifest.permission.READ_CONTACTS)

        if (noPermission) {
            mMultiStateHelper.multiStateView.viewState = MultiStateView.VIEW_STATE_ERROR
            mMultiStateHelper.permissionErrorButton.setOnClickListener({
                requestPermission(
                        Manifest.permission.READ_CONTACTS,
                        PermissionRequestCode.CP_PERMISSION_INIT_CONTACTS,
                        getString(R.string.tContactsListPermissionRequestExplanation))
            })
        } else if (presenter.isDataProviderLoading()) {
            mMultiStateHelper.multiStateView.viewState = MultiStateView.VIEW_STATE_LOADING
            mMultiStateHelper.noDataView.setText(presenter.getLoadingString())
        } else if (presenter.isDataProviderEmpty()) {
            mMultiStateHelper.multiStateView.viewState = MultiStateView.VIEW_STATE_EMPTY
            mMultiStateHelper.noDataView.text = presenter.getErrorDataString()
        } else {
            mMultiStateHelper.multiStateView.viewState = MultiStateView.VIEW_STATE_CONTENT

        }
        updateSelectedTab()
        updateSelectedSpinerPosition()
        mBuddyRequestsButtons?.setVisibility(if (presenter.areRequestsButtonsVisible()) View.VISIBLE else View.GONE)

    }


    override fun createDialog(which: Int, data: Bundle?): Dialog? {
        when (which) {
            CONTACT_EDIT_OVERLAY -> return ScreenHolderDialog
                    .builder(activity)
                    .screen(EScreenList.CONTACT_EDIT)
                    .style(ScreenHolderDialog.DIALOG_MAX_SIZE_NO_MARGIN)
                    .bundle(data)
                    .build()
            SURE_DELETE_DIALOG_ID -> {
                return AlertDialog.Builder(activity)
                        .setMessage(R.string.tAreYouSureDialog)
                        .setCancelable(false)
                        .setPositiveButton(R.string.tYes) { d, w -> presenter.deleteActiveItem() }
                        .setNegativeButton(R.string.tNo) { d, w -> }
                        .create()
            }


            FAVORITE_CALL_DIALOG_ID -> {


                val favorite = data?.getParcelable<FavoriteDataItem>(KEY_FAVORITE_ITEM)
                val numbers = data?.getParcelableArrayList<ArrayListItem>(KEY_FAVORITE_NUMBERS)
                return createFavoritesDialog(favorite!!, numbers!!)
            }
        }

        return super.createDialog(which, data)
    }

    private fun createFavoritesDialog(favorite: FavoriteDataItem, numbers: List<ArrayListItem>): Dialog {
        // prepare the dialog
        val result = AppCompatDialog(activity)
        result.setTitle(R.string.tFavoriteCallOptions)
        result.setContentView(R.layout.dialog_favorites_call_options)
        result.setCancelable(true)
        result.setOnDismissListener { dialog -> dismissDialog(FAVORITE_CALL_DIALOG_ID) }

        // update the checkbox
        val rememberChoice = result.findViewById<CheckBox>(R.id.fav_call_options_remember)
        if (favorite.preferredAction != FavoriteDataItem.ACTION_TYPE_NONE) {
            rememberChoice?.isChecked = true
        }

        // prepare the number list UI
        val list = result.findViewById<RecyclerView>(R.id.fav_call_options_list)
        list?.layoutManager = LinearLayoutManager(activity)
        val adapter = ArrayListAdapter(list!!, numbers)

        // when he picks the number, hide the dialog and call that number
        adapter.setOnItemClickListener { item, index ->
            dismissDialog(FAVORITE_CALL_DIALOG_ID)

            // update check choice
            val pickedNumber = numbers[index].secondaryText
            presenter.updateFavoriteCallChoice(favorite, pickedNumber, rememberChoice!!.isChecked)

            presenter.directlyCallFavorite(favorite, pickedNumber!!)
        }
        list.adapter = adapter

        return result
    }


    override fun onMenuItemClick(menuItem: MenuItem?): Boolean {
        when (menuItem!!.itemId) {
            R.id.contact_option_view -> presenter.viewDetailsScreen()
            R.id.contact_option_edit -> presenter.viewEditScreen()
            R.id.contact_option_delete -> handleDeleteDialog()
            R.id.contact_option_remove_favorite -> presenter.deleteActiveItem()
            R.id.contact_option_call -> presenter.handleFavoriteView()
            R.id.mi_search -> mSearchView!!.open(presenter.getActiveTab().cleanSearchOnClose)
            R.id.mi_add_buddy, R.id.mi_add_contact -> handleAddPerson()
            R.id.contact_option_add_favorite -> presenter.addContactToFavorite()


        }
        return super.onMenuItemClick(menuItem)
    }

    fun handleDeleteDialog() {
        val noPermission = presenter.getActiveTab().hasPermissionForCheck && !checkPermission(Manifest.permission.WRITE_CONTACTS)
        if (noPermission) {
            presenter.setPermissionRequestSource(DELETE_DIALOG)
            requestPermission(Manifest.permission.WRITE_CONTACTS, PermissionRequestCode.CP_PERMISSION_WRITE_CONTACTS_FROM_CONTACTS_LIST, activity.getString(R.string
                    .tPermissionContacts))
            return
        }
        showDialog(SURE_DELETE_DIALOG_ID)
    }


    fun handleAddPerson() {
        when (presenter.getActiveTab().hasPermissionForCheck && (!checkPermission(Manifest.permission.READ_CONTACTS) || !checkPermission(Manifest.permission.WRITE_CONTACTS))) {
            true -> {
                if (!checkPermission(Manifest.permission.READ_CONTACTS)) {
                    presenter.setPermissionRequestSource(ADD_PERSON)
                    requestPermission(
                            Manifest.permission.READ_CONTACTS,
                            PermissionRequestCode.CP_PERMISSION_INIT_CONTACTS,
                            getString(R.string.tContactsListPermissionRequestExplanation))
                    return

                }
                if (!checkPermission(Manifest.permission.WRITE_CONTACTS)) {
                    presenter.setPermissionRequestSource(ADD_PERSON)
                    requestPermission(Manifest.permission.WRITE_CONTACTS, PermissionRequestCode.CP_PERMISSION_WRITE_CONTACTS_FROM_CONTACTS_LIST, activity.getString(R.string
                            .tPermissionContacts))
                    return
                }
            }
            false -> {
                when (presenter.getActiveTab().tagName) {
                    CONTACTS -> addContact()
                    BUDDIES -> addBuddy()
                    FAVORITES -> showScreenForResult(EScreenList.FAVORITES_PICKER)
                }
            }
        }

    }


    override fun createDialogForResult(whichScreen: IScreenEnum, data: Bundle): ScreenHolderDialog? {
        when (whichScreen as EScreenList) {
            EScreenList.FAVORITES_PICKER -> {
                val dialog: ScreenHolderDialog
                if (isInTabletMode) {
                    val color = Coloring.get().decodeColor(presenter.toolbarColor)
                    dialog = ScreenHolderDialog
                            .builder(activity)
                            .screen(EScreenList.FAVORITES_PICKER)
                            .anchorView(toolbar!!.findViewById(R.id.mi_add_contact))
                            .gravity(Gravity.BOTTOM)
                            .arrowColor(color)
                            .backgroundColor(color)
                            .bundle(data)
                            .build()
                } else {
                    dialog = ScreenHolderDialog
                            .builder(activity)
                            .screen(EScreenList.FAVORITES_PICKER)
                            .style(ScreenHolderDialog.DIALOG_MAX_SIZE_NO_MARGIN)
                            .bundle(data)
                            .build()
                }

                return dialog
            }
        }

        return super.createDialogForResult(whichScreen, data)
    }

    @MainThread
    override fun onNewMessage(message: Bundle, sender: IScreenEnum) {
        super.onNewMessage(message, sender)
        if (sender === EScreenList.ACCOUNT_SELECT) {
            val accountId = message.getInt(AccountSelectScreen.KEY_SELECTED_ACCOUNT, -1)
            presenter.getAllBuddyAddableAccounts().forEach({ account ->
                if (account.id == accountId) {
                    handleAddBuddyScreen(account)
                    return
                }
            })
        }
    }


    fun handleAccountChooserScreen() {
        val result = Bundle(2)
        result.putInt(AccountSelectScreen.KEY_FILTER, AccountSelectScreen.FILTER_IM_ACTIVE_NO_HARDWIRED)
        result.putString(AccountSelectScreen.KEY_TITLE, getString(string.tChooseAccountRecipient))
        showScreenForResult(EScreenList.ACCOUNT_SELECT, ScreenHolderDialog.DIALOG_BOTTOM_SHEET, result)
    }

    fun handleAddBuddyScreen(account: Account) {
        val bundle = android.os.Bundle()
        if (account.type == EAccountType.Sip) {
            bundle.putString(ContactEditPresenter.KEY_SCREEN_ACTION, ContactEditPresenter.ACTION_ADD)
            bundle.putSerializable(GlobalConstants.KEY_CONTACT_TYPE, GlobalConstants.EContactsFilterType.BUDDIES)
            bundle.putString(ContactEditPresenter.KEY_ACCOUNT_NICKNAME, account.getStr(EAccountSetting.Nickname))
            goToContactEditScreen(bundle)

        } else if (account.type == EAccountType.Xmpp) {
            bundle.putString(AddXmppBuddyPresenter.KEY_ACCOUNT_NICKNAME, account.getStr(EAccountSetting.Nickname))
            if (presenter.isPresenceStatusOffline()) toastShort(getString(R.string.tAddXmppBuddyWhileOffline)) else mCoordinator.flow(bundle).goTo(EScreenList.XMPP_BUDDY_ADD)
        }

    }

    private fun addBuddy() {
        when (presenter.getAllBuddyAddableAccounts().size) {
            0 -> toastShort(R.string.tMissingXmppAndSipAccounts);
            1 -> handleAddBuddyScreen(presenter.getAllBuddyAddableAccounts().get(0))
            else -> {
                handleAccountChooserScreen()
            }

        }

    }

    private fun addContact() {
        val bundle = Bundle()
        bundle.putString(ContactEditPresenter.KEY_SCREEN_ACTION, ContactEditPresenter.ACTION_ADD)
        bundle.putSerializable(GlobalConstants.KEY_CONTACT_TYPE, GlobalConstants.EContactsFilterType.CONTACTS)
        goToContactEditScreen(bundle)
    }

    override fun updateMenuItems(menu: Menu, updateKey: String?) {
        super.updateMenuItems(menu, updateKey)


        val activeTabItem = presenter.getActiveTab()
        if (TextUtils.isEmpty(updateKey) || AbstractScreen.MENU_INFLATION_NORMAL == updateKey || AbstractScreen.MENU_INFLATION_UPDATE == updateKey) {

            if (!activeTabItem.hasSearchFromMenu) menu.removeItem(R.id.mi_search)

            if (activeTabItem.tagName == GlobalConstants.EContactsFilterType.BUDDIES) {
                menu.removeItem(R.id.mi_add_contact)
            } else {
                menu.removeItem(R.id.mi_add_buddy)
            }


        } else {
            //MENU_INFLATION_POPUP
            if (activeTabItem.tagName == GlobalConstants.EContactsFilterType.CONTACTS) {
                if (!presenter.isContactFavorite()) {
                    menu.removeItem(R.id.contact_option_remove_favorite)
                } else {
                    menu.removeItem(R.id.contact_option_add_favorite)
                }
            }


        }

    }

    protected fun setupSpinner() {
        if (presenter.isRequestAvailable())
            SpinnerUtils.populateSpinner(mSpinner, getString(R.string.tDisplayAllBuddies), getString(R.string.tDisplayOnlyOnlineBuddies), getString(R.string
                    .tDisplayOnlyBuddyRequests))
        else SpinnerUtils.populateSpinner(mSpinner, getString(R.string.tDisplayAllBuddies), getString(R.string.tDisplayOnlyOnlineBuddies))

        SpinnerUtils.setSpinnerListener(mSpinner, object : SpinnerUtils.OnItemSelectedAdapter() {
            override fun onItemSelected(parent: AdapterView<*>, view: View, position: Int, id: Long) {
                presenter.setSubFilterType(position)

                //FIXME saveLayoutState()
            }
        })



        updateSelectedSpinerPosition()
    }

    private fun updateSelectedSpinerPosition() {
        val position = presenter.getSubFilterType()
        if (position >= 0 && position < mSpinner.getAdapter().count) {
            if (position != mSpinner.getSelectedItemPosition()) {
                mSpinner.setSelection(position)
            }
        }
    }


    @MainThread
    override fun onRequestPermissionsResult(requestCode: Int, permissions: Array<String>, grantResults: IntArray) {
        super.onRequestPermissionsResult(requestCode, permissions, grantResults)
        when (requestCode) {
            PermissionRequestCode.CP_PERMISSION_INIT_CONTACTS -> if (grantResults.size > 0 && grantResults[0] == PackageManager.PERMISSION_GRANTED) {
                presenter.initDataProviders()
                reloadDataScreen()
                when (presenter.getPermisssionRequestSoruce()) {
                    ADD_PERSON -> handleAddPerson()

                }

            }
            PermissionRequestCode.CP_PERMISSION_WRITE_CONTACTS_FROM_CONTACTS_LIST -> if (grantResults.size > 0 && grantResults[0] == PackageManager.PERMISSION_GRANTED) {
                when (presenter.getPermisssionRequestSoruce()) {
                    DELETE_DIALOG -> handleDeleteDialog()
                    ADD_PERSON -> handleAddPerson()
                }

            } else if (grantResults.size > 0 && grantResults[0] == PackageManager.PERMISSION_DENIED) {
                debug("CP_PERMISSION_WRITE_CONTACTS Not Granted")
            }
        }
    }
}
