package com.bria.voip.ui.main.im

import android.Manifest
import android.annotation.SuppressLint
import android.app.Dialog
import android.content.*
import android.content.pm.PackageManager
import android.graphics.drawable.AnimationDrawable
import android.net.Uri
import android.os.Build
import android.os.Bundle
import android.os.Parcelable
import android.speech.RecognizerIntent
import android.support.annotation.MainThread
import android.support.constraint.ConstraintLayout
import android.support.v4.content.FileProvider
import android.support.v4.widget.SwipeRefreshLayout
import android.support.v7.app.AlertDialog
import android.support.v7.widget.*
import android.text.Editable
import android.text.InputType
import android.text.TextUtils
import android.text.TextWatcher
import android.util.DisplayMetrics
import android.view.*
import android.view.View.GONE
import android.view.View.VISIBLE
import android.view.inputmethod.EditorInfo
import android.widget.*
import com.bria.common.analytics.BIAnalytics
import com.bria.common.controller.collaboration.utils.matcher.ParticipantMatcher
import com.bria.common.controller.contacts.buddy.Buddy
import com.bria.common.controller.contacts.buddy.XmppBuddy
import com.bria.common.controller.contacts.local.data.PhoneNumber
import com.bria.common.controller.im.ImpsUtils
import com.bria.common.controller.im.refactoring.adapter.FilterableInstantMessageListAdapter
import com.bria.common.controller.im.refactoring.adapter.InstantMessageListAdapter
import com.bria.common.controller.im.refactoring.chatroom.MentionsFinder
import com.bria.common.controller.im.refactoring.dataprovider.InstantMessageDataProvider
import com.bria.common.controller.im.refactoring.db.entities.ImConversationData
import com.bria.common.controller.settings.ESetting
import com.bria.common.mdm.MdmUtils
import com.bria.common.permission.PermissionRequestCode
import com.bria.common.ui.util.EditTextUtils
import com.bria.common.uiframework.activities.Orion
import com.bria.common.uiframework.annotations.Clickable
import com.bria.common.uiframework.annotations.Inject
import com.bria.common.uiframework.annotations.Layout
import com.bria.common.uiframework.branding.AbstractCustomizer
import com.bria.common.uiframework.coloring.Coloring
import com.bria.common.uiframework.dialogs.ScreenHolderDialog
import com.bria.common.uiframework.lists.recycler.IOnRecyclerViewItemEventsListener
import com.bria.common.uiframework.lists.recycler.ScrollLinearLayoutManager
import com.bria.common.uiframework.presenters.IPresenterEvent
import com.bria.common.uiframework.screens.AbstractScreen
import com.bria.common.uiframework.screens.IScreenEnum
import com.bria.common.uireusable.CustomSearchView
import com.bria.common.uireusable.datatypes.MessageListItemData
import com.bria.common.util.*
import com.bria.common.util.kotlin.setInvisible
import com.bria.common.util.kotlin.setVisible
import com.bria.voip.BriaSipGraph
import com.bria.voip.R
import com.bria.voip.ui.main.contacts.BuddyDisplayScreenPresenter
import com.bria.voip.ui.main.contacts.ChooseNumberOfXmppBuddyScreen
import com.bria.voip.ui.main.im.chatroom.members.ChatRoomMembersScreen
import com.bria.voip.ui.main.im.chatroom.mentions.ChatRoomMentionsTokenizer
import com.bria.voip.ui.main.misc.EPhoneActivityList
import com.bria.voip.ui.main.misc.EScreenList
import com.bria.voip.ui.shared.helpers.isVisible
import com.bria.voip.ui.shared.helpers.setVisible
import com.bria.voip.ui.shared.pickers.BuddyPickerScreen
import com.bria.voip.ui.shared.pickers.BuddyPickerScreen.CHOSEN_BUDDY_IDS
import com.bria.voip.ui.shared.pickers.ContactPickerScreen
import com.bria.voip.ui.shared.pickers.SmsPickerScreen
import com.bumptech.glide.Glide
import com.bumptech.glide.load.engine.DiskCacheStrategy
import com.bumptech.glide.request.RequestOptions
import io.reactivex.android.schedulers.AndroidSchedulers
import io.reactivex.rxkotlin.addTo
import io.reactivex.schedulers.Schedulers
import java.io.File
import java.util.concurrent.TimeUnit

@Layout(R.layout.conversation_screen)
@com.bria.common.uiframework.annotations.Menu(R.menu.chat_menu)
open class ConvScreen<Presenter : ConvPresenter> : AbstractScreen<Presenter>(), View.OnKeyListener {

    companion object {
        private val TAG = "ConvScreen"

        private val KEY_LAYOUT_STATE_LIST = "LAYOUT_CONVERSATION_STATE"
        private val KEY_LAYOUT_STATE_SAVED_CONVERSATION_ID = "LAYOUT_STATE_SAVED_CONVERSATION_ID"
        private val KEY_FILE_CONTAINER_VISIBLE = "FILE_CONTAINER_VISIBLE"
        private val KEY_EXPANDED_PARTICIPANT_PANEL_VISIBLE = "KEY_EXPANDED_PARTICIPANT_PANEL_VISIBLE"
        private val KEY_LAYOUT_STATE_FOCUSED = "LAYOUT_STATE_FOCUSED"
        private val MESSAGE_SELECTION_START_KEY = "MESSAGE_SELECTION_START_KEY"
        private val MESSAGE_SELECTION_END_KEY = "MESSAGE_SELECTION_END_KEY"
        private val KEY_LAYOUT_STATE_NEW_MESSAGES_FLAG = "LAYOUT_NEW_MESSAGES_STATE"
        private val SEARCH_STATE = "CONV_SCREEN_SEARCH_STATE"
        //DIALOGS
        private val CONTACT_EDIT_OVERLAY = 0xD0001
        private val DIALOG_DELETE_IMS = 0xD00002
        private val DELETE_CHAT_DIALOG = 0xD00003
        private val CHAT_ROOM_OVERLAY: Int = 0xD0006;

        private val COLLAB_SCREEN_SMALL_SIZE = 2000

        fun replaceDisplayNameWithMention(text: Editable, isChatRoom: () -> Boolean, getBuddyByDisplayName: (String) -> XmppBuddy?) {
            // special case for mentions by display name
            if (isChatRoom()) {
                if (text.length >= 3) {
                    // at minimum we need to have '@a '
                    val lastChar = text.last()
                    if (MentionsFinder.isMentionDelimiter(lastChar)) {
                        //ok. user typed a space

                        val lastIndexOf = text.lastIndexOf("@")
                        if (lastIndexOf >= 0) {
                            // ok. there is @ sign somewhere before

                            if (lastIndexOf == 0 || MentionsFinder.isMentionDelimiter(text[lastIndexOf - 1])) {
                                // ok. @ is at beginning of string, or after some whitespace

                                val displayName = text.substring(lastIndexOf + 1, text.length)

                                val buddy = getBuddyByDisplayName(displayName)
                                if (buddy != null) {
                                    val username = ImpsUtils.getUsernameFromAddress(buddy.imAddress)

                                    val newText = "@$username "
                                    if (displayName == newText) {
                                        // do not change text, it is the same
                                    } else {
                                        text.replace(lastIndexOf, text.length, newText)
                                    }
                                }
                            }
                        }
                    }
                }
            }
        }
    }

    @Inject(R.id.conversation_screen_toolbar_container)
    private lateinit var mFakeToolbar: ConstraintLayout

    @Inject(R.id.conversation_screen_ivPresenceImage)
    private lateinit var mPresenceIcon: ImageView

    @Inject(id = R.id.conversation_screen_tvContactName_image)
    private lateinit var mTitleViewIcon: ImageView

    @Inject(id = R.id.conversation_screen_tvContactName, back = ESetting.ColorNavBar, inv = true, tag = AbstractCustomizer.Tag.OSO)
    private lateinit var mTitleView: TextView

    @Inject(id = R.id.conversation_screen_tvPresenceNote, back = ESetting.ColorNavBar, inv = true, tag = AbstractCustomizer.Tag.OSO)
    private lateinit var mSubtitleView: TextView

    @Clickable
    @Inject(id = R.id.compose_message_send_message, fore = ESetting.ColorBrandTint, fade = true)
    private lateinit var mSendButton: ImageView

    @Inject(R.id.conversation_screen_messages_container)
    private lateinit var mListContainer: FrameLayout

    @Inject(R.id.conversation_screen_progress)
    private lateinit var mProgressBar: ProgressBar

    @Inject(R.id.conversation_screen_swipe_refresh)
    private lateinit var swipeRefresh: SwipeRefreshLayout

    @Inject(R.id.conversation_screen_recycler_view)
    private lateinit var recyclerList: RecyclerView

    @Inject(R.id.conversation_screen_filterable_swipe_refresh)
    private lateinit var filterableSwipeRefresh: SwipeRefreshLayout

    @Inject(R.id.conversation_screen_filterable_recycler_view)
    private lateinit var filterableRecyclerList: RecyclerView

    @Inject(R.id.conversation_screen_filterable_no_results)
    private lateinit var filterableNoResults: TextView

    @Inject(R.id.conversation_screen_participants_container)
    private lateinit var mParticipantsShortContainer: ViewGroup

    @Inject(R.id.conversation_screen_participants_panel_container)
    private lateinit var mParticipantsContainer: ViewGroup

    @Inject(R.id.conversation_screen_participants_panel_text_view)
    private lateinit var mParticipantsPanelShortList: TextView

    @Clickable
    @Inject(id = R.id.conversation_screen_participant_panel_details_button, fore = ESetting.ColorBrandTint, tag = AbstractCustomizer.Tag.FCS)
    private lateinit var mParticipantsPanelDetailsButton: TextView

    @Inject(R.id.conversation_screen_expanded_participants_panel_container)
    private lateinit var mParticipantsExpandedPanelContainer: ViewGroup

    @Inject(R.id.conversation_screen_expanded_participants_panel_text_view)
    private lateinit var mParticipantsExpandedPanelText: TextView

    @Clickable
    @Inject(id = R.id.conversation_screen_expanded_participant_panel_hide_button, fore = ESetting.ColorBrandTint, tag = AbstractCustomizer.Tag.FCS)
    private lateinit var mParticipantsExpandedPanelHideButton: TextView

    @Inject(R.id.conversation_screen_compose_container)
    private lateinit var mComposeContainer: View

    @Inject(R.id.compose_message_text)
    private lateinit var mMessageText: MultiAutoCompleteTextView

    @Inject(R.id.compose_message_counter)
    private lateinit var letterCounter: TextView

    @Inject(R.id.conversation_screen_ivTyping)
    private lateinit var typingImage: ImageView

    @Inject(R.id.conversation_screen_more_messages_container)
    private lateinit var mMoreMessagesContainer: LinearLayout

    @Inject(R.id.conversation_screen_more_messages)
    @Clickable
    private lateinit var mMoreMessagesText: TextView

    @Inject(R.id.compose_message_text_voice)
    @Clickable
    private lateinit var voiceInput: ImageView

    @Inject(id = R.id.conversation_screen_msel_toolbar, back = ESetting.ColorNavBar, tag = AbstractCustomizer.Tag.OSO)
    private lateinit var mSelectToolbar: LinearLayout

    @Inject(R.id.conversation_screen_holder)
    private lateinit var mScreenHolder: ConstraintLayout

    @Clickable
    @Inject(id = R.id.conversation_screen_msel_back, back = ESetting.ColorNavBar, inv = true)
    private lateinit var mSelectBack: ImageView

    @Inject(id = R.id.conversation_screen_msel_number, back = ESetting.ColorNavBar, inv = true)
    private lateinit var mSelectNumber: TextView

    @Clickable
    @Inject(id = R.id.conversation_screen_msel_del, back = ESetting.ColorNavBar, inv = true)
    private lateinit var mSelectDelete: ImageView

    @Clickable
    @Inject(id = R.id.conversation_screen_msel_copy, back = ESetting.ColorNavBar, inv = true)
    private lateinit var mSelectCopy: ImageView

    @Clickable
    @Inject(id = R.id.conversation_screen_msel_forward, back = ESetting.ColorNavBar, inv = true)
    private lateinit var mSelectForward: ImageView

    @Clickable
    @Inject(id = R.id.compose_message_send_file)
    private lateinit var mSendFile: ImageView

    @Clickable
    @Inject(id = R.id.compose_message_send_doc)
    private lateinit var mSendDoc: ImageView

    @Clickable
    @Inject(id = R.id.compose_message_send_image)
    private lateinit var mSendImage: ImageView

    @Inject(id = R.id.conversation_screen_send_file_container)
    private lateinit var mSendFileContainer: ConstraintLayout

    @Inject(id = R.id.conversation_screen_search_view, col = false)
    protected lateinit var mSearchView: CustomSearchView

    private lateinit var adapter: InstantMessageListAdapter
    private lateinit var filterableAdapter: FilterableInstantMessageListAdapter
    private var animationDrawable: AnimationDrawable? = null
    private lateinit var layoutManager: LinearLayoutManager
    private var menuItemClickEnabled: Boolean = true

    private var sendFileContainerVisible = false

    private val chatRoomMentionsAdapter = BriaSipGraph.chatRoomMentionsAdapter()

    @MainThread
    override fun onCreate(bundle: Bundle?) {
        super.onCreate(bundle)

        val size = activity.resources.getDimensionPixelSize(R.dimen.article_thumbnail_width)
        val options = RequestOptions()
                .diskCacheStrategy(DiskCacheStrategy.RESOURCE).override(size, size).skipMemoryCache(true).error(R.drawable.article_photo_placeholder)
        val glide = Glide.with(activity)
        val builder = glide.asBitmap()
        typingImage.setBackgroundResource(R.drawable.chat_typing)
        animationDrawable = typingImage.background as AnimationDrawable

        adapter = InstantMessageListAdapter(activity, builder, options, ParticipantMatcher(50, activity), clickListener, presenter.dataProvider, callback, presenter.imListAdapterMentionsHelper)
        filterableAdapter = FilterableInstantMessageListAdapter(activity, ParticipantMatcher(50, activity), clickListener, presenter.filterableDataProvider, presenter.imListAdapterMentionsHelper, filterAdapterCallbacks)
        layoutManager = ScrollLinearLayoutManager(activity, LinearLayoutManager.VERTICAL, false)
        layoutManager.stackFromEnd = true
        layoutManager.supportsPredictiveItemAnimations()
        recyclerList.addOnScrollListener(object : RecyclerView.OnScrollListener() {
            override fun onScrolled(recyclerView: RecyclerView, dx: Int, dy: Int) {
                super.onScrolled(recyclerView, dx, dy)
                //Log.d(TAG, "onScrolled: from " + (mLayoutManager.findFirstVisibleItemPosition() + " to: " + mLayoutManager.findLastVisibleItemPosition()));
                updateVisibleMessages()
                if (isScrolledToTheBottom()) {
                    mMoreMessagesContainer.visibility = View.GONE
                }
            }

            override fun onScrollStateChanged(recyclerView: RecyclerView, newState: Int) {
                super.onScrollStateChanged(recyclerView, newState)
                when (newState) {
                    RecyclerView.SCROLL_STATE_IDLE -> glide.resumeRequests()
                    AbsListView.OnScrollListener.SCROLL_STATE_TOUCH_SCROLL, AbsListView.OnScrollListener.SCROLL_STATE_FLING -> glide.pauseRequests()
                }
            }
        })
        //Collab Chat Patch
        if (isInsideCallScreen && !isInTabletMode) {
            val metrics = DisplayMetrics()
            activity.windowManager.defaultDisplay.getMetrics(metrics)
            if (metrics.heightPixels < COLLAB_SCREEN_SMALL_SIZE) {
                mScreenHolder.minHeight = resources.getDimensionPixelSize(R.dimen.collab_conversation_screen_minheight)
            } else {
                mScreenHolder.minHeight = resources.getDimensionPixelSize(R.dimen.popup_window_default_height)
            }
        }
        recyclerList.adapter = adapter
        recyclerList.layoutManager = layoutManager
        swipeRefresh.visibility = View.VISIBLE
        recyclerList.visibility = View.VISIBLE
        recyclerList.itemAnimator = DefaultItemAnimator()
        filterableRecyclerList.adapter = filterableAdapter
        val filterableLayoutManager = ScrollLinearLayoutManager(activity, LinearLayoutManager.VERTICAL, false)
        filterableRecyclerList.layoutManager = filterableLayoutManager
        filterableRecyclerList.visibility = View.GONE
        filterableRecyclerList.itemAnimator = DefaultItemAnimator()
        val dividerItemDecoration = DividerItemDecoration(activity, filterableLayoutManager.orientation);
        filterableRecyclerList.addItemDecoration(dividerItemDecoration);
        mSubtitleView.text = ""
        mSubtitleView.visibility = View.GONE
        mPresenceIcon.setImageDrawable(null)
        mPresenceIcon.visibility = View.GONE
        mMoreMessagesContainer.visibility = View.GONE
        mMessageText.imeOptions = EditorInfo.IME_ACTION_SEND
        mMessageText.setRawInputType(InputType.TYPE_CLASS_TEXT)
        mMessageText.setOnEditorActionListener { _, actionId, _ ->
            var reacted = false
            if (actionId == EditorInfo.IME_ACTION_SEND) {
                sendMessage()
                reacted = true
            }
            reacted
        }
        TooltipCompat.setTooltipText(mSelectForward, getString(R.string.tForward))
        TooltipCompat.setTooltipText(mSelectCopy, getString(R.string.tCopy))
        TooltipCompat.setTooltipText(mSelectDelete, getString(R.string.tDelete))
        mSearchView.setEventHandler(object : CustomSearchView.EventHandler {
            override fun searchFor(searchString: String?) {
                filterableAdapter.filterData(searchString ?: "")
            }

            override fun onClosed() {
                updateRecyclerViewVisibility()
                filterableAdapter.filterData("")
            }
            override fun onOpened() {
                updateRecyclerViewVisibility()
            }
            override fun onClear() {
                filterableAdapter.filterData("")
            }
        })

        mSearchView.setDelayedSearchInterval(0)
    }

    fun updateRecyclerViewVisibility() {
        if (mSearchView.isOpen) {
            recyclerList.visibility = View.GONE
            swipeRefresh.visibility = View.GONE
            if (filterableAdapter.itemCount > 0) {
                filterableRecyclerList.visibility = View.VISIBLE
                filterableSwipeRefresh.visibility = View.VISIBLE
                filterableNoResults.visibility = View.GONE
            } else {
                filterableRecyclerList.visibility = View.GONE
                filterableSwipeRefresh.visibility = View.GONE
                filterableNoResults.visibility = View.VISIBLE
            }
            filterableAdapter.selfInfo = presenter.getSelfInfo()
        } else {
            recyclerList.visibility = View.VISIBLE
            swipeRefresh.visibility = View.VISIBLE
            filterableRecyclerList.visibility = View.GONE
            filterableSwipeRefresh.visibility = View.GONE
            filterableNoResults.visibility = View.GONE
        }
    }

    @MainThread
    override fun onStart(bundle: Bundle?) {
        super.onStart(bundle)
        presenter.subscribe(this)
        if (bundle != null) {
            presenter.initConversation(bundle)
        }
        mMessageText.setOnKeyListener(this)
        mMessageText.addTextChangedListener(mMessageTextWatcher)
        typingImage.visibility = View.GONE
        letterCounter.visibility = View.GONE
        // mMessageText.filters = presenter.createMsgFilters()
        updateSwipeRefresh()
        typingImage.setBackgroundResource(R.drawable.chat_typing)
        animationDrawable = typingImage.background as AnimationDrawable
        typingImage.visibility = View.GONE
        voiceInput.visibility = if (presenter.isVoiceInputEnabled()) View.VISIBLE else View.GONE
        //presenter.setupVoiceInputButton(voiceInput, mMessageText, false)
        updateLastUnsentMessage()

        adapter
            .selectedMessageCountObservable
            .debounce(100, TimeUnit.MILLISECONDS, Schedulers.computation())
            .observeOn(AndroidSchedulers.mainThread())
            .subscribe(
                    { updateSelectionToolbar(it) },
                    { CrashInDebug.with(TAG, it) })
            .addTo(mStartStopDisposables)


        // calculate compose container visibility
        mSearchView
            .isOpenObservable

            // when search is open, hide compose container
            .map { !it }

            .subscribe({
                mComposeContainer.setVisible(it)
            }, {
                CrashInDebug.with(TAG, it)
            })
            .addTo(mStartStopDisposables)

        presenter.copySelectedVisibleObservable
            .subscribeOn(Schedulers.computation())
            .observeOn(AndroidSchedulers.mainThread())
            .subscribe({
                mSelectCopy.setVisible(it)
            }, {
                CrashInDebug.with(TAG, it)
            })
            .addTo(mStartStopDisposables)
    }

    @MainThread
    override fun onNewConfig(bundle: Bundle?) {
        super.onNewConfig(bundle)
        if (bundle != null && !presenter.isSameId(bundle)) {
            setExpandedParticipantsPanelVisibility(false)
            sendFileContainerVisible = false
            updateSendFileMenu()
        }
        if (mMessageText.isEnabled) {
            presenter.setLastUnsentMessage(mMessageText.text.toString())
        }
        if (bundle != null) {
            presenter.initConversation(bundle)
        }
    }

    @MainThread
    @SuppressLint("SetTextI18n")
    override fun onPresenterEvent(event: IPresenterEvent<*, *>) {
        val type = event.type as ConvPresenter.Events
        Log.d(TAG, "OnPresenterEvent ${type.name}")
        when (type) {
            ConvPresenter.Events.GO_UP -> {
                presenter.sendTypingNotification(false)
                if (isVisible) post({ coordinator.flow().goUp() }, 500)
            }
            ConvPresenter.Events.PUBLISH_ACTIVE_CONVERSATION -> publishActiveConversation(event.data as Bundle)
            ConvPresenter.Events.DESELECT_ALL_CONVERSATIONS -> post({ sendMessage(event.data as Bundle, EScreenList.CONVERSATION_LIST) }, 200)
            ConvPresenter.Events.USER_TYPING -> animateTyping(event.data != null && event.data as Boolean)
            ConvPresenter.Events.CONVERSATION_UPDATED -> updateScreen()
            ConvPresenter.Events.SET_INPUT_TEXT -> setInputText(event.data as String)
            ConvPresenter.Events.TITLE_UPDATED -> {
                mTitleView.text = if (event.data != null) event.data as String else presenter.getTitle()
                invalidateMenu()
            }
            ConvPresenter.Events.INVALIDATE_MENU -> invalidateMenu()
            ConvPresenter.Events.GO_TO_CONTACT_EDIT_SCREEN -> {
                if (event.data != null) {
                    val bundle = event.data as Bundle
                    if (isInTabletMode) {
                        showDialog(CONTACT_EDIT_OVERLAY, bundle)
                    } else {
                        mCoordinator.flow(bundle).goTo(EScreenList.CONTACT_EDIT)
                    }
                }
            }
            ConvPresenter.Events.GO_TO_ADD_XMPP_BUDDY_SCREEN -> {
                if (event.data != null) mCoordinator.flow(event.data as Bundle).goTo(EScreenList.XMPP_BUDDY_ADD)
            }
            ConvPresenter.Events.ON_IM_ERROR -> toastLong((event.data as BriaError).description)
            ConvPresenter.Events.SHOW_XMPP_NUMBER_CHOOSER -> {
                val xmppBuddy = event.data as XmppBuddy
                val bundle = Bundle().apply {
                    putString(ChooseNumberOfXmppBuddyScreen.IN_BUDDY_KEY, xmppBuddy.uniqueIdentifier)
                }

                showScreenForResult(EScreenList.CHOOSE_NUMBER_OF_XMPP_BUDDY, bundle)
            }
            ConvPresenter.Events.SHOW_TOAST -> {
                if (event.data is Int) toastShort(event.data as Int)
                if (event.data is String) toastShort(event.data as String)
            }
            ConvPresenter.Events.FORWARD_MESSAGES -> {
                if (event.data != null) mCoordinator.flow().withBundle(event.data as Bundle).goTo(EScreenList.CONVERSATION)

                adapter.clearSelection()
            }
            ConvPresenter.Events.START_CONVERSATION -> {
                if (event.data != null) mCoordinator.flow().withBundle(event.data as Bundle).goTo(EScreenList.CONVERSATION)
            }
            ConvPresenter.Events.MCU_ROOM_STATE_CHANGED -> {
                if (event.data != null) {
                    val isActive = event.data as Boolean
                    if (isActive) {
                        mMessageText.hint = getString(R.string.tTypeToCompose)
                    } else {
                        mMessageText.hint = if (presenter.isChatRoom()) getString(R.string.tRoomNotAvailable) else ""
                    }
                    mMessageText.isEnabled = isActive
                    setupActionBar()
                }
            }
            ConvPresenter.Events.ACCOUNTS_CHANGED -> {
                val isSending = canSendMessage(false)
                mMessageText.isEnabled = isSending
                if (isSending) {
                    mMessageText.hint = getString(R.string.tTypeToCompose)
                } else {
                    mMessageText.hint = if (presenter.isChatRoom()) getString(R.string.tRoomNotAvailable) else ""
                }
                if (!isSending) {
                    mMessageText.clearFocus()
                }
                invalidateMenu()
                mSendFile.visibility = if (canSendFile()) VISIBLE else GONE
                if (!canSendFile()) {
                    sendFileContainerVisible = false
                }
                updateSendFileMenu()

            }
            ConvPresenter.Events.UPDATE_ACTION_BAR -> setupActionBar()
            ConvPresenter.Events.MESSAGES_FETCHED -> swipeRefresh.isRefreshing = false
            ConvPresenter.Events.ADD_MESSAGE_WORD -> {
                mMessageText.setText(mMessageText.text.toString() + " " + event.data)
            }
            ConvPresenter.Events.CANCELED_LOADING -> {
            }
            ConvPresenter.Events.CLEAR_SELECTION -> {
                adapter.clearSelection()
            }
            ConvPresenter.Events.UPDATE_PARTICIPANTS_BAR -> {
                updateParticipantsContainer()
            }
            ConvPresenter.Events.SEND_TO_CLIPBOARD -> {
                if (presenter.mdmAllowsCopying) {
                    val getSelectedMessagesAsStringResult = event.data as InstantMessageDataProvider.GetSelectedMessagesAsStringResult

                    val clipboardManager = activity.getSystemService(Context.CLIPBOARD_SERVICE) as ClipboardManager
                    val clipData = ClipData.newPlainText("", getSelectedMessagesAsStringResult.text)
                    clipboardManager.primaryClip = clipData

                    if (getSelectedMessagesAsStringResult.someWereSkipped) {
                        toastShort(R.string.tSomeOfTheMessagesWereNotCopied)
                    } else {
                        toastShort(R.string.tMessagesCopiedToClipboard)
                    }
                }

                adapter.clearSelection()
            }
        }
    }

    @MainThread
    override fun onResume() {
        super.onResume()
        presenter.validateConversation()
        //getActivity().getWindow().setSoftInputMode(WindowManager.LayoutParams.SOFT_INPUT_ADJUST_RESIZE); // ???\
        //processVoiceInput()
        letterCounter.text = presenter.getSMSCounterValue(mMessageText.text.length)

        if (!isInTabletMode && !isInsideCallScreen) {
            // onResume is called twice, after second requestFocus() call selection is changed
            EditTextUtils.requestFocusAndPreserveSelection(mMessageText)
        }

        updateScreen()
        if (presenter.isTyping()) {
            animateTyping(true)
        }

        adapter.parseHyperlinks(presenter.shouldParseHyperlinks())
    }

    @MainThread
    override fun onSaveState(stateBundle: Bundle) {
        super.onSaveState(stateBundle)
        stateBundle.putParcelable(KEY_LAYOUT_STATE_LIST, layoutManager.onSaveInstanceState())
        stateBundle.putLong(KEY_LAYOUT_STATE_SAVED_CONVERSATION_ID, presenter.conversationData.getId())
        stateBundle.putBoolean(KEY_LAYOUT_STATE_NEW_MESSAGES_FLAG, mMoreMessagesContainer.visibility == View.VISIBLE)
        stateBundle.putBoolean(KEY_FILE_CONTAINER_VISIBLE, sendFileContainerVisible)
        stateBundle.putBoolean(KEY_EXPANDED_PARTICIPANT_PANEL_VISIBLE, mParticipantsExpandedPanelContainer.isVisible)
        stateBundle.putBoolean(KEY_LAYOUT_STATE_FOCUSED, mMessageText.isFocusable)
        if (mMessageText.isEnabled) {
            stateBundle.putInt(MESSAGE_SELECTION_START_KEY, mMessageText.selectionStart)
            stateBundle.putInt(MESSAGE_SELECTION_END_KEY, mMessageText.selectionEnd)
        }
        stateBundle.putBundle(SEARCH_STATE, mSearchView.saveState())
    }

    private var lastListState: Bundle? = null

    @MainThread
    override fun onRestoreState(stateBundle: Bundle) {
        super.onRestoreState(stateBundle)
        val layoutState = stateBundle.getParcelable<Parcelable>(KEY_LAYOUT_STATE_LIST)
        if (layoutState != null) {
            lastListState = Bundle()
            lastListState?.putParcelable(KEY_LAYOUT_STATE_LIST, layoutState)
            lastListState?.putLong(KEY_LAYOUT_STATE_SAVED_CONVERSATION_ID, stateBundle.getLong(KEY_LAYOUT_STATE_SAVED_CONVERSATION_ID, -1))
            mMoreMessagesContainer.visibility = if (stateBundle.getBoolean(KEY_LAYOUT_STATE_NEW_MESSAGES_FLAG, false)) View.VISIBLE else View.GONE
            sendFileContainerVisible = stateBundle.getBoolean(KEY_FILE_CONTAINER_VISIBLE, false)
            setExpandedParticipantsPanelVisibility(stateBundle.getBoolean(KEY_EXPANDED_PARTICIPANT_PANEL_VISIBLE, false))
            updateSendFileMenu()
            //mMoreMessagesContainer.setVisibility(stateBundle.getBoolean(KEY_LAYOUT_STATE_FOCUSED, false) ? View.VISIBLE : View.GONE);
        }
        val selectionStart = stateBundle.getInt(MESSAGE_SELECTION_START_KEY, mMessageText.length())
        val selectionEnd = stateBundle.getInt(MESSAGE_SELECTION_END_KEY, mMessageText.length())
        mSearchView.restoreState(stateBundle.getBundle(SEARCH_STATE))
        mMessageText.setSelection(selectionStart, selectionEnd)
    }

    private fun canSendFile(): Boolean = presenter.isFileTransferEnabled() && canSendMessage(false)

    private val clickListener = object : IOnRecyclerViewItemEventsListener {
        override fun onItemClick(item: View?, index: Int) {
            if (adapter.getSelectedMessageCount() > 0) {
                adapter.toggleSelectionByIndex(index)
            }
        }

        override fun onItemLongClick(item: View?, index: Int) {
            if (adapter.getSelectedMessageCount() == 0) {
                adapter.toggleSelectionByIndex(index)
            } else {
                onItemClick(item, index)
            }
        }

    }

    private fun updateSelectionToolbar(selectedMessageCount: Int) {
        mSelectForward.visibility = if (isInsideCallScreen) View.GONE else View.VISIBLE
        mSelectDelete.visibility = if (presenter.isChatRoom()) View.GONE else View.VISIBLE
        mSelectNumber.text = selectedMessageCount.toString()
        if (selectedMessageCount == 0) {
            mSelectToolbar.visibility = View.GONE
            mFakeToolbar.visibility = View.VISIBLE
        } else {
            mSelectToolbar.visibility = View.VISIBLE
            mFakeToolbar.visibility = View.GONE
        }
    }

    @MainThread
    override fun updateMenuItems(menu: Menu, updateKey: String?) {
        super.updateMenuItems(menu, updateKey)
        if (!presenter.canAddMoreParticipants() || isInsideCallScreen) {
            menu.removeItem(R.id.mi_chat_add_more)
        }

        if (!presenter.canCall() || isInsideCallScreen) {
            menu.removeItem(R.id.mi_chat_call)
        }
        if (!presenter.collabEnabled() || presenter.isConferenceLive() || isInsideCallScreen || (presenter.isChatRoom() && presenter.isRemotelyLeft())) {
            menu.removeItem(R.id.mi_chat_start_conf)
        }
        if (!presenter.collabEnabled() || !presenter.isConferenceHosted() || isInsideCallScreen || (presenter.isChatRoom() && presenter.isRemotelyLeft())) {
            menu.removeItem(R.id.mi_invite_to_conf)
        }

        if (!presenter.cannedEnabled()) {
            menu.removeItem(R.id.mi_chat_canned)
        }

        if (!presenter.isDeleteMenuItemVisible() || isInsideCallScreen) {
            menu.removeItem(R.id.mi_chat_delete)
        }

        if (!presenter.isLeaveRoomMenuItemVisible() || isInsideCallScreen) {
            menu.removeItem(R.id.mi_chat_leave_room)
        }

        if (!presenter.canSaveBuddy()) {
            menu.removeItem(R.id.mi_chat_add_buddy)
        }

        if (!presenter.canSaveContact()) {
            menu.removeItem(R.id.mi_chat_add_contact)
        }

        if (presenter.getParticipantBundle() == null) {
            menu.removeItem(R.id.mi_chat_view_buddy)
        }

        if (!presenter.canAddToExisting()) {
            menu.removeItem(R.id.mi_chat_add_to_existing)
        }

        if (!presenter.isChatRoom()) {
            menu.removeItem(R.id.mi_chat_search)
        }

        if (!presenter.isChatRoom() || presenter.isRemotelyLeft()) {
            menu.removeItem(R.id.mi_chat_room_members)
            menu.removeItem(R.id.mi_chat_room_configuration)
        }
    }

    @MainThread
    override fun onMenuItemClick(menuItem: MenuItem): Boolean {
        if (menuItemClickEnabled) {
            menuItemClickEnabled = false
            post({ menuItemClickEnabled = true }, 500)

            when (menuItem.itemId) {
                R.id.mi_chat_add_more -> {
                    saveComposedText()
                    val bundle = Bundle()
                    bundle.putStringArrayList(BuddyPickerScreen.ALREADY_PRESENT_PARTICIPANTS, presenter.getParticipantList())
                    bundle.putSerializable(BuddyPickerScreen.BUDDY_TYPE, Buddy.EBuddyType.XMPP)
                    showScreenForResult(EScreenList.BUDDY_PICKER, bundle)
                    return true
                }
                R.id.mi_chat_call -> {
                    presenter.call()
                    return true
                }
                R.id.mi_chat_start_conf -> {
                    presenter.startHostWithParticipantInvite()
                    return true
                }
                R.id.mi_invite_to_conf -> {
                    presenter.inviteParticipantToJoinConf()
                    return true
                }
                R.id.mi_chat_canned -> {
                    saveComposedText()
                    showScreenForResult(EScreenList.CANNED_IM)
                    BIAnalytics.get().reportUIClick("quick-responses-used-in-chat")
                    return true
                }
                R.id.mi_chat_room_members -> {
                    val bundle = Bundle(1)
                    bundle.putLong(ChatRoomMembersScreen.KEY_CHAT_ROOM_ID, presenter.conversationData.getId())
                    mCoordinator.flow().withBundle(bundle).goTo(EScreenList.CHAT_ROOM_MEMBERS_SCREEN)
                    return true
                }
                R.id.mi_chat_room_configuration -> {
                    if (presenter.isActiveRoom()) {
                        val bundle = Bundle(2)
                        bundle.putLong(GlobalConstants.KEY_CHAT_ROOM_ID, presenter.conversationData.getId())
                        goToChatRoom(bundle)
                    } else {
                        toastLong(getString(R.string.tChatRoomServiceNotReady))
                    }
                    return true
                }
                R.id.mi_chat_delete ->{
                    showDialog(DELETE_CHAT_DIALOG)
                    return true
                }
                R.id.mi_chat_leave_room -> {
                    presenter.handleDeleteConversation()
                    return true
                }
                R.id.mi_chat_add_buddy -> {
                    if (presenter.conversationType != ImConversationData.IM_SIP ||
                            checkOrRequestContactsPermission(PermissionRequestCode.CP_PERMISSION_ADD_SIP_BUDDY)) {
                        presenter.handleSaveBuddy()
                    }
                    return true
                }
                R.id.mi_chat_add_contact -> {
                    if (checkOrRequestContactsPermission(PermissionRequestCode.CP_PERMISSION_ADD_CONTACT_FROM_CONVERSATION_DISPLAY)) {
                        presenter.handleSaveContact()
                    }
                    return true
                }
                R.id.mi_chat_view_buddy -> {
                    val participantBundle = presenter.getParticipantBundle()
                    if (participantBundle == null) {
                        CrashInDebug.with(name, "Clicked on 'view buddy' item, but no participant bundle available..")
                    } else {
                        Orion.get().showActivity(activity, EPhoneActivityList.MAIN, GlobalConstants.INTENT_ACTION_SHOW_CONTACT, participantBundle, false)
                    }

                    return true
                }
                R.id.mi_chat_add_to_existing -> {
                    if (checkOrRequestContactsPermission(PermissionRequestCode.CP_PERMISSION_UPDATE_CONTACT_FROM_CONVERSATION_DISPLAY)) {
                        showContactPicker()
                    }
                    return true
                }
                R.id.mi_chat_search -> {
                    mSearchView.open(true)
                    //mybe disable adding messsages
                }
            }
        }

        return super.onMenuItemClick(menuItem)
    }

    @MainThread
    override fun createDialogForResult(whichScreen: IScreenEnum, data: Bundle): ScreenHolderDialog? {
        when (whichScreen) {
            EScreenList.CHOOSE_NUMBER_OF_XMPP_BUDDY -> {
                return ScreenHolderDialog
                    .builder(activity)
                    .screen(EScreenList.CHOOSE_NUMBER_OF_XMPP_BUDDY)
                    .style(ScreenHolderDialog.DIALOG_MIN_SIZE)
                    .showTitle(false)
                    .bundle(data)
                    .build()
            }

            EScreenList.BUDDY_PICKER, EScreenList.SMS_PICKER, EScreenList.CONTACT_PICKER -> {
                return if (isInTabletMode) {
                    val color = Coloring.get().decodeColor(presenter.toolbarColor)

                    ScreenHolderDialog
                            .builder(activity)
                            .screen(whichScreen)
                            .anchorView(toolbar!!.findViewById(R.id.conversation_list_new))
                            .gravity(Gravity.BOTTOM)
                            .arrowColor(color)
                            .backgroundColor(color)
                            .bundle(data)
                            .build()
                } else {
                    ScreenHolderDialog.builder(activity).screen(whichScreen).style(ScreenHolderDialog.DIALOG_MAX_SIZE_NO_MARGIN).bundle(data).build()
                }
            }
            EScreenList.CANNED_IM ->
                return ScreenHolderDialog.builder(activity).screen(whichScreen).style(ScreenHolderDialog.DIALOG_MAX_SIZE_NO_MARGIN).bundle(data).build()

            EScreenList.GALLERY ->
                return ScreenHolderDialog.builder(activity).screen(whichScreen).style(ScreenHolderDialog.DIALOG_MAX_SIZE_NO_MARGIN).bundle(data).build()
            EScreenList.FILE_PICKER ->
                return ScreenHolderDialog.builder(activity).screen(whichScreen).style(ScreenHolderDialog.DIALOG_MAX_SIZE_NO_MARGIN).bundle(data).build()
        }
        return super.createDialogForResult(whichScreen, data)
    }

    private fun showContactPicker() {
        val bundle = Bundle(1)
        bundle.putInt(ContactPickerScreen.MAX_NUMBER_OF_CHOICES, 1)
        showScreenForResult(EScreenList.CONTACT_PICKER, bundle)
    }



    private fun goToChatRoom(bundle: Bundle) {
//        if (isInTabletMode) {
//            showDialog(CHAT_ROOM_OVERLAY, bundle)
//        } else {
            mCoordinator.flow(bundle).goTo(EScreenList.CHAT_ROOM_DETAILS_SCREEN)
//        }
    }

    @MainThread
    override fun onStop(finishing: Boolean) {
        super.onStop(finishing)
        presenter.sendTypingNotification(false)
        presenter.unsubscribe()
        saveComposedText()

        activity.window.setSoftInputMode(WindowManager.LayoutParams.SOFT_INPUT_ADJUST_NOTHING)
        mMessageText.removeTextChangedListener(mMessageTextWatcher)
        if (finishing) {

            mSearchView.close()
            mSearchView.setEventHandler(null)
        }
    }

    @MainThread
    override fun onPause(finishing: Boolean) {
        super.onPause(finishing)
        presenter.sendTypingNotification(false)
    }

    private fun saveComposedText() {
        if (mMessageText.isEnabled) {
            presenter.setLastUnsentMessage(mMessageText.text.toString())
        }
    }

    private fun setupActionBar() {
        mTitleView.text = presenter.getTitle()
        if (presenter.isPrivateChatRoom()) {
            mTitleViewIcon.setVisible()
        } else {
            mTitleViewIcon.setInvisible()
        }
        if (presenter.isPresenceIconVisible()) {
            val presenceIcon = presenter.getPresenceIcon()
            mPresenceIcon.setImageDrawable(presenceIcon)
            mPresenceIcon.visibility = View.VISIBLE
        } else {
            mPresenceIcon.visibility = GONE
        }
        mSubtitleView.visibility = if (presenter.isGroupChat || presenter.isSMS() || presenter.isCollab()) View.GONE else View.VISIBLE
        mSubtitleView.text = presenter.getSubtitleText()
    }

    private fun updateScreen() {
        setupActionBar()
        val isSending = canSendMessage(true)
        mMessageText.isEnabled = isSending
        if (isSending) {
            mMessageText.hint = getString(R.string.tTypeToCompose)
        } else {
            mMessageText.hint = if (presenter.isChatRoom()) getString(R.string.tRoomNotAvailable) else ""
        }
        if (!isSending) {
            mMessageText.clearFocus()
        }

        if (presenter.isChatRoom()) {
            val chatRoom = presenter.conversationData.getChatRoom()
            chatRoomMentionsAdapter.setChatRoom(Optional.fromNullable(chatRoom))
            mMessageText.setAdapter(chatRoomMentionsAdapter)
            mMessageText.threshold = 1
            mMessageText.setTokenizer(ChatRoomMentionsTokenizer())
        } else {
            mMessageText.setRawInputType(InputType.TYPE_CLASS_TEXT);
            mMessageText.setAdapter(null)
        }

        invalidateMenu()
        updateSwipeRefresh()
        if (presenter.remoteSyncEnabled()) {
            fetchSyncedMsgsStart()
        }
        if (MdmUtils.isCopyPasteRestricted(activity)) {
            MdmUtils.blockCopyPasteOnEditText(mMessageText)
        }

        updateParticipantsContainer()
        mSendFile.visibility = if (canSendFile()) VISIBLE else GONE
        updateSendFileMenu()
    }

    private fun updateParticipantsContainer() {
        if (presenter.isGroupChat && presenter.hasParticipants() && !presenter.isCollab()) {
            mParticipantsContainer.visibility = VISIBLE
            mParticipantsPanelShortList.text = presenter.getParticipantNames()
            mParticipantsExpandedPanelText.text = presenter.getParticipantsText()
        } else {
            mParticipantsContainer.visibility = GONE
        }

    }

    private fun updateLastUnsentMessage() {
        val lastUnsentMsg = presenter.getLastUnsentMessage()

        if(lastUnsentMsg != mMessageText.text.toString()){
            // setText changes selection
            if (!TextUtils.isEmpty(lastUnsentMsg)) {
                mMessageText.setText(lastUnsentMsg)
            } else {
                mMessageText.setText("")
            }
        }
    }

    private fun publishActiveConversation(bundle: Bundle) {
        if (isInTabletMode && !isInsideCallScreen) {
            val convBundle = Bundle()
            convBundle.putInt(GlobalConstants.KEY_CONVERSATION_TYPE, presenter.conversationType)
            mCoordinator.flow().withBundle(convBundle).goTo(EScreenList.CONVERSATION_LIST)
            mHandler.postDelayed({ sendMessage(bundle, EScreenList.CONVERSATION_LIST) }, 300)
        }
    }

    private fun animateTyping(showAnimation: Boolean) {
        var visibility = View.VISIBLE

        if (showAnimation) {
            typingImage.visibility = visibility
            if (!animationDrawable!!.isRunning) {
                animationDrawable!!.start()
            }
        } else {
            animationDrawable?.stop()
            animationDrawable?.selectDrawable(0)
            visibility = View.GONE
            typingImage.visibility = visibility
        }
    }

    @MainThread
    override fun onClick(v: View) {
        when (v.id) {
            R.id.compose_message_send_message -> sendMessage()
            R.id.conversation_screen_participant_panel_details_button -> setExpandedParticipantsPanelVisibility(true)
            R.id.conversation_screen_expanded_participant_panel_hide_button -> setExpandedParticipantsPanelVisibility(false)
            R.id.conversation_screen_msel_back -> {
                adapter.clearSelection()
            }
            R.id.compose_message_text_voice -> {
                val intent = Intent(RecognizerIntent.ACTION_RECOGNIZE_SPEECH)
                intent.putExtra(RecognizerIntent.EXTRA_LANGUAGE_MODEL, RecognizerIntent.LANGUAGE_MODEL_FREE_FORM)
                intent.putExtra(RecognizerIntent.EXTRA_PROMPT, activity.getString(R.string.tWaitingForVoice))
                activity.startActivityForResult(intent, GlobalConstants.INTENT_CODE_VOICE_INPUT_IM)
            }
            R.id.conversation_screen_msel_del -> showDialog(DIALOG_DELETE_IMS)
            R.id.conversation_screen_msel_copy -> {
                copySelectedMessages()
            }
            R.id.conversation_screen_msel_forward -> {
                openForwardToDialog()
            }
            R.id.conversation_screen_more_messages -> {
                adapter.setCurrentVisibleMessages(layoutManager.findFirstVisibleItemPosition(), adapter.itemCount - 1)
                recyclerList.scrollToPosition(adapter.itemCount - 1)
            }
            R.id.compose_message_send_file -> {
                sendFileContainerVisible = !sendFileContainerVisible
                updateSendFileMenu()
            }
            R.id.compose_message_send_image -> {
                val bundle = Bundle()
                bundle.putString(GlobalConstants.KEY_SENDER_NAME, presenter.getTitle())
                showScreenForResult(EScreenList.GALLERY, bundle)
            }
            R.id.compose_message_send_doc -> {
                showScreenForResult(EScreenList.FILE_PICKER)
            }
        }
        super.onClick(v)
    }

    private fun updateSendFileMenu() {
        if (sendFileContainerVisible) {
            if (!isInTabletMode) {
                // on tablet there is enough vertical space
                hideKeyboard()
            }
        }

        mSendFileContainer.visibility = if (sendFileContainerVisible) VISIBLE else GONE
    }

    override fun onSoftKeyboardOpen(keyboardHeightPx: Int) {
        if (!isInTabletMode) {
            // on tablet there is enough vertical space
            sendFileContainerVisible = false
            updateSendFileMenu()
        }
    }

    private fun copySelectedMessages() {
        presenter.prepareSelectedMessagesForCopying()
    }

    private fun sendMessage(): Boolean {
        if (presenter.isWifiOnlyRegistration() && presenter.noWiFiConnectivity()) {
            checkWiFiWarning();
            return false;
        }
        if (!canSendMessage(true)) {
            return false
        }
        val success = presenter.sendMessage(mMessageText.text.toString())
        if (success) {
            mMessageText.setText("")
            presenter.setLastUnsentMessage("")
        }
        return success
    }

    private fun canSendMessage(showNotice: Boolean): Boolean {
        val errorMsgId = presenter.canSendMessages()
        if (showNotice && errorMsgId != 0) {
            if (isVisible) toastShort(errorMsgId)
        }
        return errorMsgId == 0
    }

    @MainThread
    override fun onDestroy(finishing: Boolean) {
        super.onDestroy(finishing)
        adapter.cleanup()

        chatRoomMentionsAdapter.close()
    }

    @MainThread
    override fun onNewMessage(message: Bundle, sender: IScreenEnum) {
        super.onNewMessage(message, sender)
        when (sender) {
            EScreenList.CHOOSE_NUMBER_OF_XMPP_BUDDY -> {
                val number = message.getString(ChooseNumberOfXmppBuddyScreen.RESULT_SELECTED_NUMBER)

                presenter.callXmppNumber(number)
            }
            EScreenList.BUDDY_PICKER -> {
                invalidateMenu()
                if (message.containsKey(ConvPresenter.KEY_CHAT_ACTION)) {
                    if (message.getString(ConvPresenter.KEY_CHAT_ACTION) == ConvPresenter.ACTION_FORWARD) {
                        message.getStringArrayList(CHOSEN_BUDDY_IDS)?.let {
                            presenter.forwardSelectedMessagesTo(it)
                        }
                    }
                } else {
                    message.getStringArrayList(CHOSEN_BUDDY_IDS)?.let {
                        presenter.addParticipants(it)
                    }
                }
            }
            EScreenList.SMS_PICKER -> {
                invalidateMenu()
                if (message.containsKey(ConvPresenter.KEY_CHAT_ACTION)) {
                    if (ConvPresenter.ACTION_FORWARD == message.getString(ConvPresenter.KEY_CHAT_ACTION)) {
                        message.getParcelableArrayList<PhoneNumber>(SmsPickerScreen.SELECTED_NUMBERS)?.let {
                            presenter.forwardSMSTo(it)
                        }
                    }
                }
            }
            EScreenList.CANNED_IM -> {
                parseCannedIm(message.getString(CannedImScreen.KEY_MESSAGE_TEXT, ""))
            }
            EScreenList.GALLERY -> {
                BIAnalytics.get().reportUIClick(" file-transfer-send")
                presenter.sendImages(message.getStringArray(GlobalConstants.KEY_IMAGES_TO_SEND))
            }
            EScreenList.FILE_PICKER -> {
                BIAnalytics.get().reportUIClick(" file-transfer-send")
                presenter.sendFiles(message.getStringArray(GlobalConstants.KEY_FILES_TO_SEND))
            }
            EScreenList.CONTACT_PICKER -> {
                val contactIdList = message.getStringArrayList(ContactPickerScreen.SELECTED_CONTACT_IDS)
                if (contactIdList != null && contactIdList.isNotEmpty()) {
                    presenter.handleAddToExisting(contactIdList[0])
                }
            }
        }
    }

    @MainThread
    override fun createDialog(which: Int, data: Bundle?): Dialog? {
        when (which) {
            DIALOG_DELETE_IMS -> {
                val builder = AlertDialog.Builder(activity)
                builder.setTitle(R.string.tDelete)
                        .setMessage(String.format(getString(R.string.tDeleteMessages), adapter.getSelectedMessageCount()))
                        .setIcon(R.drawable.ic_delete)
                        .setCancelable(true)
                        .setPositiveButton(R.string.tYes) { _, _ ->
                            presenter.deleteSelectedMessages()
                        }
                        .setNegativeButton(R.string.tNo) { dialogInterface, _ -> dialogInterface.dismiss() }
                return builder.create()
            }
            DELETE_CHAT_DIALOG -> {
                val message = when (presenter.conversationData.isChatRoom()) {
                    true -> getString(R.string.tChatRoomAreYouSureYouWantToDeleteThisRoom)
                    false -> getString(R.string.tAreYouSureDialog)
                }

                val builder1 = AlertDialog.Builder(activity)
                builder1.setMessage(message)
                        .setCancelable(false)
                        .setPositiveButton(getString(R.string.tYes)) { _, _ -> presenter.handleDeleteConversation() }
                        .setNegativeButton(getString(R.string.tNo)) { dialog, _ -> dialog.dismiss() }
                return builder1.create()
            }
            CONTACT_EDIT_OVERLAY -> return ScreenHolderDialog
                    .builder(activity)
                    .screen(EScreenList.CONTACT_EDIT)
                    .style(ScreenHolderDialog.DIALOG_MAX_SIZE_NO_MARGIN)
                    .bundle(data)
                    .build()
            CHAT_ROOM_OVERLAY -> return ScreenHolderDialog
                .builder(activity)
                .screen(EScreenList.CHAT_ROOM_DETAILS_SCREEN)
                .style(ScreenHolderDialog.DIALOG_MAX_SIZE_NO_MARGIN)
                .bundle(data)
                .build()
            else -> {
                return super.createDialog(which, data)
            }
        }
    }

    private fun parseCannedIm(message: String) {
        var typedMsg = mMessageText.text.toString()
        val selStart = mMessageText.selectionStart
        val selEnd = mMessageText.selectionEnd

        typedMsg = typedMsg.substring(0, selStart) + message + typedMsg.substring(selEnd)
        mMessageText.setText(typedMsg)
        var totalLength = selStart + message.length
        if (totalLength > GlobalConstants.MAX_IM_MESSAGE_LENGTH) {
            totalLength = GlobalConstants.MAX_IM_MESSAGE_LENGTH
            Toast.makeText(activity, R.string.tMaxMeassage, Toast.LENGTH_LONG).show()
        }
        mMessageText.setSelection(totalLength)
        mMessageText.requestFocus()
    }

    @Suppress("UNCHECKED_CAST")
    override fun getPresenterClass(): Class<Presenter> = ConvPresenter::class.java as Class<Presenter>

    override fun getTitle(): String? = ""

    private fun openForwardToDialog() {
        saveComposedText()
        val bundle = Bundle()
        bundle.putString(ConvPresenter.KEY_CHAT_ACTION, ConvPresenter.ACTION_FORWARD)
        if (presenter.isImType || presenter.isChatRoom()) {
            bundle.putBoolean("FORWARD_TO", true)
            if (presenter.dataProvider.hasTransferMessagesInSelection() || presenter.isRoomFeatureEnabled()) {
                bundle.putInt(SmsPickerScreen.MAX_NUMBER_OF_CHOICES, 1)
            }

            if (presenter.dataProvider.getSelectedFileTransferMessages().isNotEmpty()) {
                bundle.putSerializable(BuddyPickerScreen.BUDDY_TYPE, Buddy.EBuddyType.XMPP)
            }

            showScreenForResult(EScreenList.BUDDY_PICKER, bundle)
        } else {
            bundle.putInt(SmsPickerScreen.MAX_NUMBER_OF_CHOICES, 1)
            bundle.putStringArrayList(SmsPickerScreen.ALREADY_PRESENT_PHONES, presenter.getPhoneList())
            showScreenForResult(EScreenList.SMS_PICKER, bundle)
        }
    }

    private fun isScrolledToTheBottom(): Boolean = layoutManager.itemCount - 2 <= layoutManager.findLastVisibleItemPosition()

    private fun isScreenActive(): Boolean = state == AbstractScreen.EScreenState.RESUMED && isVisible

    private fun updateVisibleMessages() {
        if (isScreenActive()) {
            val first = layoutManager.findFirstVisibleItemPosition()
            val last = layoutManager.findLastVisibleItemPosition()
            adapter.setCurrentVisibleMessages(first, last)
        }
    }

    private fun setInputText(text: String) {
        if (!TextUtils.isEmpty(text)) {
            mMessageText.setText(text)
            mMessageText.setSelection(text.length)
        } else {
            mMessageText.setText("")
        }
    }

    private fun setExpandedParticipantsPanelVisibility(visible: Boolean) {
        mParticipantsExpandedPanelContainer.setVisible(visible)
        mParticipantsShortContainer.setVisible(!visible)
    }

    private fun updateSwipeRefresh() {
        swipeRefresh.setOnRefreshListener {
            Log.d(TAG, "onRefresh: ")
            adapter.loadMoreMessages()
        }
        swipeRefresh.isRefreshing = false
        filterableSwipeRefresh.isEnabled =false
    }

    private val mRunnableImSyncLoadComplete = Runnable { fetchSyncedMsgsEnd() }

    private fun fetchSyncedMsgsStart() {
        // See https://code.google.com/p/android/issues/detail?id=77712
        swipeRefresh.post { swipeRefresh.isRefreshing = true }

        mHandler.postDelayed(mRunnableImSyncLoadComplete, 5000)
        //nvtd
        //getPresenter().fetchRangeRevision();
    }

    private fun fetchSyncedMsgsEnd() {
        mHandler.removeCallbacks(mRunnableImSyncLoadComplete)
        swipeRefresh.isRefreshing = false
    }

    private val mMessageTextWatcher = object : TextWatcher {
        override fun onTextChanged(text: CharSequence, start: Int, before: Int, count: Int) {
            if (text.length >= presenter.getMessageMaxLength()) {
                toastLong(R.string.tMaxMeassage)
            }
        }

        override fun beforeTextChanged(text: CharSequence, start: Int, count: Int, after: Int) {}

        private var inReplaceDisplayNameWithMention = false

        override fun afterTextChanged(text: Editable) {
            presenter.sendTypingNotification(!TextUtils.isEmpty(text))
            while (text.length > presenter.getMessageMaxLength()) {
                //if emoji is inserted last it's not sufficient to delete only last char
                // protobuffer will have issue otherwise
                if (ImpsUtils.isLast2CharEmoji(text)) {
                    text.delete(text.length - 2, text.length)
                } else {
                    text.delete(text.length - 1, text.length)
                }

            }

            if (!inReplaceDisplayNameWithMention) {
                inReplaceDisplayNameWithMention = true
                try {
                    replaceDisplayNameWithMention(text, presenter::isChatRoom, presenter::getBuddyByDisplayName)
                } finally {
                    inReplaceDisplayNameWithMention = false
                }
            }
        }
    }

    @MainThread
    override fun onKey(v: View, keyCode: Int, event: KeyEvent): Boolean {
        if (event.action == KeyEvent.ACTION_DOWN && keyCode == KeyEvent.KEYCODE_BACK) {
            animateTyping(false)
        }
        return false
    }

    private val filterAdapterCallbacks = object : FilterableInstantMessageListAdapter.ICallbacks {
        override fun goToMessage(messageID: Long) {
            mSearchView.close()
            updateRecyclerViewVisibility()
            val position = adapter.markMessageAndGetPosition(messageID)
            if (position > 0) {
                recyclerList.scrollToPosition(position)
            }

        }

        override fun displayBuddy(buddy: XmppBuddy) {
            displayXmppBuddy(buddy)
        }

        override fun listUpdated() {
            updateRecyclerViewVisibility()
        }
    }

    private val callback = object : InstantMessageListAdapter.IInstantMessageListAdapterCallback {
        override fun canSendMessage(): Boolean = canSendMessage(true)

        override fun openFile(filePath: String?) {
            filePath?.let {
                val file = File(filePath)
                val uri: Uri
                uri = if (Build.VERSION.SDK_INT >= Build.VERSION_CODES.N) {
                    FileProvider.getUriForFile(activity, activity.packageName, file)
                } else {
                    Uri.fromFile(file)
                }
                val intent = Intent(Intent.ACTION_VIEW)
                //intent.setDataAndType(uri, URLConnection.guessContentTypeFromName(uri.toString()))
                intent.data = uri
                intent.addFlags(Intent.FLAG_GRANT_READ_URI_PERMISSION)
                intent.addFlags(Intent.FLAG_ACTIVITY_NEW_TASK)
                try {
                    activity.startActivity(intent)
                } catch (e: ActivityNotFoundException) {
                    toastLong("No handler for this type of file.")
                }

            } ?: toastLong("File could not be found")
        }

        override fun listUpdated(desiredPosition: Int?) {
            swipeRefresh.isRefreshing = false
            filterableSwipeRefresh.isRefreshing = false
            if (lastListState != null) {
                Log.d(TAG, "Loading previous screen state")
                val layoutState = lastListState?.getParcelable<Parcelable>(KEY_LAYOUT_STATE_LIST)
                val conversationID = lastListState?.getLong(KEY_LAYOUT_STATE_SAVED_CONVERSATION_ID, -1)
                if (conversationID == presenter.conversationData.getId()) {
                    Log.d(TAG, "Confirmed same Id")
                    recyclerList.layoutManager?.onRestoreInstanceState(layoutState)
                }
                lastListState = null
            } else {
                Log.d(TAG, "scroll to: $desiredPosition")
                desiredPosition?.let {
                    recyclerList.scrollToPosition(desiredPosition)
                }

            }
        }

        override fun setMoreMessagesVisibility(visible: Boolean) {
            mMoreMessagesContainer.visibility = if (visible) VISIBLE else GONE
        }

        override fun acceptIncomingFileClicked(messageListItemData: MessageListItemData) {
            presenter.acceptIncomingFileClicked(messageListItemData)
        }

        override fun displayBuddy(buddy: XmppBuddy) {
            displayXmppBuddy(buddy)
        }
    }

    private fun displayXmppBuddy(buddy: XmppBuddy) {
        val bundle = Bundle()
        bundle.putString(BuddyDisplayScreenPresenter.KEY_BUDDY_ID, ImpsUtils.getBuddyKey(buddy))

        mCoordinator.flow(bundle).goTo(EScreenList.BUDDY_DISPLAY)
    }

    private fun checkOrRequestContactsPermission(requestCode: Int): Boolean {
        if (!checkPermission(Manifest.permission.WRITE_CONTACTS)) {
            requestPermission(Manifest.permission.WRITE_CONTACTS, requestCode, getString(R.string.tPermissionContacts))
            return false
        }

        return true
    }

    @MainThread
    override fun onRequestPermissionsResult(requestCode: Int, permissions: Array<String>, grantResults: IntArray) {
        super.onRequestPermissionsResult(requestCode, permissions, grantResults)

        if (grantResults.isEmpty()) {
            return
        }

        if (grantResults[0] == PackageManager.PERMISSION_GRANTED) {
            when (requestCode) {
                PermissionRequestCode.CP_PERMISSION_ADD_SIP_BUDDY -> presenter.handleSaveBuddy()
                PermissionRequestCode.CP_PERMISSION_ADD_CONTACT_FROM_CONVERSATION_DISPLAY -> presenter.handleSaveContact()
                PermissionRequestCode.CP_PERMISSION_UPDATE_CONTACT_FROM_CONVERSATION_DISPLAY -> showContactPicker()
                PermissionRequestCode.CP_PERMISSION_WRITE_EXTERNAL_STORAGE -> adapter.notifyDataSetChanged()
            }
        }
    }

}
