package com.bria.voip.ui.main.im.chatroom.details

import android.app.Dialog
import android.os.Bundle
import android.support.annotation.MainThread
import android.support.v7.app.AlertDialog
import android.view.Gravity
import android.view.View
import android.widget.Button
import android.widget.ImageView
import android.widget.TextView
import com.bria.common.controller.Controllers
import com.bria.common.controller.buddy.BuddyAvailability
import com.bria.common.controller.contacts.buddy.Buddy
import com.bria.common.uiframework.annotations.Clickable
import com.bria.common.uiframework.annotations.Inject
import com.bria.common.uiframework.annotations.Layout
import com.bria.common.uiframework.coloring.Coloring
import com.bria.common.uiframework.dialogs.ScreenHolderDialog
import com.bria.common.uiframework.presenters.IPresenterEvent
import com.bria.common.uiframework.screens.AbstractScreen
import com.bria.common.uiframework.screens.IScreenEnum
import com.bria.common.util.GlobalConstants
import com.bria.common.util.Log
import com.bria.voip.R
import com.bria.voip.ui.main.im.chatroom.members.ChatRoomMembersScreen
import com.bria.voip.ui.main.misc.EScreenList
import com.bria.voip.ui.shared.pickers.ChatRoomMembersPickerScreen
import com.bria.voip.ui.shared.pickers.generic.AbstractPickerScreen

@Layout(R.layout.room_details_screen)
class ChatRoomDetailsScreen<Presenter : ChatRoomDetailsPresenter > : AbstractScreen<Presenter>() {

    @Inject(R.id.room_details_room_name)
    private lateinit var name: TextView

    @Inject(R.id.room_details_room_creation_info)
    private lateinit var creationInfo: TextView

    @Inject(R.id.room_details_room_description_value)
    private lateinit var description: TextView

    @Inject(R.id.room_details_room_topic_value)
    private lateinit var topic: TextView

    @Clickable
    @Inject(R.id.room_details_edit)
    private lateinit var edit: Button

    @Clickable
    @Inject(R.id.room_details_add_members_icon)
    private lateinit var addIcon: ImageView

    @Clickable
    @Inject(R.id.room_details_add_members_text)
    private lateinit var addText: TextView

    @Clickable
    @Inject(R.id.room_details_add_members_overlay)
    private lateinit var add: ImageView

    @Clickable
    @Inject(R.id.room_details_members_overlay)
    private lateinit var members: ImageView

    @Clickable
    @Inject(R.id.room_details_notifications_overlay)
    private lateinit var notifications: ImageView

    @Inject(R.id.room_details_notifications_value)
    private lateinit var notificationsValue: TextView

    @Clickable
    @Inject(R.id.room_details_leave_room_overlay)
    private lateinit var leave: ImageView

    @Clickable
    @Inject(R.id.room_details_leave_room_icon)
    private lateinit var leaveIcon: ImageView

    @Clickable
    @Inject(R.id.room_details_leave_room_text)
    private lateinit var leaveText: TextView


    companion object {
        private const val TAG = "ChatRoomConfScreen"

        private const val DELETE_DIALOG = 17
    }

    override fun onCreate(bundle: Bundle?) {
        super.onCreate(bundle)
        bundle?.getLong(GlobalConstants.KEY_CHAT_ROOM_ID, -1L)?.let { id ->
            if (id == -1L && presenter.chatRoom == null) {
                post({mCoordinator.flow().goUp()})
            }
        }
    }

    override fun onStart(bundle: Bundle?) {
        super.onStart(bundle)
        presenter.subscribe(this)
        updateScreen()
        bundle?.getLong(GlobalConstants.KEY_CHAT_ROOM_ID, -1L)?.let { id ->
            presenter.setChatId(id)
        }
    }

    override fun onSaveState(stateBundle: Bundle) {
        stateBundle.putLong(GlobalConstants.KEY_CHAT_ROOM_ID, presenter.getId())
        super.onSaveState(stateBundle)
    }
    override fun onStop(finishing: Boolean) {
        super.onStop(finishing)
        presenter.unsubscribe()
    }

    override fun onDestroy(finishing: Boolean) {
        super.onDestroy(finishing)
        if (finishing) destroyPresenter()
    }

    override fun onClick(v: View) {
        when (v.id) {
            R.id.room_details_edit -> { coordinator.flow(presenter.getChatRoomData()).goTo(EScreenList.CHAT_ROOM_EDIT_SCREEN) }
            R.id.room_details_add_members_overlay -> {
                val bundle = Bundle()
                bundle.putStringArrayList(ChatRoomMembersPickerScreen.FIXED_MEMBERS, ArrayList(presenter.activeMembers))
                val initiallySelectedIds = Controllers.get().buddy
                        .getListOfBuddiesSorted(Buddy.EBuddyType.XMPP, BuddyAvailability.All)
                        .filter { presenter.tentativeMembers.contains(it.imAddress) }
                        .map { it.uniqueIdentifier }
                        .toTypedArray()
                bundle.putStringArray(AbstractPickerScreen.INITIALLY_SELECTED_IDS, initiallySelectedIds)
                showScreenForResult(EScreenList.CHAT_ROOMS_MEMBERS_PICKER_SCREEN, bundle)
            }
            R.id.room_details_members_overlay  -> {
                val bundle = Bundle(1)
                bundle.putLong(ChatRoomMembersScreen.KEY_CHAT_ROOM_ID, presenter.getId())
                mCoordinator.flow().withBundle(bundle).goTo(EScreenList.CHAT_ROOM_MEMBERS_SCREEN)
            }
            R.id.room_details_notifications_overlay -> {
                coordinator.flow(presenter.getChatRoomData()).goTo(EScreenList.CHAT_ROOM_NOTIFICATION_SETTINGS_SCREEN)
            }
            R.id.room_details_leave_room_overlay -> {
                if (presenter.isUserOwner()) {
                    showDialog(DELETE_DIALOG)
                } else {
                    // no question
                    presenter.leaveRoom()
                }
            }

        }
        super.onClick(v)
    }

    override fun createDialog(which: Int, data: Bundle?): Dialog? {
        when (which) {
            DELETE_DIALOG -> {
                return AlertDialog.Builder(activity)
                    .setMessage(R.string.tChatRoomAreYouSureYouWantToDeleteThisRoom)
                    .setCancelable(false)
                    .setPositiveButton(getString(R.string.tYes)) { _, _ -> presenter.leaveRoom() }
                    .setNegativeButton(getString(R.string.tNo)) { dialog, _ -> dialog.dismiss() }
                    .create()
            }
        }

        return super.createDialog(which, data)
    }

    override fun createDialogForResult(whichScreen: IScreenEnum, data: Bundle): ScreenHolderDialog? {
        if (whichScreen === EScreenList.CHAT_ROOMS_MEMBERS_PICKER_SCREEN) {
            if (!isInTabletMode) {
                return ScreenHolderDialog
                        .builder(activity)
                        .screen(whichScreen)
                        .style(ScreenHolderDialog.DIALOG_MAX_SIZE_NO_MARGIN)
                        .bundle(data)
                        .build()
            } else {
                val color = Coloring.get().decodeColor(presenter.toolbarColor)
                return ScreenHolderDialog
                        .builder(activity)
                        .screen(whichScreen)
                        .anchorView(addText)
                        .gravity(Gravity.BOTTOM)
                        .arrowColor(color)
                        .backgroundColor(color)
                        .bundle(data)
                        .build()
            }
        }

        return super.createDialogForResult(whichScreen, data)
    }

    @MainThread
    override fun onNewMessage(message: Bundle, sender: IScreenEnum) {
        super.onNewMessage(message, sender)

        when (sender) {
            EScreenList.CHAT_ROOMS_MEMBERS_PICKER_SCREEN -> {
                val buddyList = message.getStringArrayList(ChatRoomMembersPickerScreen.CHOSEN_BUDDY_IDS)
                presenter.inviteMembers(buddyList)
            }
        }
    }

    @Suppress("UNCHECKED_CAST")
    override fun getPresenterClass(): Class<Presenter> = ChatRoomDetailsPresenter::class.java as Class<Presenter>


    override fun onPresenterEvent(event: IPresenterEvent<*, *>) {
        val type = event.type as ChatRoomDetailsPresenter.Events
        Log.d(TAG, "OnPresenterEvent ${type.name}")
        when (type) {
            ChatRoomDetailsPresenter.Events.UPDATE_SCREEN -> updateScreen()
            ChatRoomDetailsPresenter.Events.GO_UP -> coordinator.flow(event.data as Bundle).goUp()
            ChatRoomDetailsPresenter.Events.SHOW_INFO -> toastLong(event.data as String)
        }
    }

    override fun onUpNavigationClicked(v: View?) {
        presenter.cleanup()
        super.onUpNavigationClicked(v)
    }

    private fun updateScreen() {
        name.text = presenter.getName()
        description.text = presenter.getDescription()
        topic.text = presenter.getTopic()
        creationInfo.text = presenter.getCreationInfo()
        leaveText.text = if (presenter.isUserOwner()) getString(R.string.tDeleteChatRoom) else getString(R.string.tLeaveChatRoom)
        notificationsValue.text = presenter.getNotificationValue()
        if (presenter.isPublic() || (!presenter.isUserOwner())) {
            addIcon.visibility = View.GONE
            addText.visibility = View.GONE
        } else {
            addIcon.visibility = View.VISIBLE
            addText.visibility = View.VISIBLE
        }
        if (presenter.isLeft()) {
            leave.visibility = View.GONE
            leaveText.visibility = View.GONE
            leaveIcon.visibility = View.GONE
        } else {
            leave.visibility = View.VISIBLE
            leaveText.visibility = View.VISIBLE
            leaveIcon.visibility = View.VISIBLE
        }
    }

    override fun getTitle(): String? = activity.getString(R.string.tRoomSettings)
}
