package com.bria.voip.ui.main.im.chatroom.join

import android.content.Context
import android.graphics.Bitmap
import android.support.v7.content.res.AppCompatResources
import android.support.v7.widget.RecyclerView
import android.text.format.DateUtils
import android.view.View
import android.widget.ImageView
import android.widget.TextView
import com.bria.common.R
import com.bria.common.controller.im.refactoring.chatroom.ChatRoomColorBuilder
import com.bria.common.controller.im.refactoring.roomdb.LocalRoomInfo
import com.bria.common.controller.im.refactoring.roomdb.entities.ChatRoom
import com.bria.common.controller.settings.ESetting
import com.bria.common.controller.settings.core.Settings
import com.bria.common.ui.util.ExpandableAreaController
import com.bria.common.ui.util.SearchQueryColorizer
import com.bria.common.uiframework.coloring.Coloring
import com.bria.common.uiframework.lists.recycler.AbstractMultiSelectRecyclerAdapter
import com.bria.common.uiframework.lists.recycler.AbstractRecyclerAdapter
import com.bria.common.util.AndroidUtils
import com.bria.voip.ui.shared.helpers.isVisible
import com.bria.voip.ui.shared.helpers.setVisible
import java.text.SimpleDateFormat

open class JoinRoomAdapter(
        private val settings: Settings,
        recyclerView: RecyclerView,
        private val callback : JoinRoomAdapterCallback,
        private val joinRoomsDataProvider: JoinRoomsDataProvider,
        private val state: State
        ) : AbstractMultiSelectRecyclerAdapter<LocalRoomInfo, JoinRoomAdapter.ViewHolder>(
        recyclerView,
        joinRoomsDataProvider,
        R.layout.join_room_item) {

    inner class ViewHolder internal constructor(itemView: View) : AbstractRecyclerAdapter.AbstractViewHolder(itemView) {

        private var localRoomInfo: LocalRoomInfo? = null

        private val name : TextView = itemView.findViewById(R.id.join_room_item_name)
        private val created: TextView = itemView.findViewById(R.id.join_room_item_created)
        private val initials : TextView = itemView.findViewById(R.id.join_room_item_initials)
        private val selected : ImageView = itemView.findViewById(R.id.join_room_item_squared_avatar)

        private val moreArrow: View = itemView.findViewById(R.id.join_room_item_more)
        private val moreContainer: View = itemView.findViewById(R.id.join_room_item_more_container)

        private val expandableAreaController = ExpandableAreaController(moreArrow, moreContainer, ::onExpandChanged)

        private val descriptionLabel: TextView = itemView.findViewById(R.id.join_room_item_description_label)
        private val descriptionValue: TextView = itemView.findViewById(R.id.join_room_item_description_value)
        private val topicLabel: TextView = itemView.findViewById(R.id.join_room_item_topic_label)
        private val topicValue: TextView = itemView.findViewById(R.id.join_room_item_topic_value)


        private fun getTimeString(time : Long, context : Context) : String {
            val sdf = SimpleDateFormat("HH:mm")
            val dateString = DateUtils.getRelativeTimeSpanString(time, System.currentTimeMillis(), DateUtils.DAY_IN_MILLIS, DateUtils.FORMAT_SHOW_DATE) as String
            //val timeString = sdf.format(Date(time))

            return context.getString(R.string.tChatRoomCreatedOnTime, dateString)
        }

        fun bindToModel(roomInfo: LocalRoomInfo) {
            this.localRoomInfo = roomInfo

            name.text = roomInfo.name
            searchQueryColorizer.colorize(name, joinRoomsDataProvider.searchQuery)
            if (roomInfo.creationTime > 0) {
                created.text = getTimeString(roomInfo.creationTime, created.context)
            } else {
                created.text = ""
            }

            if (isSelected(position)) {
                selected.setImageBitmap(mCheckMark)
                selected.visibility = View.VISIBLE
                initials.visibility = View.GONE

            } else {
                selected.visibility = View.GONE
                initials.visibility = View.VISIBLE
            }

            initials.text = ChatRoom.getInitials(roomInfo.name)
            initials.setBackgroundColor(ChatRoomColorBuilder.getColorFromString(roomInfo.name))
            if (!roomInfo.hasInfo) {
                callback.getRoomInfoFor(roomInfo.accId, roomInfo.cleanedJid)
            }


            expandableAreaController.setExpanded(state.expandedItems.contains(roomInfo))

            descriptionValue.text = roomInfo.description
            // show only if there is something
            descriptionValue.setVisible(descriptionValue.text.isNotBlank())
            descriptionLabel.setVisible(descriptionValue.isVisible)

            topicValue.text = roomInfo.topic
            // show only if there is something
            topicValue.setVisible(topicValue.text.isNotBlank())
            topicLabel.setVisible(topicValue.isVisible)

            // show arrow only if there is something to expand
            moreArrow.setVisible(descriptionValue.isVisible || topicValue.isVisible)
        }

        private fun onExpandChanged(isExpanded: Boolean) {
            if (localRoomInfo == null) {
                return
            }

            if (isExpanded) {
                state.expandedItems.add(localRoomInfo!!)
            } else {
                state.expandedItems.remove(localRoomInfo!!)
            }
        }
    }

    private val mCheckMark = createSquaredCheckMark(mContext.get()?.getResources()?.getDimensionPixelSize(R.dimen.list_item_size_half) ?: 0)

    private var searchQueryColorizer = SearchQueryColorizer(Coloring.get().decodeColor(settings.getStr(ESetting.ColorBrandTint)))


    override fun provideNewViewHolder(inflatedView: View, viewType: Int): ViewHolder {
        return ViewHolder(inflatedView)
    }

    override fun onNewViewHolderReady(viewHolder: ViewHolder?, viewType: Int) {}

    override fun onUpdateNeeded(viewHolder: ViewHolder, position: Int) {
        val roomInfo = joinRoomsDataProvider.getItemAt(position)

        viewHolder.bindToModel(roomInfo)
    }

    private fun createSquaredCheckMark(size: Int): Bitmap {
        val brandTint = Coloring.get().decodeColor(settings.getStr(ESetting.ColorBrandTint))
        var checkMark = AppCompatResources.getDrawable(mContext.get()!!, R.drawable.ic_chatroom_select)
        checkMark = Coloring.get().colorDrawable(checkMark, brandTint)

        return AndroidUtils.drawableToBitmap(checkMark, size, size)
    }

    interface JoinRoomAdapterCallback {
        fun getRoomInfoFor(accID : String, chatKey : String)
    }

    class State {
        val expandedItems: MutableSet<LocalRoomInfo> = mutableSetOf()
    }
}
