package com.bria.voip.ui.main.im.chatroom.join

import com.bria.common.controller.Controllers
import com.bria.common.controller.accounts.AccountsFilter
import com.bria.common.controller.im.refactoring.roomdb.LocalRoomInfo
import com.bria.common.controller.settings.core.Settings
import com.bria.common.uiframework.presenters.AbstractPresenter
import com.bria.common.uiframework.presenters.IPresenterEventTypeEnum
import com.bria.common.uireusable.dataprovider.IFilterableDataProvider
import com.bria.common.util.Log
import com.bria.voip.R
import io.reactivex.disposables.CompositeDisposable
import io.reactivex.schedulers.Schedulers
import java.util.concurrent.TimeUnit

class JoinRoomPresenter : AbstractPresenter() {

    companion object {
        private const val TAG = "JoinRoomPresenter"
    }

    // dependencies

    val settings
        get() = Settings.get(context)

    //


    private var disposable = CompositeDisposable()

    var title: String = getString(R.string.tJoinRooms)



    val dataProvider = JoinRoomsDataProvider(Controllers.get().im.imData.chatRoomRepo)

    var waitingForRoomCreation = ""

    val waitingForJoin = mutableListOf<String>()

    var adapterState = JoinRoomAdapter.State()
        private set


    enum class Events : IPresenterEventTypeEnum {
        DATA_LOADED,
        DATA_LOADING,
        ERROR_JOINING_THE_ROOM,
        GO_UP,
        GO_TO;

        override fun getType(): IPresenterEventTypeEnum = this
    }

    private val dataProviderListener = object : IFilterableDataProvider.DataProviderListener {
        override fun onDataLoaded(isEmpty: Boolean) {
            firePresenterEvent(Events.DATA_LOADED, isEmpty)
        }

        override fun onDataLoading() {
            //firePresenterEvent(Events.DATA_LOADING)
        }
    }

    init {
        disposable.add(Controllers.get().im.imData.chatRoomRepo.getOnServerRoomListChangedObservable()
                .sample(2, TimeUnit.SECONDS, Schedulers.computation())
                .subscribe({
                    dataProvider.refilter()
                }, {}))

        disposable.add(Controllers.get().im.chatApi.chatRoomApi.onErrorEventObservable.subscribe({
            if (waitingForJoin.contains(it.chatKey)) {
                waitingForJoin.clear()
                firePresenterEvent(Events.ERROR_JOINING_THE_ROOM, it.chatKey)

            }
        }, {

        }))

        dataProvider.attachWeakListener(dataProviderListener)
    }


    fun initialize() {
        filterBy("")
        adapterState = JoinRoomAdapter.State()
    }

    fun updateList() {
        Controllers.get().im.chatApi.updateRoomList(false)
    }

    var activeFilter = ""

    fun filterBy(searchString: String?) {
        dataProvider.filter(searchString)
        activeFilter = searchString ?: ""
    }


    fun joinSelectedRooms(selectedRooms: List<String>) {

        Controllers.get().accounts.getAccounts(AccountsFilter.XMPP).firstOrNull()?.let {
            Controllers.get().im.chatApi.getXmppAccount(it)?.let { xmppAccount ->
                val selectedInfos = mutableListOf<LocalRoomInfo>()
                selectedRooms.forEach { selectedRoom ->
                    dataProvider.getByRoomKey(selectedRoom)?.let { selectedInfos.add(it) }
                }
                waitingForJoin.addAll(selectedInfos.map { it.cleanedJid })
                Controllers.get().im.chatApi.chatRoomApi.joinRooms(selectedInfos, xmppAccount)
                //if one go to chat room session else go up
                if (selectedRooms.size > 1) {
                    waitingForJoin.clear()
                    firePresenterEvent(Events.GO_UP)
                } else {
                    disposable.add(Controllers.get().im.imData.chatRoomRepo.getOnRoomAddedObservable().firstOrError().subscribe({
                        Log.d(TAG, "OnRoomJoined: ${it[0].chatKey} name: ${it[0].name}")
                        if (it.isNotEmpty()) {
                            waitingForJoin.clear()
                            firePresenterEvent(Events.GO_TO, it[0].id)
                        } else {
                            waitingForJoin.clear()
                            firePresenterEvent(Events.GO_UP)
                        }
                    }, {
                        waitingForJoin.clear()
                        firePresenterEvent(Events.GO_UP)
                    }))
                }
            }
        }

    }

    fun getRoomInfoFor(accId: String, chatKey: String) {
        Controllers.get().im.chatApi.chatRoomApi.getRoomInfo(accId, chatKey)
    }


}
