package com.bria.voip.ui.main.im.chatroom.members

import com.bria.common.controller.contacts.ContactsSortBy
import com.bria.common.controller.contacts.buddy.*
import com.bria.common.uireusable.dataprovider.AbstractFilterableListDataProvider
import com.bria.common.uireusable.dataprovider.FilterQueryMatcher
import com.bria.common.uireusable.dataprovider.IDataChangeListener
import com.bria.common.util.IObservable
import com.bria.common.util.SyncObservableDelegate

class ChatRoomMembersDataProvider(
        private val buddyController: IBuddyCtrlEvents,
        private val contactsSortBy: ContactsSortBy,
        buddyObservable: IObservable<IBuddyCtrlObserver>
) : AbstractFilterableListDataProvider<XmppBuddy>() {

    private var buddies = listOf<XmppBuddy>()
    private val mObservers = SyncObservableDelegate<IDataChangeListener>()

    private var mBuddyCtrlObserver: IBuddyCtrlObserver = object : IBuddyCtrlObserver {
        override fun onBuddyListUpdated() {
            mObservers.notifyObservers { it.dataChanged() }
        }

        override fun onBuddyPresenceChanged(buddy: Buddy) {
            val indexOfFirst = mDisplayedData.indexOfFirst { it.buddy == buddy }
            val isDisplayed = indexOfFirst >= 0
            if (isDisplayed) {
                mObservers.notifyObservers { it.itemChanged(indexOfFirst) }
            }
        }

        override fun onBuddyFilterChanged(aBuddyFilterType: IBuddyCtrlEvents.EBuddyFilterType?) {}
    }

    init {
        buddyObservable.attachWeakObserver(mBuddyCtrlObserver)
    }

    fun setBuddies(buddies: List<XmppBuddy>) {
        this.buddies = buddies

        refilter()
    }

    override fun provideFullDataSet(): List<XmppBuddy> {
        return buddies
            .sortedWith(BuddyComparator(contactsSortBy))
    }

    override fun provideFilteredDataSet(fullDataSet: List<XmppBuddy>, query: String): List<XmppBuddy> {
        val filterQueryMatcher = FilterQueryMatcher(query)

        return fullDataSet
                .filter { filterQueryMatcher.isMatch(it.firstName, it.lastName) }
                .toMutableList()
    }

    val observable : IObservable<IDataChangeListener>
        get() = mObservers
}
