package com.bria.voip.ui.main.im.chatroom.members

import android.app.Dialog
import android.os.Bundle
import android.os.Parcelable
import android.support.annotation.MainThread
import android.support.design.widget.FloatingActionButton
import android.support.v7.app.AlertDialog
import android.support.v7.widget.LinearLayoutManager
import android.support.v7.widget.RecyclerView
import android.view.MenuItem
import android.view.View
import android.widget.ProgressBar
import android.widget.TextView
import com.bria.common.controller.contacts.buddy.XmppBuddy
import com.bria.common.controller.im.ImpsUtils
import com.bria.common.controller.settings.ESetting
import com.bria.common.ui.CreateDialogThreadCheck
import com.bria.common.uiframework.annotations.Clickable
import com.bria.common.uiframework.annotations.Inject
import com.bria.common.uiframework.annotations.Layout
import com.bria.common.uiframework.dialogs.ScreenHolderDialog
import com.bria.common.uiframework.presenters.IPresenterEvent
import com.bria.common.uiframework.screens.AbstractScreen
import com.bria.common.uiframework.screens.IScreenEnum
import com.bria.common.uireusable.CustomSearchView
import com.bria.common.util.Log
import com.bria.voip.R
import com.bria.voip.ui.main.contacts.BuddyDisplayScreenPresenter
import com.bria.voip.ui.main.contacts.ChooseNumberOfXmppBuddyScreen
import com.bria.voip.ui.main.contacts.PhoneNumberActionsScreen
import com.bria.voip.ui.main.misc.EScreenList
import com.bria.voip.ui.shared.helpers.setVisible

@Layout(R.layout.chat_room_members_screen)
@com.bria.common.uiframework.annotations.Menu(R.menu.chat_room_members_menu)
class ChatRoomMembersScreen : AbstractScreen<ChatRoomMembersPresenter>() {

    companion object {
        private const val TAG = "ChatRoomMembersScreen"

        private const val KEY_LINEAR_LAYOUT_STATE = "LINEAR_LAYOUT_STATE"
        private const val SEARCH_STATE = "SEARCH_STATE"

        const val KEY_CHAT_ROOM_ID = "KEY_CHAT_ROOM_ID"
    }

    private val PHONE_ITEM_ACTIONS_DIALOG = 0xD00005


    @Inject(id = com.bria.voip.R.id.chat_room_members_screen_search_view, col = false)
    private lateinit var mSearchView: CustomSearchView

    @Inject(id = R.id.chat_room_members_progress_bar, fore = ESetting.ColorBrandTint)
    private lateinit var mProgressBar: ProgressBar

    @Inject(R.id.chat_room_members_screen_recycler_view)
    private lateinit var mRecyclerView: RecyclerView

    @Inject(R.id.chat_room_members_screen_info)
    private lateinit var mInfo: TextView

    @Clickable
    @Inject(id = R.id.chat_room_members_screen_add_member, back = ESetting.ColorBrandDefault)
    private lateinit var mAddMember: FloatingActionButton

    private lateinit var mLinearLayoutManager: LinearLayoutManager
    private lateinit var mAdapter: ChatRoomMembersAdapter


    override fun getPresenterClass(): Class<ChatRoomMembersPresenter> = ChatRoomMembersPresenter::class.java

    override fun getTitle(): String? = getString(R.string.tChatRoomMembers)


    override fun onCreate(bundle: Bundle?) {
        super.onCreate(bundle)

        presenter.subscribe(this)

        parseBundle(bundle)

        mLinearLayoutManager = LinearLayoutManager(activity)
        mRecyclerView.setLayoutManager(mLinearLayoutManager)

        mAdapter = ChatRoomMembersAdapter(
                presenter.settings,
                mRecyclerView,
                QueriesHandlerImpl(),
                ActionsHandlerImpl(),
                presenter.dataProvider,
                presenter.adapterState)
        mRecyclerView.adapter = mAdapter

        if (bundle != null) {
            val parcelable = bundle.getParcelable<Parcelable>(KEY_LINEAR_LAYOUT_STATE)
            if (parcelable != null) {
                mLinearLayoutManager.onRestoreInstanceState(parcelable)
            }

            mSearchView.restoreState(bundle.getBundle(SEARCH_STATE))
        }

        mSearchView.setEventHandler(object : CustomSearchView.EventHandler {
            override fun searchFor(searchString: String?) {
                presenter.filterBy(searchString)
            }

            override fun onClosed() {}
            override fun onOpened() {}
            override fun onClear() {}
        })

        mSearchView.setDelayedSearchInterval(0)
    }

    override fun onNewConfig(bundle: Bundle?) {
        super.onNewConfig(bundle)

        parseBundle(bundle)
    }

    private fun parseBundle(bundle: Bundle?) {
        if (bundle != null) {
            // after rotate we will not get chat room id and need to rely on data in presenter
            if (bundle.containsKey(KEY_CHAT_ROOM_ID)) {
                presenter.initForRoom(bundle.getLong(KEY_CHAT_ROOM_ID))
            }
        }
    }

    override fun onDestroy(finishing: Boolean) {
        presenter.unsubscribe()

        super.onDestroy(finishing)
    }

    override fun onStart(bundle: Bundle?) {
        super.onStart(bundle)
    }

    override fun onStop(finishing: Boolean) {
        if (finishing) {
            mSearchView.close()
            mSearchView.setEventHandler(null)
        }

        super.onStop(finishing)
    }



    override fun onSaveState(stateBundle: Bundle) {
        stateBundle.putParcelable(KEY_LINEAR_LAYOUT_STATE, mLinearLayoutManager.onSaveInstanceState())
        stateBundle.putBundle(SEARCH_STATE, mSearchView.saveState())

        super.onSaveState(stateBundle)
    }

    override fun onResume() {
        super.onResume()

    }

    override fun onPause(finishing: Boolean) {
        super.onPause(finishing)
    }

    override fun onNewMessage(message: Bundle, sender: IScreenEnum) {
        when (sender) {
            EScreenList.CHOOSE_NUMBER_OF_XMPP_BUDDY -> {
                val xmppChatParticipantKey = message.getString(ChooseNumberOfXmppBuddyScreen.RESULT_BUDDY_KEY)
                val number = message.getString(ChooseNumberOfXmppBuddyScreen.RESULT_SELECTED_NUMBER)

                presenter.callXmppNumber(xmppChatParticipantKey, number)
            }
        }

        super.onNewMessage(message, sender)
    }

    override fun createDialogForResult(whichScreen: IScreenEnum, data: Bundle): ScreenHolderDialog? {
        when (whichScreen) {
            EScreenList.CHOOSE_NUMBER_OF_XMPP_BUDDY -> {
                return ScreenHolderDialog
                    .builder(activity)
                    .screen(EScreenList.CHOOSE_NUMBER_OF_XMPP_BUDDY)
                    .style(ScreenHolderDialog.DIALOG_MIN_SIZE)
                    .showTitle(false)
                    .bundle(data)
                    .build()
            }
        }

        return super.createDialogForResult(whichScreen, data)
    }

    @MainThread
    override fun createDialog(which: Int, data: Bundle?): Dialog? {
        when (which) {
            PHONE_ITEM_ACTIONS_DIALOG -> {
                return ScreenHolderDialog
                    .builder(activity)
                    .screen(EScreenList.PHONE_ACTION_SELECT)
                    .bottomSheet()
                    .bundle(data)
                    .build()
            }
            else -> {
                return super.createDialog(which, data)
            }
        }
    }

    override fun onMenuItemClick(menuItem: MenuItem?): Boolean {
        when (menuItem?.itemId) {

            R.id.chat_room_members_mi_search -> {
                mSearchView.open(true)
                return true
            }
        }
        return super.onMenuItemClick(menuItem)

    }

    override fun onClick(v: View) {
        when (v.id) {
            R.id.chat_room_members_screen_add_member -> {

                CreateDialogThreadCheck.check()

                AlertDialog.Builder(activity)
                    .setMessage("What to do, what to do...")
                    .setNeutralButton("OK", null)
                    .create()
                    .show()
            }
        }

        super.onClick(v)
    }

    override fun onPresenterEvent(event: IPresenterEvent<*, *>) {
        val type = event.type as ChatRoomMembersPresenter.Events
        Log.d(TAG, "OnPresenterEvent ${type.name}")

        when (type) {
            ChatRoomMembersPresenter.Events.DATA_LOADING -> {
                mProgressBar.setVisible(true)
                mRecyclerView.setVisible(false)
                mInfo.setVisible(false)
            }
            ChatRoomMembersPresenter.Events.DATA_LOADED -> {
                mProgressBar.setVisible(false)

                if (event.data != null) {
                    val noData = event.data as Boolean
                    if (noData) {
                        mRecyclerView.setVisible(false)
                        mInfo.setVisible(true)

                        if (mSearchView.isOpen) {
                            mInfo.text = getString(R.string.tNoResults)
                        } else {
                            mInfo.text = getString(R.string.tChatRoomNoMembers)
                        }
                    } else {
                        mRecyclerView.setVisible(true)
                        mInfo.setVisible(false)

                        mAdapter.notifyDataChanged()
                    }
                }
            }
            ChatRoomMembersPresenter.Events.GO_UP -> mCoordinator.flow().goUp()
            ChatRoomMembersPresenter.Events.SHOW_NUMBER_CHOOSER_FOR_CALL -> {
                val xmppBuddy = event.data as XmppBuddy
                val bundle = Bundle().apply {
                    putString(ChooseNumberOfXmppBuddyScreen.IN_BUDDY_KEY, xmppBuddy.uniqueIdentifier)
                }

                showScreenForResult(EScreenList.CHOOSE_NUMBER_OF_XMPP_BUDDY, bundle)
            }
            ChatRoomMembersPresenter.Events.GO_TO_IM_CONVERSATION ->  mCoordinator.flow (event.data as android.os.Bundle).goTo(EScreenList.CONVERSATION)
            ChatRoomMembersPresenter.Events.SHOW_MESSAGE_LONG ->  toastLong(event.data as String)
            ChatRoomMembersPresenter.Events.SHOW_PHONE_ITEM_ACTIONS_DIALOG -> showPhoneItemActionsDialog(event)
            ChatRoomMembersPresenter.Events.SHOW_SNACK -> showSnack(event.data as String)
        }
    }

    private fun showPhoneItemActionsDialog(event: IPresenterEvent<*, *>) {
        val phoneActionsDialogParams = event.data as ChatRoomMembersPresenter.PhoneActionsDialogParams
        val bundle = Bundle()
        bundle.putString(PhoneNumberActionsScreen.KEY_CONTACT_NAME, phoneActionsDialogParams.contactName)
        bundle.putParcelable(PhoneNumberActionsScreen.KEY_PHONE_NUMBER, phoneActionsDialogParams.phoneNumber)
        showDialog(PHONE_ITEM_ACTIONS_DIALOG, bundle)
    }

    inner class QueriesHandlerImpl : ChatRoomMembersAdapter.QueriesHandler {
        override fun amIOwner(): Boolean {
            TODO()
        }
    }

    inner class ActionsHandlerImpl : ChatRoomMembersAdapter.ActionsHandler {
        override fun onBuddyClicked(buddy: XmppBuddy) {
            val bundle = Bundle()
            bundle.putString(BuddyDisplayScreenPresenter.KEY_BUDDY_ID, ImpsUtils.getBuddyKey(buddy))

            mCoordinator.flow().withBundle(bundle).goTo(EScreenList.BUDDY_DISPLAY)
        }

        override fun onCallClicked(buddy: XmppBuddy) {
           presenter.call(buddy)
        }

        override fun onMessageClicked(buddy: XmppBuddy) {
            presenter.sendMessageTo(buddy)
        }

        override fun onRemoveClicked(buddy: XmppBuddy) {
            presenter.removeFromChatRoom(buddy)
        }

        override fun onResendInvitationClicked(buddy: XmppBuddy) {
            presenter.resendInvitation(buddy)
        }
    }
}

