package com.bria.voip.ui.main.im.chatroom.mentions

import android.text.SpannableString
import android.text.Spanned
import android.text.TextUtils
import android.widget.MultiAutoCompleteTextView
import com.bria.common.controller.im.refactoring.chatroom.MentionsFinder

class ChatRoomMentionsTokenizer : MultiAutoCompleteTextView.Tokenizer {

    /**
     * Returns the start of the token that ends at offset
     * <code>cursor</code> within <code>text</code>.
     */
    override fun findTokenStart(text: CharSequence, cursor: Int): Int {
        var i = cursor

        // find @ to the left
        val indexOfLast = text.subSequence(0, cursor).indexOfLast { it == '@' }
        if(indexOfLast == -1){
            return cursor
        }

        if (indexOfLast == 0) {
            // @ is at the beginning
            return indexOfLast
        }

        val c = text[indexOfLast - 1]
        if (MentionsFinder.isMentionDelimiter(c)) {
            // whitespace preceding @ is ok
            return indexOfLast
        }

        //other chars not
        return cursor
    }

    /**
     * Returns the end of the token (index after last char)
     * that begins at offset <code>cursor</code> within <code>text</code>.
     */
    override fun findTokenEnd(text: CharSequence, cursor: Int): Int {
        var i = cursor
        val len = text.length

        while (i < len) {
            val c = text[i]
            if (MentionsFinder.isMentionDelimiter(c)) {
                return i
            } else {
                i++
            }
        }

        return len
    }

    /**
     * Returns <code>text</code>, modified, if necessary, to ensure that
     * it ends with a token terminator (for example a space or comma).
     */
    override fun terminateToken(text: CharSequence): CharSequence {
        if (text is Spanned) {
            val sp = SpannableString(text.toString() + " ")
            TextUtils.copySpansFrom(text, 0, text.length, Any::class.java, sp, 0)
            return sp
        } else {
            return text.toString() + " "
        }
    }
}
