package com.bria.voip.ui.main.im.chatroom.notifications

import android.os.Bundle
import com.bria.common.controller.Controllers
import com.bria.common.controller.im.refactoring.roomdb.entities.ChatRoom
import com.bria.common.controller.im.refactoring.roomdb.entities.ChatRoomNotificationLevel
import com.bria.common.uiframework.presenters.AbstractPresenter
import com.bria.common.uiframework.presenters.IPresenterEventTypeEnum
import com.bria.common.util.GlobalConstants
import com.bria.voip.R
import com.bria.voip.ui.main.im.chatroom.details.ChatRoomDetailsPresenter
import com.bria.voip.ui.main.im.chatroom.edit.ChatRoomEditScreen

class ChatRoomNotificationsSettingsPresenter : AbstractPresenter() {

    enum class Events : IPresenterEventTypeEnum {
        UPDATE_SCREEN,
        GO_UP;
        override fun getType(): IPresenterEventTypeEnum = this
    }

    private var chatRoom: ChatRoom? = null

    fun setChatId(id: Long) {
        offloadLightWork {
            Controllers.get().im.imData.chatRoomRepo.getChatRoom(id)?.let {
                chatRoom = it
                if (activeLevel == null) activeLevel = it.notificationsLevel
                firePresenterEvent(Events.UPDATE_SCREEN)
            }

        }
    }

    fun getId(): Long {
        return chatRoom?.id ?: -1
    }

    fun actionSave() {
        offloadLightWork {
            Controllers.get().im.imData.chatRoomRepo.getChatRoom(getId())?.let {
                it.notificationsLevel = activeLevel ?: ChatRoomNotificationLevel.ALL
                Controllers.get().im.imData.chatRoomRepo.updateChatRoom(it)
                firePresenterEvent(Events.GO_UP, getChatRoomData())
            }
        }
    }

    fun getChatRoomData(): Bundle {
        val bundle = Bundle(1)
        bundle.putLong(GlobalConstants.KEY_CHAT_ROOM_ID, chatRoom?.id ?: 0)
        return bundle
    }

    fun setMute(checked: Boolean) {
        if (checked) {
            if (activeLevel != ChatRoomNotificationLevel.MUTE) {
                previousLevel = activeLevel
            }
            activeLevel = ChatRoomNotificationLevel.MUTE
        } else {
            activeLevel = previousLevel
        }
        firePresenterEvent(Events.UPDATE_SCREEN)
    }

    var activeLevel: ChatRoomNotificationLevel? = null
    set(value) {
        if (activeLevel != ChatRoomNotificationLevel.MUTE) {
            previousLevel = field
        }
        field = value
        firePresenterEvent(Events.UPDATE_SCREEN)
    }
    var previousLevel : ChatRoomNotificationLevel? = ChatRoomNotificationLevel.ALL

}
