package com.bria.voip.ui.shared.pickers

import android.support.annotation.MainThread
import com.bria.common.controller.buddy.BuddyAvailability
import com.bria.common.controller.contacts.buddy.Buddy
import com.bria.common.controller.contacts.buddy.IBuddyCtrlEvents
import com.bria.common.controller.contacts.buddy.IBuddyCtrlObserver
import com.bria.common.modules.BriaGraph
import com.bria.common.network.INetworkObserver
import com.bria.common.network.NetworkModule
import com.bria.common.uiframework.presenters.IPresenterEventTypeEnum
import com.bria.common.uireusable.dataprovider.IChipsViewDataProvider
import com.bria.common.uireusable.dataprovider.IFilterableDataProvider
import com.bria.voip.ui.shared.pickers.generic.AbstractAdvancedPickerPresenter

class ChatRoomMembersPickerPresenter : AbstractAdvancedPickerPresenter<Buddy>() {
    enum class ChatRoomMembersPickerPresenterEvents : IPresenterEventTypeEnum {
        NETWORK_CONNECTION_LOST,
        PRESENCE_CHANGED;

        override fun getType(): IPresenterEventTypeEnum {
            return this
        }
    }

    private val mNetworkModule: NetworkModule = BriaGraph.networkModule

    private val mBuddyCtrlObserver = BuddyCtrlObserver()
    private val mNetworkObserver = NetworkObserver()

    private val mChatRoomMembersPickerChipsProvider by lazy {
        ChatRoomMembersPickerChipsProvider(BriaGraph.buddyCtrl)
    }

    @MainThread
    override fun onSubscribe() {
        super.onSubscribe()

        mObservables.buddy.attachWeakObserver(mBuddyCtrlObserver)
        mNetworkModule.attachWeakObserver(mNetworkObserver)
    }

    @MainThread
    override fun onUnsubscribe() {
        super.onUnsubscribe()

        mObservables.buddy.detachObserver(mBuddyCtrlObserver)
        mNetworkModule.detachObserver(mNetworkObserver)
    }

    override fun createDataProvider(): IFilterableDataProvider<Buddy>? {
        return ChatRoomMembersPickerProvider()
    }

    override fun getChipsDataProvider(): IChipsViewDataProvider<Buddy> {
        return mChatRoomMembersPickerChipsProvider
    }


    override fun processDataInBackground(data: MutableList<Buddy>): Any {
        return data
    }

    private val specificDataProvider: ChatRoomMembersPickerProvider
        get() = dataProvider as ChatRoomMembersPickerProvider

    fun setOnlineStatusFilter(onlineStatus: BuddyAvailability) {
        specificDataProvider.onlineStatus = onlineStatus
    }

    fun setMembers(fixedMembers: Set<String>) {
        specificDataProvider.setAlreadyPresentBuddyIds(fixedMembers)
    }

    fun isNetworkAvailable(): Boolean = mNetworkModule.connectionType != INetworkObserver.ENetworkType.NONE


    inner class NetworkObserver : INetworkObserver {
        override fun onNetworkConnected(conType: INetworkObserver.ENetworkType?, cellType: INetworkObserver.EMobileType?) {
            refilter()
        }

        override fun onNetworkDisconnected() {
            refilter()
        }

        override fun onPhoneStateChanged(status: INetworkObserver.EPhoneMonitoring?) {}
    }

    inner class BuddyCtrlObserver : IBuddyCtrlObserver {

        override fun onBuddyListUpdated() {
            refilter();
        }

        override fun onBuddyPresenceChanged(buddy: Buddy) {
            if (specificDataProvider.onlineStatus == IBuddyCtrlEvents.EBuddyFilterType.ONLINE) {
                refilter()
            } else {
                for (i in 0 until dataProvider.itemsCount) {
                    val localBuddy = dataProvider.getItemAt(i)
                    if (buddy.uniqueIdentifier == localBuddy.uniqueIdentifier) {
                        firePresenterEvent(ChatRoomMembersPickerPresenterEvents.PRESENCE_CHANGED, i.toLong())
                    }
                }
            }
        }

        override fun onBuddyFilterChanged(aBuddyFilterType: IBuddyCtrlEvents.EBuddyFilterType?) {}
    }
}
